@@ -10,7 +10,7 @@
 
 #include "mew.h"
 
-private char version_message[] = "version 0.20 20010710 Kazu Yamamoto";
+private char version_message[] = "version 0.21 20010908 Kazu Yamamoto";
 
 #ifdef HAVE_UNISTD_H
 # include <sys/types.h>
@@ -161,7 +161,7 @@ help_message[] = {
 	" -f  <fields>  Fields to display.",
 	" -h            Display this help message.",
 	" -i  <file>    A file name to be read.",
-	" -l  <length>  Field max length to dump.",
+	" -l  <length>  Field max length to dump. (0 means no limit)",
 	" -n            Don't use fstat().",
 	" -p  <pattern> Pick pattern.",
 	" -s  <src>     Message source: '+folder range'.",
@@ -594,7 +594,7 @@ print_for_sort(FILE *fp, char *fname, char *fld) {
 	if (beg == NULL)
 		printf("%s: \n", fname);
 	else {
-		printf("%s", fname);
+		printf("%s:", fname);
 		p = beg;
 		while (p < end)
 			putchar(*p++);
