@@ -32,7 +32,7 @@ enum ptype {
 	P_OR,  /* |, || */
 	P_NOT, /* ! */
 	P_LSQ, /* ( */
-	P_RSQ, /* ) */
+	P_RSQ  /* ) */
 };
 
 struct pat {
