@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 22, 1999
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -113,41 +112,53 @@
 ;;; Due to the spec of mew-complete, alist must be passed.
 ;;;
 
-(defsubst mew-alias-get (key)
-  (mew-addrbook-alias-get key mew-addrbook-alist))
-
-(defsubst mew-alias-next (key)
-  (mew-addrbook-alias-next key mew-addrbook-alist))
+(defvar mew-alias-expand-prefix nil
+  "A variable to make 'mew-alias-expand' elegant.")
 
+;; for completion
 (defun mew-addrbook-alias-get (key alist)
-  (let ((addrs (mew-addrbook-alias-get1 key alist 0)))
-    (cond
-     ((stringp addrs) addrs)
-     ((listp addrs)
-      (mapconcat (function identity) (nreverse addrs) ", "))
-     (t key))))
+  (let* ((mew-alias-expand-prefix nil)
+	 (addrs (mew-alias-expand key alist 0))
+	 (addrx (mapcar (function mew-addrstr-append-domain) addrs))
+	 (ret (mapconcat (function identity) addrx ", ")))
+    (if mew-alias-expand-prefix
+	(concat mew-alias-expand-prefix ":" ret ";")
+      ret)))
 
-(defun mew-addrbook-alias-get1 (key alist count)
-  "Expand KEY to addresses according ALIST.
-If addresses is a list, that follows one-of convention and
-return the first member of the list.
-If addresses is a string, expands it recursively."
-  (let* ((crnt (mew-adbkent-addrs (mew-addrbook-alias-hit key alist)))
-	 (keys (and (stringp crnt)
-		    (mapcar (function mew-chop) (mew-split crnt ?,))))
-	 ret tmp)
-    (cond
-     ((> count mew-expand-max-depth) key)
-     ((null crnt) key)
-     ((listp crnt) (car crnt))
-     (t
-      (while keys
-	(setq tmp (mew-addrbook-alias-get1 (car keys) alist (1+ count)))
-	(if (listp tmp)
-	    (setq ret (nconc tmp ret))
-	  (setq ret (cons tmp ret)))
-	(setq keys (cdr keys)))
-      ret))))
+(defun mew-alias-expand-addrs (key alist count)
+  (let ((keys (mew-split key ?,))
+	ret)
+    (setq keys (mapcar (function mew-chop) keys))
+    (setq keys (delete "" keys))
+    (while keys
+      (setq ret (nconc ret (mew-alias-expand (car keys) alist count)))
+      (setq keys (cdr keys)))
+    ret))
+
+(defun mew-alias-expand (key alist count)
+  "Expand KEY to a list of addresses according to ALIST.
+Infinite loop is prevented by COUNT and 'mew-expand-max-depth'.
+Before calling, 'mew-alias-expand-prefix' must be set 'nil'.
+If \"prefix:a,b,c;\" is given, 'mew-alias-expand-prefix'
+is set to \"prefix\", and (expanded-a expanded-b expanded-c) is
+returned."
+  (cond
+   ((> count mew-expand-max-depth) key)
+   ((string-match "^\\([^:]+\\):\\([^;]+\\);$" key)
+    (if mew-alias-expand-prefix (error ":; must not recurse!"))
+    (setq mew-alias-expand-prefix (mew-match 1 key))
+    (setq key (mew-match 2 key))
+    (mew-alias-expand-addrs key alist (1+ count)))
+   (t
+    (let ((crnt (mew-adbkent-addrs (mew-addrbook-alias-hit key alist))))
+      (cond
+       ((null crnt) (list key))
+       ((listp crnt) (list (car crnt)))
+       ((string-match "[^:]+:[^;]+;" crnt)
+	(mew-alias-expand crnt alist (1+ count)))
+       ((string-match "," crnt)
+	(mew-alias-expand-addrs crnt alist (1+ count)))
+       (t (list crnt)))))))
 
 (defun mew-addrbook-alias-next (key alist)
   (let* ((addrs (mew-adbkent-addrs
@@ -198,7 +209,7 @@ If addresses is a string, expands it recursively."
   ;; But mew-alist-auto-alist is the newest-first order.
   ;; So, just cons user-addr to mew-addrbook-alist.
   ;; This produces a different alist made by mew-addrbook-setup.
-  ;; But we don't care.
+  ;; But we do not care.
   (setq mew-addrbook-alist (cons user-addr mew-addrbook-alist)))
 
 (defun mew-addrbook-alias-delete (addr)
@@ -328,8 +339,8 @@ executed with '\\[universal-argument]', it will add personal information.  Other
 it will add an alias."
   (interactive "P")
   (mew-summary-display nil)
-  (let ((fld (mew-current-get-fld (mew-frame-id)))
-	(msg (mew-current-get-msg (mew-frame-id)))
+  (let ((fld (mew-summary-folder-name))
+	(msg (mew-summary-message-number2))
 	(buf (mew-buffer-message))
 	from shortname addrs name)
     (save-excursion
@@ -427,19 +438,19 @@ The keys that are defined for this mode are:
      ((string= mode mew-addrbook-mode-alias)
       (cond
        ((and (null shortname) (null addrs))
-	(setq errmsg "Must fill both Shortname and Addresses."))
+	(setq errmsg "Must fill both Shortname and Addresses"))
        ((null shortname)
-	(setq errmsg "Must fill Shortname."))
+	(setq errmsg "Must fill Shortname"))
        ((null addrs)
-	(setq errmsg "Must fill Addresses."))))
+	(setq errmsg "Must fill Addresses"))))
      (t
       (cond
        ((null addrs)
-	(setq errmsg "Must fill Addresses."))
+	(setq errmsg "Must fill Addresses"))
        ((and (null shortname) (null nickname) (null name))
-	(setq errmsg "Must fill Shortname or Nickname or Name."))
+	(setq errmsg "Must fill Shortname or Nickname or Name"))
        ((and name (string-match "^\"[^\"]*[^\000-\177]" name))
-	(setq errmsg "Remove quote around non-ASCII Name.")))))
+	(setq errmsg "Remove quote around non-ASCII Name")))))
     (if errmsg
 	(message errmsg)
       (save-excursion
@@ -482,15 +493,15 @@ The keys that are defined for this mode are:
       (cond
        (uniqp
 	(mew-addrbook-kill 'no-msg)
-	(message "Registered to Addrbook."))
+	(message "Registered to Addrbook"))
        (t
-	(message "Shortname is already used. Change Shortname."))))))
+	(message "Shortname is already used. Change Shortname"))))))
 
 (defun mew-addrbook-kill (&optional no-msg)
   "Kill Addrbook mode."
   (interactive "P")
   (mew-remove-buffer (current-buffer))
-  (or no-msg (message "Not registered.")))
+  (or no-msg (message "Not registered")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
