@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -17,32 +16,30 @@
   "Prepare an attachment region in the bottom of the draft.
 To compose a multipart message, you should execute this command first."
   (interactive)
-  (if (not (mew-header-p))
-      (message "Attachments region can be prepared before making a message only")
-    (if (mew-attach-p)
-	(message "Attachments region have been already prepared")
-      (let* ((attachdir (mew-attachdir))
-	     (cp (expand-file-name mew-draft-coverpage attachdir))
-	     (dir (file-name-nondirectory attachdir)))
-	(if (mew-attach-p)
-	    (message "Attachments already exist.")
-	  (mew-attach-set)
-	  (unless (file-exists-p attachdir) (mew-make-directory attachdir))
-	  (unless (file-exists-p cp) (write-region "" nil cp 'append 'no-msg))
-	  (unless mew-encode-syntax
-	    (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
-	  (mew-encode-syntax-print mew-encode-syntax)
-	  (mew-attach-goto-number 'here '(2))
-	  (if (and (not no-scroll)
-		   (not (pos-visible-in-window-p
-			 (point-max)
-			 (get-buffer-window (current-buffer)))))
-	      (set-window-start (get-buffer-window (current-buffer))
-				(mew-attach-begin)))
-	  (setq buffer-undo-list nil))))))
+  (if (mew-attach-p)
+      (message "Attachments region has already been prepared")
+    (let* ((attachdir (mew-attachdir))
+	   (cp (expand-file-name mew-draft-coverpage attachdir))
+	   (dir (file-name-nondirectory attachdir)))
+      (if (mew-attach-p)
+	  (message "Attachments already exist")
+	(mew-attach-set)
+	(unless (file-exists-p attachdir) (mew-make-directory attachdir))
+	(unless (file-exists-p cp) (write-region "" nil cp 'append 'no-msg))
+	(unless mew-encode-syntax
+	  (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
+	(mew-encode-syntax-print mew-encode-syntax)
+	(mew-attach-goto-number 'here '(2))
+	(if (and (not no-scroll)
+		 (not (pos-visible-in-window-p
+		       (point-max)
+		       (get-buffer-window (current-buffer)))))
+	    (set-window-start (get-buffer-window (current-buffer))
+			      (mew-attach-begin)))
+	(setq buffer-undo-list nil)))))
 
 (defun mew-attach-expand-path (syntax nums)
-  "ignore the first dir"
+  "Ignore the first dir"
   (let ((path ""))
     (while (cdr nums)
       (setq syntax (aref syntax (+ mew-syntax-magic (1- (car nums)))))
@@ -236,17 +233,17 @@ attachments region.
   "Delete this file or this directory in attachments."
   (interactive)
   (if (not (mew-attach-not-line02-1-dot))
-      (message "Can't delete here.")
+      (message "Cannot delete here")
     (if (= (mew-attach-line) 1)
 	(if (not (y-or-n-p "Delete entire attachments? "))
-	    (message "attachments were not deleted")
+	    (message "Attachments were not deleted")
 	  (mew-delete-directory-recursively (mew-attachdir))
 	  (setq mew-encode-syntax nil)
 	  (mew-attach-clear)
 	  (if (and mew-xemacs-p mew-icon-p)
 	      (set-specifier default-toolbar
 			     (cons (current-buffer) mew-draft-toolbar)))
-	  (message "attachments were deleted"))
+	  (message "Attachments were deleted"))
       (let* ((nums (mew-syntax-nums))
 	     (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	     (attachdir (mew-attachdir))
@@ -260,12 +257,12 @@ attachments region.
 		    "Delete %s? ")))
 	;; attachdir / {subdir/} name
 	(when (y-or-n-p (format msg ename))
-	  (message "Deleting %s ... " ename)
+	  (message "Deleting %s..." ename)
           (if (file-exists-p fullname)
               (if dirp
                   (mew-delete-directory-recursively fullname)
                 (delete-file fullname)))
-	  (message "Deleting %s ... done" ename)
+	  (message "Deleting %s...done" ename)
 	  (setq mew-encode-syntax
 		(mew-syntax-remove-entry mew-encode-syntax nums))
 	  (mew-encode-syntax-print mew-encode-syntax))))))
@@ -278,7 +275,7 @@ attachments region.
   "Create a subdirectory(i.e. multipart) on '.' in attachments."
   (interactive)
   (if (not (mew-attach-not-line012-1))
-      (message "Can't create a sub-multipart here.")
+      (message "Cannot create a sub-multipart here")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (insert-default-directory nil)
@@ -290,7 +287,7 @@ attachments region.
       ;; attachdir / dir
       (setq dirpath (mew-random-filename attachdir 1 nil))
       (if (file-exists-p dirpath)
-	  (message "Could not make a directory, sorry")
+	  (message "Cannot make a directory, sorry")
 	(setq dir (file-name-nondirectory dirpath))
 	(mew-make-directory dirpath)
 	(setq mew-encode-syntax
@@ -302,14 +299,14 @@ attachments region.
 
 (defun mew-attach-duplicate (func action setmode &optional from to default)
   (if (not (mew-attach-not-line012-1))
-      (message "Can't %s here." action)
+      (message "Cannot %s here" action)
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
 	   (frompath from)
 	   (efile (and to (concat subdir to)))
 	   (tofile to)
-	   FILE)
+	   ct msg FILE)
       (while (or (null frompath)
 		 (file-directory-p (file-chase-links frompath)))
 	(setq frompath (mew-draft-input-file-name
@@ -325,12 +322,24 @@ attachments region.
       (funcall func frompath FILE)
       (if (and setmode (/= mew-file-mode (mew-file-get-mode FILE)))
 	  (set-file-modes FILE mew-file-mode))
+      (setq ct (mew-ctdb-ct (mew-ctdb-by-file tofile)))
+      (unless ct
+	(setq ct (mew-content-type (mew-tinfo-get-case)))
+	(cond
+	 ((string= ct mew-ct-txt)
+	  (setq msg (substitute-command-keys
+		     "Attached as text. Type '\\<mew-draft-attach-map>\\[mew-attach-toggle]' to toggle text to binary")))
+	 ((string= ct mew-ct-apo)
+	  (setq msg (substitute-command-keys
+		     "Attached as binary. Type '\\<mew-draft-attach-map>\\[mew-attach-toggle]' to toggle binary to text")))))
       (setq mew-encode-syntax
 	    (mew-syntax-insert-entry
 	     mew-encode-syntax 
 	     nums
-	     (mew-encode-syntax-single tofile nil nil nil nil 'cdp)))
-      (mew-encode-syntax-print mew-encode-syntax))))
+	     (mew-encode-syntax-single tofile (list ct) nil nil nil 'cdp)))
+      (mew-encode-syntax-print mew-encode-syntax)
+      (if mew-attach-move-next-after-copy (mew-attach-next))
+      (if msg (message msg)))))
 
 (defun mew-attach-link (&optional from to)
   "Link a file with a symbolic link on '.' in attachments.
@@ -345,6 +354,29 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (mew-attach-duplicate (function copy-file) "copy" t from to))
 
+(defun mew-attach-toggle ()
+  "Toggle text/binary in attachments."
+  (interactive)
+  (if (not (mew-attach-not-line0-1-dot))
+      (message "Cannot change %s here" mew-ct:)
+    (let* ((nums (mew-syntax-nums))
+	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
+	   (file (mew-syntax-get-file syntax))
+	   (ctl (mew-syntax-get-ct syntax))
+	   (ct (mew-syntax-get-value ctl 'cap))
+	   cte)
+      (if (not (or (string= ct mew-ct-txt) (string= ct mew-ct-apo)))
+	  (message "Can't toggle %s" ct)
+	(if (string= ct mew-ct-txt)
+	    (setq ct mew-ct-apo)
+	  (setq ct mew-ct-txt))
+	(setq ctl (list ct))
+	(mew-syntax-set-ct syntax ctl)
+	(setq cte (mew-ctdb-cte (mew-ctdb-by-ct ct)))
+	(mew-syntax-set-cte syntax cte)
+	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file))
+	(mew-encode-syntax-print mew-encode-syntax)))))
+
 (defun mew-attach-link-message ()
   "Link the message displayed in Message mode with a symbolic link
 on '.' in attachments."
@@ -361,7 +393,7 @@ on '.' in attachments."
   "Open a new file into a buffer on '.' in attachments."
   (interactive)
   (if (not (mew-attach-not-line012-1))
-      (message "Can't find a new file here.")
+      (message "Cannot find a new file here")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
@@ -372,7 +404,7 @@ on '.' in attachments."
       ;; attachdir / file
       (setq filepath (mew-random-filename attachdir 1 nil mew-text-suffix))
       (if (null filepath)
-	  (message "Could not make a text file, sorry.")
+	  (message "Cannot make a text file, sorry")
 	(setq file (file-name-nondirectory filepath))
 	(setq mew-encode-syntax
 	      (mew-syntax-insert-entry
@@ -391,7 +423,7 @@ on '.' in attachments."
   "Sampling voice and insert as audio file on '.' in attachments"
   (interactive)
   (if (not (mew-attach-not-line012-1))
-      (message "Can't attach audio data here.")
+      (message "Cannot attach audio data here")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
@@ -403,7 +435,7 @@ on '.' in attachments."
       ;; attachdir / file
       (setq filepath (mew-random-filename attachdir 1 nil mew-audio-suffix))
       (if (null filepath)
-	  (message "Could not make a file for audio, sorry.")
+	  (message "Cannot make a file for audio, sorry")
 	(setq file (file-name-nondirectory filepath))
 	(save-excursion
 	  (set-buffer buf)
@@ -411,10 +443,10 @@ on '.' in attachments."
 	  (condition-case nil
 	      (progn
 		(if (not (y-or-n-p "Are you ready? "))
-		    (message "Nothing is recoded.")
+		    (message "Nothing is recoded")
 		  (message
 		   (substitute-command-keys
-		    "Type '\\[keyboard-quit]' to finish recording ... "))
+		    "Type '\\[keyboard-quit]' to finish recording..."))
 		  (mew-plet
 		   (apply (function call-process)
 			  (nth 0 mew-prog-audio2)
@@ -423,7 +455,7 @@ on '.' in attachments."
 	    (quit
 	     (message
 	      (substitute-command-keys
-	       "Type '\\[keyboard-quit]' to finish recording... done."))
+	       "Type '\\[keyboard-quit]' to finish recording...done"))
 	     (setq recorded t)
 	     (mew-flet
 	      (write-region (point-min) (point-max) filepath nil 'no-msg)))))
@@ -445,22 +477,23 @@ on '.' in attachments."
   "Unmark. The original mark appears."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't undo encoding here")
+      (message "Cannot undo encoding here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums)))
       (mew-syntax-set-privacy syntax nil)
       (mew-syntax-set-decrypters syntax nil)
+      ;; 
       (mew-syntax-set-cte syntax
-			  (mew-syntax-get-cte
-			   (mew-encode-syntax-single 
-			    (mew-syntax-get-file syntax))))
+			  (mew-ctdb-cte
+			   (mew-ctdb-by-ct
+			    (car (mew-syntax-get-ct syntax)))))
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-type ()
   "Change the data type(Content-Type:) in attachments."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't change %s here" mew-ct:)
+      (message "Cannot change %s here" mew-ct:)
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (file (mew-syntax-get-file syntax))
@@ -490,9 +523,9 @@ on '.' in attachments."
 
 (defun mew-attach-charset-base (param)
   (if (not mew-mule-p)
-      (message "This commands cannot be used on this Emacs")
+      (message "This command cannot be used on this Emacs")
     (if (not (mew-attach-not-line0-1-dot))
-	(message "Can't specify character set here")
+	(message "Cannot specify character set here")
       (let* ((nums (mew-syntax-nums))
 	     (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	     (file (mew-syntax-get-file syntax))
@@ -502,7 +535,7 @@ on '.' in attachments."
 	     (char (mew-syntax-get-param ctl param))
 	     charset)
 	(if (not (mew-ct-textp ct))
-	    (message "Can't specify character set to %s" ct)
+	    (message "Cannot use character set %s" ct)
 	  (unless char (setq char "guess"))
 	  (setq charset (mew-input-type
 			 (if (string= param "charset")
@@ -521,7 +554,7 @@ on '.' in attachments."
   "Input a description(Content-Description:) in attachments."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't describe here")
+      (message "Cannot describe here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ocd (or (mew-syntax-get-cd syntax) (mew-syntax-get-file syntax)))
@@ -535,7 +568,7 @@ on '.' in attachments."
   "Input Content-Disposition: in attachments."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't set disposition here")
+      (message "Cannot set disposition here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
@@ -553,13 +586,13 @@ on '.' in attachments."
   "Put the 'B' mark to encode with Base64 in attachments."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't encode with base64 here")
+      (message "Cannot encode with base64 here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
 	   (ct (mew-syntax-get-value ctl 'cap)))
       (if (or (mew-ct-messagep ct) (mew-ct-multipartp ct))
-	  (message "Can't encode with base64 here")
+	  (message "Cannot encode with base64 here")
 	(mew-syntax-set-cte syntax mew-b64)
 	(mew-encode-syntax-print mew-encode-syntax)))))
 
@@ -567,13 +600,13 @@ on '.' in attachments."
   "Put the 'Q' mark to encode with Quoted-Printable in attachments"
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't encode with quoted-printable here")
+      (message "Cannot encode with quoted-printable here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
 	   (ct (mew-syntax-get-value ctl 'cap)))
       (if (or (mew-ct-messagep ct) (mew-ct-multipartp ct))
-	  (message "Can't encode with quoted-printable here")
+	  (message "Cannot encode with quoted-printable here")
 	(mew-syntax-set-cte syntax mew-qp)
 	(mew-encode-syntax-print mew-encode-syntax)))))
 
@@ -583,7 +616,7 @@ applicable only to Text/Plain and Application/Postscript since compression
 is not effective other objects. For example, JPEG is already compressed."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
-      (message "Can't apply gzip here")
+      (message "Cannot apply gzip here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
@@ -594,7 +627,7 @@ is not effective other objects. For example, JPEG is already compressed."
 	  ;; never use compress to types other than text/plain or 
 	  ;; application/postscript. Indeed, compression is not
 	  ;; effective to compressed data such as jpeg.
-	  (message "Can't apply gzip to %s" ct)
+	  (message "Cannot apply gzip to %s" ct)
 	(mew-syntax-set-cte syntax mew-xg)
 	(mew-encode-syntax-print mew-encode-syntax)))))
 
@@ -603,9 +636,9 @@ is not effective other objects. For example, JPEG is already compressed."
   (interactive)
   (cond
    ((not (mew-attach-not-line0-1-dot))
-    (message "Can't PGP sign here"))
+    (message "Cannot PGP sign here"))
    ((null mew-pgp-ver)
-    (message "%s doesn't exist" mew-prog-pgp))
+    (message "%s does not exist" mew-prog-pgp))
    (t
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
@@ -615,7 +648,7 @@ is not effective other objects. For example, JPEG is already compressed."
        ((>= (length privacy) 2)
 	(message "Too many marks"))
        ((member (car pgp-sign) privacy) ;; xxx hard coding
-	(message "Already marked."))
+	(message "Already marked"))
        (t
 	(mew-syntax-set-privacy 
 	 syntax (append privacy pgp-sign))
@@ -627,9 +660,9 @@ Input decryptors' addresses."
   (interactive)
   (cond
    ((not (mew-attach-not-line0-1-dot))
-    (message "Can't PGP encrypt here"))
+    (message "Cannot PGP encrypt here"))
    ((null mew-pgp-ver)
-    (message "%s doesn't exist" mew-prog-pgp))
+    (message "%s does not exist" mew-prog-pgp))
    (t
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
@@ -640,7 +673,7 @@ Input decryptors' addresses."
        ((>= (length privacy) 2)
 	(message "Too many marks"))
        ((member (car pgp-enc) privacy) ;; xxx hard coding
-	(message "Already marked."))
+	(message "Already marked"))
        (t
 	;; ask towhom before set privacy for C-g
 	(if towhom
@@ -660,7 +693,7 @@ Input decryptors' addresses."
   "Open this file into a buffer in attachments."
   (interactive)
   (if (not (mew-attach-not-line012-1-dot))
-      (message "Can't find a file here.")
+      (message "Cannot find a file here")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
@@ -673,7 +706,7 @@ Input decryptors' addresses."
       ;; buffer switched
       (local-set-key "\C-c\C-q" 'mew-kill-buffer)
       (if (file-symlink-p FILE)
-	  (message "This file is indicated by a symbolic link. Take care.")))))
+	  (message "This file is a symbolic link. Take care")))))
 
 (provide 'mew-attach)
 
