@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Aug 20, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -29,12 +28,15 @@
 (defsubst mew-header-decode-get-error-str (b-or-q)
   (cdr (mew-assoc-case-equal b-or-q mew-header-decode-error-switch 0)))
 
-(defconst mew-header-decode-regex 
+(defconst mew-header-decode-regex
   "=\\?\\([^? \t]+\\)\\?\\(.\\)\\?\\([^? \t]+\\)\\?=")
 
 (defconst mew-header-decode-regex2
   "[ \t]*=\\?\\([^? \t]+\\)\\?\\(.\\)\\?\\([^? \t]+\\)\\?=")
 
+(defconst mew-header-decode-regex3
+  "^\\([ \t]+\\)=\\?[^? \t]+\\?.\\?[^? \t]+\\?=")
+
 (defconst mew-header-param-regex
   "^\\([^=*]+\\)\\(\\|\\*[0-9]+\\)\\(\\*?\\)=\\(.*\\)$")
 
@@ -86,7 +88,7 @@
     (cond
      ((not (mew-coding-system-p cs))
       mew-error-unknown-charset)
-     (fun ;; if cs is nil, mew-cs-decode-string doesn't cs-decode.
+     (fun ;; if cs is nil, mew-cs-decode-string does not cs-decode.
       (setq str (funcall fun estr))
       (if (null str)
 	  (mew-header-decode-get-error-str b-or-q)
@@ -198,7 +200,7 @@
 ;; fits in 75 length. However, it is very difficult first because
 ;; it is very difficult to know actual length of 'str' after convention
 ;; from the internal representation to charset encoding. Second because
-;; ISO-2022-JP can't be simply split. If split, extra escape sequences
+;; ISO-2022-JP cannot be simply split. If split, extra escape sequences
 ;; appear. Moreover, it is not effective to expect the length of results
 ;; because 'str' is short enough in most cases. So, we measure the length
 ;; of results. If it is longer than 75, 'str' is split and 'substr1' and
@@ -303,8 +305,8 @@
 	 SBOUND open c I)
     ;; status space, ascii, non-ascii, non-ascii-space
     ;; assumptions:
-    ;;  <> doesn't contain non-ascii characters.
-    ;;  () doesn't recurse.
+    ;;  <> does not contain non-ascii characters.
+    ;;  () does not recurse.
     ;; if " " contains non-ascii, cause an error.
     (while (< i len)
       (setq c (aref str i))
@@ -406,7 +408,7 @@
 	;;   An 'encoded-word' that appears within a
 	;;   'phrase' MUST be separated from any adjacent 'word', 'text' or
 	;;   'special' by 'linear-white-space'.
-	;; However, we don't care if an 'encoded-word' is followed by
+	;; However, we do not care if an 'encoded-word' is followed by
 	;; ":". The spec is believed too strict. 'encoded-word' can be
 	;; recognized because of the token rule.
 	(insert ":")
@@ -558,7 +560,7 @@
   (goto-char (point-min))
   (cond
    ((eq type 'struct)
-    ;; If each line doesn't end with ",", unfold it.
+    ;; If each line does not end with ",", unfold it.
     ;; In Page 5 of RFC822 says, "Unfolding is accomplished by
     ;; regarding CRLF immediately followed by a LWSP-char as
     ;; equivalent to the LWSP-char". However, it also says, 
@@ -566,6 +568,9 @@
     ;; characters (namely HTABs and SPACEs) are treated as single 
     ;; spaces and may freely surround any symbol." So, remove
     ;; continuous white spaces.
+    (when (and mew-decode-broken (looking-at "\n[ \t]+"))
+      (replace-match "" nil t))
+    (goto-char (point-min))
     (while (re-search-forward ",[ \t]+\n" nil t)
       (replace-match ",\n" nil t))
     (goto-char (point-min))
@@ -609,20 +614,22 @@
 
 (defun mew-decode-warning-fields (key err)
   (unless (member key mew-no-warning-fields)
-    (mew-xinfo-set-warning
-     (cons
+    (let (wmsg level)
       (cond
-       ((eq err 'raw)
-	(concat key " has raw text strings.\n"))
-       ((eq err 'tab)
-	(concat key " has meaningless TAB characters.\n"))
        ((eq err 'spc)
-	(concat key " has continuous space characters.\n"))
-       ((eq err 'quoted)
-	(concat key " has encoded-words in quoted text.\n"))
+	(setq level 1)
+	(setq wmsg (concat "tab/spc characters on " key " are simplified.\n")))
        ((eq err 'ctl)
-	(concat key " has illegal control codes.\n")))
-      (mew-xinfo-get-warning)))))
+	(setq level 1)
+	(setq wmsg (concat "some control code on " key " are removed.\n")))
+       ((eq err 'raw)
+	(setq level 2)
+	(setq wmsg (concat key " has raw text strings.\n")))
+       ((eq err 'quoted)
+	(setq level 2)
+	(setq wmsg (concat key " has encoded-words in quoted text.\n"))))
+      (if (>= level mew-warning-field-level)
+	  (mew-xinfo-set-warning (cons wmsg (mew-xinfo-get-warning)))))))
 
 (defun mew-decode-warning-params (param err)
   (unless (and param (member (downcase param) mew-no-warning-params))
@@ -661,7 +668,7 @@ That is, each line may be more than 75. "
 	;; 'encoded-word's is ignored". So, use looking-at.
 	(cond
 	 ((memq type '(text comma-text))
-	  ;; This is text field. Quote doesn't care!
+	  ;; This is text field. We do not care quote!
 	  ;; make use of the search above
 	  (mew-header-decode-string key)
 	  (while (looking-at mew-header-decode-regex2)
@@ -676,7 +683,7 @@ That is, each line may be more than 75. "
 	  ;; encoded-word in quoted-string should not be decoded
 	  ;; according to RFC 2047. However, if users wish
 	  ;; (ie mew-decode-broken is *non-nil*), decode it.
-	  ;; Regular expression can't express quoted string. So,
+	  ;; Regular expression cannot express quoted string. So,
 	  ;; canonicalize as follows:
 	  ;;     Not quoted line (may be null line)
           ;;     "quoted line"
@@ -720,18 +727,18 @@ That is, each line may be more than 75. "
 	  (let (ws)
 	    ;; We want to warn only in the case of ^\t.
 	    (goto-char (point-min))
-	    ;; Because lines are already unfolded, we can't tell
+	    ;; Because lines are already unfolded, we cannot tell
 	    ;; whether or not this TABs locate on the beginning
 	    ;; of a line. But this should be acceptable.
 	    (while (re-search-forward "\t+" nil t)
-	      (replace-match " " nil t)
-	      (setq ws 'tab))
+	      (or ws (setq ws t))
+	      (replace-match " " nil t))
 	    (goto-char (point-min))
 	    ;; Continuous SPCs are not illegal, just for appearance.
 	    (while (re-search-forward "  +" nil t)
-	      (or ws (setq ws 'spc))
+	      (or ws (setq ws t))
 	      (replace-match " " nil t))
-	    (if ws (mew-decode-warning-fields key ws))))
+	    (if ws (mew-decode-warning-fields key 'spc))))
       ;;
       (mew-header-sanity-check-region (point-min) (point-max) key)
       (goto-char (point-max)))))
@@ -867,13 +874,15 @@ That is, each line may be more than 75. "
     (let (pre med pst)
       (mew-decode-warning-params name 'enc)
       (while (string-match mew-header-decode-regex value)
-	;; We can't ensure that sub-function doesn't use match functions.
+	;; We cannot ensure that sub-function does not use match functions.
 	;; So, pre/pst prevents infinite loop.
 	(setq pre (substring value 0 (match-beginning 0)))
 	(setq pst (substring value (match-end 0)))
 	(setq med (mew-header-decode (mew-match 1 value)
 				     (mew-match 2 value)
 				     (mew-match 3 value)))
+	(if (string-match mew-header-decode-regex3 pst)
+	    (setq pst (substring pst (match-end 1))))
 	(setq value (concat pre med pst)))))
   value)
 
