@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -30,7 +29,7 @@
        (eq (mew-cinfo-get-size) size)))
 
 (defvar mew-xinfo-list
-  '("decode-err" "warning" "info" "multi-form" "icon-spec"
+  '("decode-err" "warning" "info" "action" "multi-form" "icon-spec"
     "pri-result" "not-decrypted" "text-body"))
 
 (mew-blinfo-defun 'mew-xinfo mew-xinfo-list)
@@ -77,13 +76,11 @@ If MUST-HIT is non-nil and no valid cache entry is found, an error occurs."
 	    (error "Cache not found.")
 	  nil)
       (setq cache (mew-cache-buffer-get entry))
-      (if (mew-folder-remotep fld)
-	  ()
-	(setq file (mew-expand-folder fld msg))
-	(if (not (file-readable-p file))
-	    (error "%s%s doesn't exist" (file-name-as-directory fld) msg))
-	(setq time (mew-file-get-time file))
-	(setq size (mew-file-get-size file)))
+      (setq file (mew-expand-folder fld msg))
+      (if (not (file-readable-p file))
+	  (error "%s%s does not exist" (file-name-as-directory fld) msg))
+      (setq time (mew-file-get-time file))
+      (setq size (mew-file-get-size file))
       (save-excursion
 	(set-buffer cache)
 	(setq ok (mew-cinfo-equal fld msg time size)))
@@ -130,7 +127,7 @@ If MUST-HIT is non-nil and no valid cache entry is found, an error occurs."
 (defun mew-cache-message (fld msg &optional unlimit no-err)
   "Cache the message specified by FLD and MSG.
 If an invalid message are cached, deletes it and caches the message again.
-If UNLIMIT is non-nil, decodes the message to be cached with out
+If UNLIMIT is non-nil, decodes the message to be cached without
 the limitations. If NO-ERR is non-nil, an error is caused
 if decode fails."
   (let* ((cbuf (current-buffer))
@@ -161,7 +158,7 @@ if decode fails."
 	 (setq errormsg (nth 1 errmsg))
 	 (throw 'return (setq cache nil))))
       (mew-cinfo-set fld msg (car tim-siz) (cdr tim-siz))
-      (mew-dinfo-set nil t)
+      (mew-dinfo-set nil t t mew-use-alternative)
       (mew-decode-syntax-clear)
       (mew-xinfo-set-text-body mew-use-text-body)
       (condition-case nil
@@ -175,7 +172,7 @@ if decode fails."
 	(quit
 	 ;; prefetching an encrypted message
 	 (mew-cache-delete)
-	 (message "MIME decoding for %s/%s is quitted." fld msg)
+	 (message "MIME decoding for %s/%s aborted" fld msg)
 	 (throw 'return (setq cache nil))))
       (mew-ainfo-set-icon msg)
       (mew-decode-syntax-set))
