@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -17,13 +16,21 @@
 ;; t means decode with MIME encoding and convert charset.
 ;; symbol means decode with MIME encoding only.
 
-(defvar mew-dinfo-list '("limit" "decode-body"))
+(defvar mew-dinfo-list '("limit" "decode-text" "decode-binary" "use-alt"))
 
 (mew-blinfo-defun 'mew-dinfo mew-dinfo-list)
 
-(defun mew-dinfo-set (lim db)
+(defun mew-dinfo-set (lim dt db alt)
   (mew-dinfo-set-limit lim)
-  (mew-dinfo-set-decode-body db))
+  (mew-dinfo-set-decode-text dt)
+  (mew-dinfo-set-decode-binary db)
+  (mew-dinfo-set-use-alt alt))
+
+(defun mew-cache-dinfo-get-use-alt (buf)
+  (when buf
+    (save-excursion
+      (set-buffer buf)
+      (mew-dinfo-get-use-alt))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -82,12 +89,13 @@
 	 (beginning-of-line)
        (goto-char (point-max))
        (insert "\n"))
-     (if (and (integerp mew-header-max-length)
-	      (> (mew-count-lines (point-min) (point))
-		 mew-header-max-length))
-	 (mew-decode-error
-	  (substitute-command-keys
-	   "Too long header. To see the message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'")))
+     (when (and (integerp mew-header-max-length)
+		(> (mew-count-lines (point-min) (point))
+		   mew-header-max-length))
+       (mew-xinfo-set-action
+	(substitute-command-keys
+	 "Too long header. To see the message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'"))
+       (error ""))
      (save-restriction
        (narrow-to-region (point-min) (point))
        (goto-char (point-min))
@@ -173,7 +181,7 @@
 	       (put-text-property (match-end 0) (1- (point))
 				  'face val-face)))))))))
 
-(defun mew-decode-rfc822-header (&optional no-property)
+(defun mew-decode-rfc822-header (&optional cnt)
   "A function to handle RFC822 header.
 Called on the beginning of the header in the narrowed region.
  - Decode and highlight RFC822 fields excluding MIME fields.
@@ -183,7 +191,7 @@ The cursor moves between mew-mv: and MIME fields.
 Return the existence MIME-Version: and the value of Subject:."
   (let* ((case-fold-search t)
 	 (visibles (make-list (length mew-field-spec) nil))
-	 (prop (and mew-use-highlight-header (not no-property)))
+	 (prop mew-use-highlight-header)
 	 key beg med subj from contents others
 	 key-face val-face N nspec visiblep mimep)
     (mew-decode-narrow-to-header
@@ -202,6 +210,12 @@ Return the existence MIME-Version: and the value of Subject:."
 	  ((string= key mew-x-mew:)
 	   ;; deleting X-Mew: on the RFC822 header
 	   (delete-region beg (point)))
+	  ((string= key mew-x-mew-uidl:)
+	   (when (and (eq cnt 1)
+		      (not (mew-folder-mdropp (mew-cinfo-get-fld))))
+	     (mew-xinfo-set-action
+	      (substitute-command-keys 
+	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\<mew-summary-mode-map>\\[mew-summary-retrieve-message]'"))))
 	  ((string-match "^Content-" key)
 	   ;; Due to PGP/MIME, properties are not put here.
 	   :; This must be "buffer-substring".
@@ -271,7 +285,7 @@ Return the existence MIME-Version: and the value of Subject:."
       (put-text-property beg (point) 'mew-noncontents t))
     ;; the beginning of the content header
     (save-excursion (mapcar (function insert) (nreverse contents)))
-    ;; 'mew-contents doesn't work due to PGP/MIME
+    ;; 'mew-contents does not work due to PGP/MIME
     (list mimep subj from)))
 
 (defun mew-decode-mime-header (&optional dct)
@@ -336,7 +350,6 @@ Return a part syntax after moving the beginning of the content body."
   ;;
   (goto-char (point-min)) ;; just in case
   (unless (re-search-forward mew-eoh nil t)
-    (mew-xinfo-set-decode-err "No end-of-header(null line) in the top level")
     (goto-char (point-max))
     (if (not (bolp)) (insert "\n"))
     (insert "\n"))
@@ -395,7 +408,7 @@ Return a part syntax after moving the beginning of the content body."
     (mew-decode-syntax-rfc822 syntax)) ;; return value
    (t
     ;; Analyze recursively.
-    (let* ((msf (mew-decode-rfc822-header)) ;; on the physical
+    (let* ((msf (mew-decode-rfc822-header cnt)) ;; on the physical
 	   (mimep (nth 0 msf))
 	   (subj (nth 1 msf))
 	   (mew-inherit-decode-signer (nth 2 msf)) ;; signature verification
@@ -420,7 +433,7 @@ Return a part syntax after moving the beginning of the content body."
 	    (forward-line)
 	  (mew-decode-error "No end-of-header(null line) in RFC822 message"))
 	;; the beginning of the BODY(i.e. the physical content body)
-	(if (eq (mew-dinfo-get-decode-body) t)
+	(if (eq (mew-dinfo-get-decode-text) t)
 	    (mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
 	(mew-syntax-set-key syntax 'message)
 	(mew-syntax-set-end syntax (1- (point)))
@@ -433,13 +446,11 @@ Return a part syntax after moving the beginning of the content body."
 ;;; Decoding singlepart
 ;;;
 
-(defun mew-decode-mime-body (ctl cte)
+(defun mew-decode-mime-body (textp ct cte &optional charset)
   ;; ((point), (point-max)) (not point-min)
-  (let* ((ct (mew-syntax-get-value ctl 'cap))
-	 (textp (mew-ct-textp ct))
-	 (linebasep (or textp (mew-ct-linebasep ct)))
+  (let* ((linebasep (mew-ct-linebasep ct))
 	 (beg (point))
-	 opt file charset fromcs decoded switch)
+	 opt file decoded switch)
     (unless (or (null cte) (mew-cte-composite-p cte))
       (when (and (mew-case-equal cte mew-b64) (fboundp 'base64-decode-region))
 	(condition-case nil
@@ -477,25 +488,27 @@ Return a part syntax after moving the beginning of the content body."
 		  file '(t nil) nil opt))
 	  (if (file-exists-p file) (delete-file file))))
        (t
-	(mew-decode-error (concat mew-prog-mime-decode " doesn't exist")))))
+	(mew-decode-error (concat mew-prog-mime-decode " does not exist")))))
     ;; charset conversion
-    (when (and textp (eq (mew-dinfo-get-decode-body) t))
-      (setq charset (mew-syntax-get-param ctl "charset"))
-      (if charset (setq fromcs (mew-charset-to-cs charset)))
-      ;; fromcs is nil if charset is "us-ascii"
-      ;; (mew-coding-system-p nil) returns t.
-      (unless (mew-coding-system-p fromcs)
-	(mew-decode-warning-body 'unknown-charset charset))
-      (when (and mew-decode-broken
-		 (or (null charset)
-		     (mew-case-equal charset mew-us-ascii))) ;; anyway
-	(goto-char beg)
-	(when (re-search-forward mew-regex-esc-or-nonascii nil t)
-	  (mew-decode-warning-body 'no-charset charset)
-	  (setq fromcs mew-cs-autoconv))
-	(goto-char beg))
-      (if (mew-coding-system-p fromcs)
-	  (mew-cs-decode-region beg (point-max) fromcs)))))
+    (if (and textp (eq (mew-dinfo-get-decode-text) t)) ;; not 'no-cs-conv
+	(mew-decode-charset-conv charset beg))))
+
+(defun mew-decode-charset-conv (charset beg)
+  (let* ((fromcs (and charset (mew-charset-to-cs charset))))
+    ;; fromcs is nil if charset is "us-ascii"
+    ;; (mew-coding-system-p nil) returns t.
+    (unless (mew-coding-system-p fromcs)
+      (mew-decode-warning-body 'unknown-charset charset))
+    (when (and mew-decode-broken
+	       (or (null charset)
+		   (mew-case-equal charset mew-us-ascii))) ;; anyway
+      (goto-char beg)
+      (when (re-search-forward mew-regex-esc-or-nonascii nil t)
+	(mew-decode-warning-body 'no-charset charset)
+	(setq fromcs mew-cs-autoconv))
+      (goto-char beg))
+    (if (mew-coding-system-p fromcs)
+	(mew-cs-decode-region beg (point-max) fromcs))))
 
 (defun mew-decode-singlepart (cnt &optional dct parent)
   ;; Called on the beginning of the content header in the narrowed region
@@ -503,6 +516,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (syntax (mew-decode-mime-header dct))
 	 (ctl (mew-syntax-get-ct syntax))
 	 (ct (mew-syntax-get-value ctl 'cap))
+	 (textp (mew-ct-textp ct))
 	 (cte (or (mew-syntax-get-cte syntax) mew-7bit))
 	 (multi-err t) (encap nil) type)
     ;; the beginning of the content body
@@ -561,15 +575,20 @@ Return a part syntax after moving the beginning of the content body."
      ;; Others
      (t
       (if (and (eq parent 'message)
-	       (or (not (mew-xinfo-get-text-body)) (not (mew-ct-textp ct))))
+	       (or (not (mew-xinfo-get-text-body)) (not textp)))
 	  (setq encap t))
       ;; even if cte is nil, call mew-decode-mime-body for charset conversion
-      (if (mew-dinfo-get-decode-body)
-	  (mew-decode-mime-body ctl cte))))
+      (if textp
+	  (if (mew-dinfo-get-decode-text)
+	      (let ((charset (mew-syntax-get-param ctl "charset")))
+		(mew-decode-mime-body t ct cte charset)))
+	(if (mew-dinfo-get-decode-binary)
+	    (mew-decode-mime-body nil ct cte)))))
     ;; ct may be changed to apo
+    ;; or single in multipart/alternative
+    ;; or single in multipart/security
     (setq type (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-    (unless (or (string= type mew-ct-msg)
-		(and mew-use-alternative (string= ct mew-ct-mla)))
+    (unless (mew-syntax-get-end syntax)
       (mew-syntax-set-end syntax (point-max)))
     (when (and (string= type mew-ct-txt)
 	       ;; highlight in the cache buffers only
@@ -607,6 +626,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (ctl (mew-syntax-get-ct syntax))
 	 (ct (mew-syntax-get-value ctl 'cap))
 	 (boundary (mew-syntax-get-param ctl "boundary"))
+	 (count 0)
 	 (parts []) part
 	 pref lastpref prefpart atpref lastatpref
 	 bregex start break)
@@ -618,7 +638,7 @@ Return a part syntax after moving the beginning of the content body."
     ;; appear in its entirety following the CRLF".
     (setq bregex (mew-decode-multipart-boundary-regex boundary))
     (unless (and (re-search-forward bregex nil t)
-	     (mew-decode-multipart-boundary-cont))
+		 (mew-decode-multipart-boundary-cont))
       (mew-decode-error (format "No first boundary for %s" ct)))
     (forward-line) ;; the beginning of the next part
     (setq start (point)) ;; the beginning of the part
@@ -629,7 +649,7 @@ Return a part syntax after moving the beginning of the content body."
 	    () ;; close-delimiter need not be followed by CRLF
 	  (forward-line)) ;; the beginning of the next part
 	(save-excursion
-	  (if (not break) (forward-line -1))
+	  (unless break (forward-line -1))
 	  (beginning-of-line)
 	  (forward-char -1) ;; skip the preceeding CRLF
 	  ;; the end of the part
@@ -638,7 +658,8 @@ Return a part syntax after moving the beginning of the content body."
 	    (goto-char (point-min))
 	    ;; the beginning of the part
 	    (setq part (mew-decode-singlepart cnt dct nil))
-	    (if (and mew-use-alternative (string= ct mew-ct-mla))
+	    (setq count (1+ count))
+	    (if (and (mew-dinfo-get-use-alt) (string= ct mew-ct-mla))
 		(progn
 		  (setq pref (or (mew-multipart-alternative-preference part)
 				 (length mew-mime-multipart-alternative-list)))
@@ -657,7 +678,7 @@ Return a part syntax after moving the beginning of the content body."
 	      (setq parts (vconcat parts (vector part))))))
 	(setq start (point)) ;; the beginning of the part
 	(when break
-	  (if (and mew-use-alternative (string= ct mew-ct-mla))
+	  (if (and (mew-dinfo-get-use-alt) (string= ct mew-ct-mla))
 	      (let* ((beg (mew-syntax-get-begin prefpart))
 		     (ctl (mew-syntax-get-ct prefpart))
 		     (ct (mew-syntax-get-value ctl 'cap))
@@ -669,12 +690,13 @@ Return a part syntax after moving the beginning of the content body."
 		  (mew-xinfo-get-info)))
 		(throw 'multipart prefpart))
 	    (throw 'multipart (vconcat syntax parts)))))
-      ;; Let's try to decode even if boundary is not found.
+      ;; Let's try to decode the first part even if boundary is not found.
       ;; This should save truncated messages.
-      (save-restriction
-	(narrow-to-region start (point-max))
-	(goto-char (point-min))
-	(mew-decode-singlepart cnt dct nil))
+      (when (= count 0)
+	(save-restriction
+	  (narrow-to-region start (point-max))
+	  (goto-char (point-min))
+	  (mew-decode-singlepart cnt dct nil)))
       (mew-decode-error (format "No last boundary for %s" ct)))))
 
 ;;;
@@ -737,7 +759,7 @@ Return a part syntax after moving the beginning of the content body."
       (setq result (concat "unknown protocol " proto)))
      ((not existp)
       (setq result (concat (mew-decode-get-security-prog proto switch)
-			   " doesn't exist")))
+			   " does not exist")))
      (t
       (setq file3result (funcall func file1 file2))
       (setq file3 (nth 0 file3result) result (nth 1 file3result))))
@@ -821,7 +843,7 @@ Return a part syntax after moving the beginning of the content body."
       (setq result (concat "unknown protocol " proto)))
      ((not existp)
       (setq result (concat (mew-decode-get-security-prog proto switch)
-			   " doesn't exist")))
+			   " does not exist")))
      (t
       (setq result (funcall func file1 file2))))
     ;; Throw away garbage
