@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -34,6 +33,7 @@
 
 (defun mew-draft-set-local-variables ()
   (auto-save-mode mew-draft-mode-auto-save)
+  (make-local-variable 'completion-ignore-case)
   (make-local-variable 'paragraph-start)
   (setq paragraph-start (concat mew-eoh "\\|[ \t]*$\\|" page-delimiter))
   (make-local-variable 'paragraph-separate)
@@ -196,7 +196,8 @@ Moreover, '~/.mew-fib' supports aliases like;
 (defun mew-draft-mode-name (&optional header)
   (let (pcdb sub)
     (cond
-     ((mew-tinfo-get-privacy-type)
+     ((or (mew-tinfo-get-privacy-type) (mew-tinfo-get-privacy-err))
+      ;; If privacy err, don't display mew-protect-privacy-always-type etc.
       (setq pcdb (mew-pcdb-by-service (mew-tinfo-get-privacy-type)))
       (setq sub (mew-pcdb-mark pcdb)))
      ((and (mew-tinfo-get-encrypted-p) mew-protect-privacy-encrypted)
@@ -345,7 +346,7 @@ the Body: field."
     (cond
      ((null to) (insert mew-to: " \n"))
      ((stringp to) ;; To: inputed from the mini-buffer.
-      ;; Don't check to is mine. Cc: is also string
+      ;; do not check to is mine. Cc: is also string
       ;; We believe that user never specifies the same address of To: to Cc:.
       (insert mew-to: " " to "\n"))
      ;; To: collected by reply
@@ -385,20 +386,21 @@ the Body: field."
 					      (max (buffer-size) 1))))
 	  (mew-remove-buffer buf)
 	  (mew-draft-header-insert mew-x-face: xface)))
-    (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
     (mew-draft-header-insert mew-organization: (mew-organization case))
     (setq body (mew-draft-header-insert-alist other-headers))
     ;; Deleting fields defined in mew-header-alist to replace them.
     (mew-header-delete-lines (mapcar (function car) (mew-header-alist case)))
     (mew-header-goto-end)
     (mew-draft-header-insert-alist (mew-header-alist case))
+    ;; X-Mailer: must be the last
+    (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
     (mew-header-prepared)
     ;; on the body
     (if nl (insert "\n"))
     (if body (save-excursion (insert body)))
     ;; move the cursor after "To: "
     (goto-char (point-min))
-    (forward-char 4))) ;; Don't use (end-of-line) since the value may exist.
+    (forward-char 4))) ;; do not use (end-of-line) since the value may exist.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -428,7 +430,7 @@ citation prefix and label.
 ;; MUST be able to cancel by C-x u
   (interactive "P")
   (if (and (not force) (or (mew-in-header-p) (mew-in-attach-p)))
-      (message "You cannot cite a message here.")
+      (message "Cannot cite a message here")
     (let (cite beg end)
       (save-excursion
 	(set-buffer (mew-buffer-message))
@@ -468,7 +470,7 @@ citation prefix and label.
 ;; MUST be able to cancel by C-x u
   (interactive "P")
   (if (and (not force) (or (mew-in-header-p) (mew-in-attach-p)))
-      (message "You cannot cite a message here.")
+      (message "Cannot cite a message here")
     (let ((nonmewbuf mew-message-citation-buffer) ;; buffer local, so copy here
 	  (fld (mew-current-get-fld (mew-frame-id)))
 	  (msg (mew-current-get-msg (mew-frame-id)))
@@ -480,7 +482,7 @@ citation prefix and label.
 	(setq tbuf (or nonmewbuf (mew-buffer-message)))
 	(if tbuf
 	    (set-buffer tbuf)
-	  (error "No buffer to be cited."))
+	  (error "No buffer to be cited"))
 	(save-restriction
 	  ;; first prepare "cite"
 	  (widen)
@@ -508,7 +510,7 @@ citation prefix and label.
 		       (mew-cache-hit fld msg)))
 	(if tbuf
 	    (set-buffer tbuf)
-	  (error "No buffer to be cited."))
+	  (error "No buffer to be cited"))
 	(save-restriction
 	  (widen)
 	  (mew-header-goto-end)
@@ -537,7 +539,7 @@ citation prefix and label.
                 (unless irtl (goto-char (mew-header-end)))
                 (mew-draft-header-insert mew-in-reply-to: irt)))))
       (save-restriction
-	;; this gets complicated due to supercite, please don't care
+	;; this gets complicated due to supercite, please do not care
 	(narrow-to-region (point)(point)) ;; for (goto-char (point-min))
 	(insert cite)
 	(push-mark (point) t t)
@@ -705,16 +707,19 @@ the cursor position."
 ;;; Re-highlight
 ;;; 
 
+(defun mew-draft-rehighlight-body ()
+  (save-excursion
+    (let ((beg (progn (goto-char (mew-header-end)) (forward-line) (point)))
+	  (end (or (mew-attach-begin) (point-max))))
+      (mew-highlight-body-region beg end 'draft 'rehighlight))))
+
 (defun mew-draft-rehighlight ()
   "Highlight header and body again."
   (interactive)
   (let ((mod (buffer-modified-p)))
     (mew-highlight-header)
     (mew-draft-header-keymap)
-    (save-excursion
-      (let ((beg (progn (goto-char (mew-header-end)) (forward-line) (point)))
-            (end (or (mew-attach-begin) (point-max))))
-        (mew-highlight-body-region beg end 'draft 'rehighlight)))
+    (mew-draft-rehighlight-body)
     (set-buffer-modified-p mod)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -748,8 +753,9 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 			services))
 	 str)
     (setq str (completing-read "Input privacy services : " alist nil t))
-    (if (stringp str)
-	(mew-tinfo-set-privacy-type (cdr (assoc str alist)))))
+    (when (stringp str)
+      (mew-tinfo-set-privacy-type (cdr (assoc str alist)))
+      (mew-tinfo-set-privacy-err nil)))
   (mew-draft-mode-name))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -760,37 +766,55 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 (defun mew-draft-process-message (action &optional privacy signer)
   (let* ((buf (current-buffer))
 	 (case (mew-tinfo-get-case))
-	 (server (mew-smtp-server case))
-	 (ssh-server (mew-smtp-ssh-server case))
-	 (pnm (mew-smtp-info-name server ssh-server))
-	 (old-case (mew-tinfo-get-case))
-	 sendit err)
+	 (old-case case)
+	 resentp fcc guessed-case server ssh-server pnm sendit err)
+    (run-hooks 'mew-send-hook)
+    (when mew-case-guess-when-composed
+      (setq guessed-case (mew-draft-get-case-by-guess))
+      (when guessed-case
+	(if mew-case-guess-addition
+	    (setq case (mew-draft-add-case case guessed-case))
+	  (setq case guessed-case))))
+    (setq server (mew-smtp-server case))
+    (setq ssh-server (mew-smtp-ssh-server case))
+    (setq pnm (mew-smtp-info-name case))
     (if (mew-smtp-get-lock pnm)
-	(message "Another message is being sent. Try later.")
-      (run-hooks 'mew-send-hook)
-      (if mew-case-guess-when-composed
-	  (mew-draft-set-case-by-guess))
-      (unless (string= old-case (mew-tinfo-get-case))
+	(message "Another message is being sent. Try later")
+      (unless (string= old-case case)
+	(mew-tinfo-set-case case)
+	(mew-draft-mode-name (mew-tinfo-get-hdr-file))
 	(mew-draft-replace-fields old-case)
 	(when (eq action 'send)
 	  (mew-highlight-header)
 	  (unless (mew-tinfo-get-hdr-file) (mew-draft-header-keymap)))
 	(save-buffer))
-      (if (and (eq action 'send) mew-ask-send)
-	  (setq sendit (y-or-n-p "Really send this message? "))
-	(setq sendit t))
+      ;; Check resentp
+      (save-excursion
+	(goto-char (point-min))
+	(setq resentp (re-search-forward "^Resent" (mew-header-end) t)))
+      ;; Ask Subject: before the query of "Really send".
+      ;; Typing C-g here gets back to the draft.
+      (mew-encode-ask-subject)
+      (setq fcc (mew-encode-ask-fcc resentp))
+      (mew-encode-ask-newsgroups)
+      (if (eq action 'queue)
+	  (setq sendit t)
+	(if mew-ask-send
+	    (setq sendit (y-or-n-p "Really send this message? "))
+	  (setq sendit t)))
       (when sendit
 	(mew-smtp-set-case pnm case)
 	(mew-smtp-set-server pnm server)
 	(mew-smtp-set-port pnm (mew-smtp-port case))
 	(mew-smtp-set-ssh-server pnm ssh-server)
 	(mew-smtp-set-queue pnm (mew-queue-folder case))
+	(mew-smtp-set-fcc pnm fcc)
 	(mew-current-get-window-config)
 	(delete-windows-on buf) ;; just in case
 	(save-excursion
 	  (save-window-excursion
 	    (set-buffer buf)
-	    (if (mew-encode pnm case privacy signer)
+	    (if (mew-encode pnm case resentp privacy signer)
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (if (file-exists-p mdi) (delete-file mdi))
 		  (cond
