@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Nov  9, 1999
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -92,11 +91,13 @@ Execute mew-dinfo-set before calling this."
 	   (case (mew-tinfo-get-case))
 	   (server (mew-smtp-server case))
 	   (ssh-server (mew-smtp-ssh-server case))
-	   (pnm (mew-smtp-info-name server ssh-server))
-	   sendit
-	   err)
+	   (pnm (mew-smtp-info-name case))
+	   sendit resentp fcc err)
       (if (mew-smtp-get-lock pnm)
-	  (message "Another message is being sent. Try later.")
+	  (message "Another message is being sent. Try later")
+	(goto-char (point-min))
+	(setq resentp (re-search-forward "^Resent" (mew-header-end) t))
+	(setq fcc (mew-encode-ask-fcc resentp))
 	(if (and (eq action 'send) mew-ask-send)
 	    (setq sendit (y-or-n-p "Really send this message? "))
 	  (setq sendit t))
@@ -106,6 +107,7 @@ Execute mew-dinfo-set before calling this."
 	  (mew-smtp-set-port pnm (mew-smtp-port case))
 	  (mew-smtp-set-ssh-server pnm ssh-server)
 	  (mew-smtp-set-queue pnm (mew-queue-folder case))
+	  (mew-smtp-set-fcc pnm fcc)
 	  (mew-current-get-window-config)
 	  (mew-redraw)
 	  (save-excursion
@@ -124,7 +126,7 @@ Execute mew-dinfo-set before calling this."
 		(mew-header-set mew-header-separator)
 		(mew-header-delete-lines mew-field-delete-common)
 		(mew-header-delete-lines (mew-tinfo-get-field-del)))
-	      (if (mew-encode pnm case nil nil 'header)
+	      (if (mew-encode pnm case resentp nil nil 'header)
 		  (cond
 		   ((eq action 'queue)
 		    (mew-smtp-queue pnm "from Draft mode"))
@@ -159,14 +161,16 @@ a text object to be saved can be specified."
    ;; need to make a cache or a message buffer.
    (mew-summary-display nil)
    (let* ((fld (mew-summary-folder-name))
-	  (msg (mew-summary-message-number))
+	  (msg (mew-summary-message-number2))
 	  (num (mew-syntax-number))
 	  (nums (mew-syntax-number-to-nums num))
-	  (cache (or (mew-cache-hit fld msg) (mew-buffer-message)))
+	  (cbuf (mew-cache-hit fld msg))
+	  (alt  (mew-cache-dinfo-get-use-alt cbuf))
+	  (cache (or cbuf (mew-buffer-message)))
 	  (syntax (mew-syntax-get-entry (mew-cache-decode-syntax cache) nums))
 	  (action "Save")
 	  (buf (generate-new-buffer mew-buffer-prefix))
-	  PLUS1P limit msg have-hdrp bodyp beg end cdpl file ct ctl
+	  PLUS1P limit have-hdrp bodyp beg end cdpl file ct ctl
 	  doit append-p)
      (save-excursion
        ;; Due to mew-decode-broken, the filename may be changed.
@@ -199,8 +203,6 @@ a text object to be saved can be specified."
 	       (setq have-hdrp t))))
        ;; Now, let's analyze the message in the burst buffer.
        ;; This is lengthy, though, avoidable.
-       (mew-summary-goto-message)
-       (setq msg (mew-summary-message-number))
        (setq limit (length nums))
        (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
 	   (setq limit (1+ limit)))
@@ -208,7 +210,7 @@ a text object to be saved can be specified."
        (set-buffer buf)
        (mew-erase-buffer)
        (mew-insert-message fld msg mew-cs-text-for-read nil)
-       (mew-dinfo-set limit 'no-cs-conv)
+       (mew-dinfo-set limit 'no-cs-conv t alt)
        (mew-decode-for-edit)
        ;;
        (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
@@ -233,7 +235,7 @@ a text object to be saved can be specified."
 	     (setq doit t)))
        ;;
        (if (not doit)
-	   (message "Didn't save anything.")
+	   (message "Did not save anything")
 	 (let (linebasep fromcs tocs charset)
 	   (save-restriction
 	     (narrow-to-region beg end)
@@ -263,7 +265,7 @@ a text object to be saved can be specified."
 	     (mew-frwlet
 	      mew-cs-dummy
 	      (if linebasep mew-cs-text-for-write mew-cs-binary)
-	      ;; don't specify 'no-msg
+	      ;; do not specify 'no-msg
 	      (write-region (point-min) (point-max) file append-p))))))
      (mew-remove-buffer buf))))
 
@@ -278,20 +280,22 @@ according to a specified language.
 If executed with '\\[universal-argument]', coding-system is asked."
   (interactive "P")
   (if (not mew-mule-p)
-      (message "This command can't be used on non-Mule.")
+      (message "This command cannot be used on non-Mule Emacs")
     (mew-summary-msg-or-part
      ;; need to make a cache or a message buffer.
      (mew-summary-display nil)
      (let* ((fld (mew-summary-folder-name))
-	    (msg (mew-summary-message-number))
+	    (msg (mew-summary-message-number2))
 	    (num (mew-syntax-number))
 	    (nums (mew-syntax-number-to-nums num))
-	    (cache (or (mew-cache-hit fld msg) (mew-buffer-message)))
+	    (cbuf (mew-cache-hit fld msg))
+	    (alt  (mew-cache-dinfo-get-use-alt cbuf))
+	    (cache (or cbuf (mew-buffer-message)))
 	    (syntax (mew-syntax-get-entry (mew-cache-decode-syntax cache) nums))
 	    (buf (generate-new-buffer mew-buffer-prefix))
 	    (win (selected-window))
 	    (orig-lang current-language-environment)
-	    PLUS1P limit msg bodyp hbeg hend beg end start
+	    PLUS1P limit bodyp hbeg hend beg end start
 	    lang prompt tocs cs-env)
        (save-excursion
 	 ;; First of all, we should determine which part the user want to
@@ -314,8 +318,6 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	     (setq PLUS1P (mew-syntax-get-privacy body))))
 	 ;; Now, let's analyze the message in the burst buffer.
 	 ;; This is lengthy, though, avoidable.
-	 (mew-summary-goto-message)
-	 (setq msg (mew-summary-message-number))
 	 (setq limit (length nums))
 	 (if (or PLUS1P bodyp);; VERY important for PGP/MIME
 	     (setq limit (1+ limit)))
@@ -332,9 +334,9 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	  (t
 	   (setq tocs mew-cs-autoconv)
 	   (setq prompt (format "Language (%s): " orig-lang))
-	   (setq lang (read-language-name nil prompt orig-lang))
+	   (setq lang (mew-input-language-name prompt orig-lang))
 	   (setq cs-env (mew-set-language-environment-coding-systems lang))))
-	 (mew-dinfo-set limit 'no-cs-conv)
+	 (mew-dinfo-set limit 'no-cs-conv t alt)
 	 (mew-decode-for-edit)
 	 ;;
 	 (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
@@ -359,6 +361,10 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (insert-buffer-substring buf beg end)
 	    (mew-cs-decode-region start (point-max) tocs)
 	    (mew-highlight-body-region start (point-max)))
+	  ;; Page breaks
+	  (when mew-break-pages
+	    (goto-char (point-min))
+	    (mew-message-narrow-to-page))
 	  (when (and (not askcs) cs-env)
 	    (mew-reset-coding-systems (car cs-env) (cdr cs-env))))
 	 (mew-summary-display-postscript 'no-hook)
@@ -377,14 +383,14 @@ If executed with '\\[universal-argument]', coding-system is asked."
       (set-buffer buf)
       (mew-erase-buffer)
       (mew-insert-message fld msg mew-cs-text-for-read nil)
-      (mew-dinfo-set 1 nil)
+      (mew-dinfo-set 1 nil nil nil) ;; xxx
       (mew-decode-for-edit)
       (setq multi (mew-syntax-get-part mew-decode-syntax))
       (cond
        ((not (mew-syntax-multipart-p multi))
-	(message "Can't burst"))
+	(message "Cannot burst"))
        ((not (mew-folder-check folder))
-	(setq errmsg (format "%s is wrong. Nothing was processed." folder)))
+	(setq errmsg (format "%s is wrong. Nothing was processed" folder)))
        ((not (setq mstr (mew-folder-new-message folder 'num-only)))
 	(setq errmsg (format "Error in %s. Nothing was processed" folder)))
        (t
@@ -430,15 +436,13 @@ If executed with '\\[universal-argument]', coding-system is asked."
   (interactive)
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
+	 (msg (mew-summary-message-number2))
 	 (folder (mew-input-burst-folder))
-	  msg ret)
-     (save-excursion
-       (mew-summary-goto-message)
-       (setq msg (mew-summary-message-number)))
-     (message "Bursting ... ")
+	 ret)
+     (message "Bursting...")
      (setq ret (mew-summary-burst-body fld msg folder))
      (when ret
-       (message "Bursting ... done")
+       (message "Bursting...done")
        (if (y-or-n-p (format "Go to %s? " folder))
 	   (mew-summary-goto-folder t folder))
        (message "Messages from %s to %s were extracted in %s"
@@ -450,11 +454,11 @@ If executed with '\\[universal-argument]', coding-system is asked."
   (mew-summary-multi-msgs
    (let ((folder (mew-input-burst-folder))
 	 (targets FLD-MSG-LIST))
-     (message "Bursting ... ")
+     (message "Bursting...")
      (while targets
        (mew-summary-burst-body (car (car targets)) (cdr (car targets)) folder)
        (setq targets (cdr targets)))
-     (message "Bursting ... done")
+     (message "Bursting...done")
      (if (y-or-n-p (format "Go to %s? " folder))
 	 (mew-summary-goto-folder t folder)))))
 
@@ -479,8 +483,8 @@ If executed with '\\[universal-argument]', coding-system is asked."
        (mew-insert-message fld msg mew-cs-text-for-read nil)
        (mew-set-buffer-multibyte t)
        (if (not (re-search-forward sep1 nil t))
-	   (message "Can't burst this message")
-	 (message "Bursting ... ")
+	   (message "Cannot burst this message")
+	 (message "Bursting...")
 	 (forward-line 2)
 	 (setq start (point))
 	 (setq mstr (mew-folder-new-message folder 'num-only))
@@ -502,7 +506,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	   (setq start (point))
 	   (setq m (1+ m))
 	   (setq mstr (int-to-string m)))
-	 (message "Bursting ... done")
+	 (message "Bursting...done")
 	 (mew-touch-folder folder)
 	 (setq done t)))
      (mew-remove-buffer buf)
@@ -533,13 +537,9 @@ See also mew-summary-edit-again."
   (interactive "P")
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
-	 (msg (mew-summary-message-number))
+	 (msg (mew-summary-message-number2))
 	 (part (mew-syntax-nums))
 	 draftname)
-     (unless msg
-       (save-excursion
-	 (mew-summary-goto-message)
-	 (setq msg (mew-summary-message-number))))
      (cond
       ((string= fld mew-draft-folder)
        (setq draftname (mew-draft-buffer-name (mew-expand-folder fld msg)))
@@ -559,14 +559,15 @@ See also mew-summary-edit-again."
       (t
        (if part
 	   (let* ((cache (mew-cache-hit fld msg 'must-hit))
+		  (alt (mew-cache-dinfo-get-use-alt cache))
 		  (syntax (mew-cache-decode-syntax cache))
 		  (stx (mew-syntax-get-entry syntax part))
 		  (ct (mew-syntax-get-value (mew-syntax-get-ct stx) 'cap)))
 	     (if (not (string= mew-ct-msg ct))
-		 (message "Can't reedit here.")
-	       (mew-summary-reedit-for-message fld msg part keep-hdr)
+		 (message "Cannot reedit here")
+	       (mew-summary-reedit-for-message fld msg part keep-hdr alt)
 	       (run-hooks 'mew-draft-mode-reedit-hook)))
-	 (mew-summary-reedit-for-message fld msg nil keep-hdr)
+	 (mew-summary-reedit-for-message fld msg nil keep-hdr nil)
 	 (run-hooks 'mew-draft-mode-reedit-hook)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -577,6 +578,7 @@ See also mew-summary-edit-again."
 (defsubst mew-summary-edit-header-common ()
   (mew-header-delete-lines (list mew-x-mailer:))
   (mew-header-goto-end)
+  ;; X-Mailer: must be the last
   (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
   (mew-header-clear) ;; erase the old header separator
   (mew-header-prepared))
@@ -625,6 +627,7 @@ Note that the message is not a valid MIME message."
    (mew-draft-find-and-switch (mew-expand-folder fld msg))
    (mew-summary-edit-header-for-draft)
    (mew-draft-mode)
+   (mew-draft-rehighlight-body)
    (save-excursion
      (when (and (mew-encode-load-syntax) mew-encode-syntax)
        (mew-encode-syntax-delete 'all)
@@ -638,7 +641,7 @@ The message is assumed to be a valid MIME message."
   (mew-window-configure 'draft)
   ;; main part
   (mew-summary-prepare-draft
-   (mew-summary-edit-message fld msg nil)
+   (mew-summary-edit-message fld msg nil nil)
    (let* ((file (mew-expand-folder fld msg))
 	  (work (concat file mew-queue-work-suffix))
 	  (info (concat file mew-queue-info-suffix))
@@ -652,12 +655,13 @@ The message is assumed to be a valid MIME message."
        (mew-tinfo-set-case case)
        (mew-summary-edit-header-for-queue hdr)))
    (mew-draft-mode)
+   (mew-draft-rehighlight-body)
    (when mew-encode-syntax
      (save-excursion
        (mew-draft-prepare-attachments t)))
    (mew-draft-mode-name)))
 
-(defun mew-summary-reedit-for-message (fld msg part keep-hdr)
+(defun mew-summary-reedit-for-message (fld msg part keep-hdr alt)
   "Edit a message in a folder other than the draft folder.
 The message is assumed to be a valid MIME message."
   (mew-current-set-window-config)
@@ -665,10 +669,11 @@ The message is assumed to be a valid MIME message."
   ;; main part
   (mew-summary-prepare-draft
    ;; case is copied here
-   (mew-summary-edit-message fld msg part)
+   (mew-summary-edit-message fld msg part alt)
    ;; get info before backup the original
    (mew-summary-edit-header-for-message keep-hdr)
    (mew-draft-mode)
+   (mew-draft-rehighlight-body)
    (when mew-encode-syntax
      (save-excursion
        (mew-draft-prepare-attachments t)))
@@ -680,7 +685,7 @@ The message is assumed to be a valid MIME message."
 ;;;
 
 (defvar mew-summary-edit-again-regex
-  "----- Original message follows -----\\|----- Unsent message follows -----\\|--- Undelivered message follows ---")
+  "----- Original message follows -----\\|----- Unsent message follows -----\\|--- Undelivered message follows ---\\|--- Below this line is a copy of the message.")
 
 (defun mew-summary-edit-again ()
   "Edit an old fashioned error message in which the original message
@@ -704,7 +709,7 @@ mew-summary-reedit."
        (mew-set-buffer-multibyte t)
        (goto-char (point-min))
        (if (not (re-search-forward mew-summary-edit-again-regex nil t))
-	   (message "Can't edit this message again.")
+	   (message "Cannot edit this message again")
 	 (forward-line)
 	 ;; skip blank lines
 	 (while (looking-at "^$") (forward-line))
@@ -714,7 +719,7 @@ mew-summary-reedit."
 	  mew-cs-dummy mew-cs-text-for-write
 	  (write-region (point-min) (point-max) nfile nil 'no-msg))
 	 ;; A new message is stored in the temporary directory
-	 (mew-summary-reedit-for-message nfld nmsg nil nil)
+	 (mew-summary-reedit-for-message nfld nmsg nil nil nil)
 	 (run-hooks 'mew-draft-mode-edit-again-hook)
 	 (if (file-exists-p nfile) (delete-file nfile))))
      (mew-remove-buffer buf))))
@@ -745,7 +750,7 @@ mew-summary-reedit."
 	  ret
 	nil))))
 
-(defun mew-summary-edit-message (fld msg part)
+(defun mew-summary-edit-message (fld msg part alt)
   "Prepare a header and a body, optionally attachments."
   (let* ((draft (mew-folder-new-message mew-draft-folder))
 	 (attachdir (mew-attachdir draft))
@@ -755,14 +760,14 @@ mew-summary-reedit."
     (mew-draft-find-and-switch draft)
     (mew-delete-directory-recursively attachdir)
     ;; De-compose the message in the burst buffer.
-    ;; Don't decode Message/Rfc822 parts. 
+    ;; do not decode Message/Rfc822 parts. 
     ;; They will be just a attachment message.
     (save-excursion
       ;; Let's decode the message without limitations
       (set-buffer buf)
       (mew-erase-buffer)
       (mew-insert-message fld msg mew-cs-text-for-read nil)
-      (mew-dinfo-set (1+ (length part)) 'no-cs-conv)
+      (mew-dinfo-set (1+ (length part)) 'no-cs-conv t alt)
       (mew-decode-for-edit)
       (setq syntax (mew-syntax-get-entry mew-decode-syntax part))
       ;; syntax is now for the message
