@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -110,11 +109,6 @@
 ;;; Header sanity check
 ;;;
 
-(defun mew-encode-sanity-check ()
-  (mew-encode-remove-illegal-fields)
-  (mew-encode-ask-subject)
-  (mew-encode-ask-newsgroups))
-
 (defun mew-encode-remove-illegal-fields ()
   (when (mew-header-end)
     (save-excursion
@@ -122,14 +116,14 @@
 	(goto-char (mew-header-end))
 	(if (not (bolp)) (insert "\n"))
 	(narrow-to-region (point-min) (mew-header-end))
-	(let (beg med)
+	(let (beg med str)
 	  (mew-elet
 	   ;; removing null lines
 	   (goto-char (point-min))
 	   (while (and (re-search-forward "^$" nil t)
 		       (not (eobp)))
 	     (delete-char 1))
-	   ;; removing fields which don't have value.
+	   ;; removing fields which do not have value.
 	   (goto-char (point-min))
 	   (while (not (eobp))
 	     (if (not (looking-at mew-keyval))
@@ -138,9 +132,11 @@
 	       (setq med (match-end 0))
 	       (forward-line)
 	       (mew-header-goto-next)
-	       (if (string-match "^[ \t]*$"
-				 (buffer-substring med (1- (point))))
-		   (delete-region beg (point)))))))
+	       (setq str (buffer-substring med (1- (point))))
+	       ;; str may consists of multiple lines
+	       ;; So, "$" does not work. We need to use "[^ ]".
+	       (unless (string-match "[^ \t\n]" str)
+		 (delete-region beg (point)))))))
 	(let ((fields mew-multiple-field-error-list))
 	  (while fields
 	    (if (> (mew-header-count-field (car fields)) 1)
@@ -151,17 +147,32 @@
 
 (defun mew-encode-ask-subject ()
   (when (and mew-ask-subject (not (mew-header-existp mew-subj:)))
-    ;; Subject: doesn't exist
+    ;; Subject: does not exist
     (let ((subj (read-string (concat mew-subj: " "))))
-      (save-excursion
-	(goto-char (point-min))
-	(insert (format "%s %s\n" mew-subj: subj))))))
+      (mew-header-replace-value mew-subj: subj))))
+
+(defun mew-encode-ask-fcc (resentp)
+  (let* ((target (if resentp mew-resent-fcc: mew-fcc:))
+	 (folders (mew-header-get-value target))
+	 folder ret)
+    (when folders
+      (setq folders (mew-addrstr-parse-value-list2 folders))
+      (while folders
+	(setq folder (mew-canonicalize-folder (car folders)))
+	(setq folders (cdr folders))
+	(when (and mew-ask-fcc
+		   (not (file-directory-p (mew-expand-folder folder)))
+		   (not (y-or-n-p (format "%s does not exist. Create it? " folder))))
+	  (error "Edit Fcc:"))
+	(setq ret (cons folder ret))))
+    (nreverse ret)))
 
 ;; xxx NetNews: ""?
 (defun mew-encode-ask-newsgroups ()
   (when (and mew-ask-newsgroups (mew-header-existp mew-newsgroups:))
     (unless (y-or-n-p "Do you want to post to NetNews? ")
-      (mew-header-delete-lines (list mew-newsgroups:)))))
+      (save-excursion
+	(mew-header-delete-lines (list mew-newsgroups:))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -195,15 +206,11 @@
 	    (setq vals (cdr vals))
 	    (setq ins nil addrs nil)
 	    (cond
-	     ((string-match "^\\([^:]+:\\)\\([^;]*\\);$" val)
-	      (setq prefix (mew-match 1 val))
-	      (setq addr (mew-match 2 val))
-	      (setq addr (mapcar (function mew-chop) (mew-split addr ?,)))
-	      (while addr
-		(setq addrs
-		      (nconc addrs (mew-addrstr-expand-alias (car addr))))
-		(setq addr (cdr addr)))
-	      (setq ins (list (concat prefix ";"))))
+	     ((string-match "[^:]+:[^;]+;" val)
+	      (let ((mew-alias-expand-prefix nil))
+		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
+		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(setq ins (list (concat mew-alias-expand-prefix ":;")))))
 	     ((and (setq addr (mew-addrstr-parse-address val))
 		   (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
 				 val))
@@ -213,11 +220,15 @@
 	      (setq addrs (list addr))
 	      (setq ins (list (concat prefix addr suffix))))
 	     (t
-	      (setq addrs (mew-addrstr-expand-alias val))
-	      (setq ins (copy-sequence addrs))))
+	      (let ((mew-alias-expand-prefix nil))
+		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
+		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(if mew-alias-expand-prefix
+		    (setq ins (list (concat mew-alias-expand-prefix ":;")))
+		  (setq ins (copy-sequence addrs))))))
 	    (setq insl (nconc insl ins))
-	    (if (not (string-match "bcc" key)) ;; Removing Bcc:
-		(setq ret (nconc ret addrs))))
+	    (unless (string-match "bcc" key) ;; Removing Bcc:
+	      (setq ret (nconc ret addrs))))
 	  (insert " " (mapconcat (function identity) insl ", "))
 	  (setq insl nil)
 	  (forward-line))))
@@ -225,7 +236,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Dcc:, Bcc:, Fcc:
+;;; Dcc:, Bcc:
 ;;;
 
 (defun mew-encode-delete-dcc (resentp)
@@ -243,20 +254,6 @@
     (mew-header-delete-lines (list mew-bcc: mew-resent-bcc:)) ;; anyway
     bcc))
 
-(defun mew-encode-check-fcc (resentp)
-  (let* ((target (if resentp mew-resent-fcc: mew-fcc:))
-	 (folders (mew-header-get-value target))
-	 folder ret)
-    (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
-    (when folders
-      (setq folders (mew-addrstr-parse-value-list2 folders))
-      (while folders
-	(setq folder (mew-canonicalize-folder (car folders)))
-	(setq folders (cdr folders))
-	(mew-folder-check folder 'force-to-create)
-	(setq ret (cons folder ret)))
-      (nreverse ret))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Message-Id:, Date:
@@ -401,32 +398,29 @@
 ;;; Encoding a message
 ;;;
 
-(defun mew-encode (pnm case &optional privacy signer headerp)
+(defun mew-encode (pnm case resentp &optional privacy signer headerp)
   (let ((ret t))
     (if (mew-debug 'encode)
 	(let ((debug-on-error t))
-	  (mew-encode-message pnm case privacy signer headerp))
+	  (mew-encode-message pnm case resentp privacy signer headerp))
       (condition-case nil
-	  (mew-encode-message pnm case privacy signer headerp)
+	  (mew-encode-message pnm case resentp privacy signer headerp)
 	(error
 	 (mew-encode-undo)
 	 (message (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
-(defun mew-encode-message (pnm case &optional privacy signer headerp)
+(defun mew-encode-message (pnm case resentp &optional privacy signer headerp)
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
   (run-hooks 'mew-make-message-hook)
-  (let (multip resentp recipients)
+  (let (multip recipients)
     (mew-smtp-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
-    ;; Check resentp
     (goto-char (point-min))
-    (setq resentp (re-search-forward "^Resent" (mew-header-end) t))
-    (goto-char (point-min))
-    (mew-encode-sanity-check)
+    (mew-encode-remove-illegal-fields)
     ;; Destination check
     (setq recipients (mew-encode-canonicalize-address resentp))
     ;; Bcc: is not included.
@@ -447,14 +441,14 @@
 	      ;; xxx error but ...
 	      )))
 	(setq mew-encode-syntax
-	      (mew-encode-syntax-single "text-file" (list mew-ct-txt))))
+	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
       (mew-encode-make-backup))
     ;; Errors can be caused from here.
     (unless recipients (mew-encode-error "No recipient!"))
     ;; Header modifications which are not remained.
+    (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
     (mew-smtp-set-dcc pnm (mew-encode-delete-dcc resentp))
     (mew-smtp-set-bcc pnm (mew-encode-delete-bcc resentp))
-    (mew-smtp-set-fcc pnm (mew-encode-check-fcc resentp))
     (mew-encode-id-date pnm case resentp)
     (mew-encode-from-sender pnm case resentp)
     (mew-encode-learn-aliases resentp)
@@ -468,7 +462,7 @@
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
     ;;
-    (message "Making a message ... ")
+    (message "Making a message...")
     ;; save syntax before setting privacy
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
@@ -479,7 +473,7 @@
     (mew-encode-make-header headerp)
     (mew-encode-save-draft)
     (mew-overlay-delete-buffer)
-    (message "Making a message ... done")))
+    (message "Making a message...done")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
 ;;;
@@ -538,8 +532,12 @@
 	(mew-frwlet
 	 (or cs mew-cs-autoconv) mew-cs-dummy
 	 (insert-file-contents file)))
-      (mew-charset-sanity-check beg (point-max))
-      (or charset (setq charset	(mew-charset-guess-region beg (point-max))))
+      ;; A user may specify charset to convery JIS X 0201 Katakana.
+      ;; So, we need to avoid the sanity check.
+      (unless charset
+	(if mew-use-charset-sanity-check
+	    (mew-charset-sanity-check beg (point-max)))
+	(setq charset (mew-charset-guess-region beg (point-max))))
       (setq cs (mew-charset-to-cs charset))
       (unless (mew-coding-system-p cs)
 	(mew-encode-error
@@ -594,7 +592,8 @@
 	(mew-frwlet
 	 (if linebasep mew-cs-text-for-read mew-cs-binary)
 	 mew-cs-dummy
-	 (insert-file-contents file)))
+	 (let (format-alist)
+	   (insert-file-contents file))))
       (when linebasep
 	(goto-char beg)
 	(while (search-forward "\n" nil t) (replace-match "\r\n" nil t)))
@@ -619,7 +618,7 @@
 	(if (and textp (file-exists-p file))
 	    (delete-file file))))
      (t
-      (mew-encode-error (concat mew-prog-mime-encode " doesn't exist"))))
+      (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
     (list (if charset (list "charset" charset)) cte)))
 
 (defun mew-encode-singlepart (syntax &optional path depth buffered)
@@ -684,7 +683,7 @@
       (if (not (eobp))
 	  ;; The cover page exists.
 	  (setq buffered t)
-	;; The cover page doesn't exist.
+	;; The cover page does not exist.
 	(delete-region beg (point-max))
 	;; Remove the cover page entry from the syntax.
 	(setq mew-encode-syntax
@@ -827,6 +826,8 @@
 	  (if (file-exists-p file2) (delete-file file2))
 	  (if (file-exists-p file3) (delete-file file3))
 	  (mew-tinfo-set-privacy-err t)
+	  (mew-tinfo-set-privacy-type nil)
+	  (mew-draft-mode-name)
 	  (mew-encode-error errmsg))
       ;; Create multipart content-header
       (mew-header-insert mew-ct: (list ct
@@ -871,6 +872,8 @@
 	  (if (file-exists-p file1) (delete-file file1))
 	  (if (file-exists-p file2) (delete-file file2))
 	  (mew-tinfo-set-privacy-err t)
+	  (mew-tinfo-set-privacy-type nil)
+	  (mew-draft-mode-name)
 	  (mew-encode-error errmsg))
       (goto-char (point-min))
       ;; Before the signed part
@@ -929,7 +932,7 @@
 		  file t nil opt))
 	  (if (file-exists-p file) (delete-file file))))
        (t
-	(mew-encode-error (concat mew-prog-mime-encode " doesn't exist")))))))
+	(mew-encode-error (concat mew-prog-mime-encode " does not exist")))))))
 
 (defun mew-convert-message ()
   ;; called on the beginning of a header
@@ -1116,10 +1119,10 @@
   (interactive)
   (mew-elet
    (if (not (mew-encode-load-syntax))
-       (message "Can't undo")
+       (message "Cannot undo")
      (mew-erase-buffer)
      (if (not (mew-encode-insert-backup))
-	 (message "Can't undo")
+	 (message "Cannot undo")
        (mew-header-clear) ;; erase the old header separator
        (mew-header-prepared (mew-tinfo-get-hdr-file))
        (if mew-encode-syntax (mew-draft-prepare-attachments))
