@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar  6, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -67,18 +66,8 @@ requires PTY.")
     (if (eq window-system 'x) (setq mew-icon-p t))
     (setq mew-gemacs-p t)
     (require 'mew-gemacs))
-  (cond
-   ((null mew-mule-p)
-    (setq mew-mule-ver 0)
-    (require 'mew-mule0))
-   (t
-    (cond
-     (mew-unibyte-p
-      (setq mew-mule-ver 0)
-      (require 'mew-mule0))
-     (t
-      (setq mew-mule-ver 3)
-      (require 'mew-mule3)))))))
+  (setq mew-mule-ver 3)
+  (require 'mew-mule3)))
 
 ;;;
 ;;; Emacs vs XEmacs
@@ -93,11 +82,6 @@ requires PTY.")
   ;;
   (defsubst mew-mark () (mark t))
   ;;
-  (defsubst mew-pop-to-buffer (buf)
-    (setq buf (get-buffer-create buf))
-    (select-window (display-buffer buf nil (selected-frame)))
-    (set-buffer buf))
-  ;;
   (add-hook 'pre-idle-hook 'mew-summary-cook-window)
   ;;
   (defalias 'mew-md5 'md5))
@@ -109,8 +93,6 @@ requires PTY.")
   ;;
   (defsubst mew-mark () (marker-position (mark-marker)))
   ;;
-  (defalias 'mew-pop-to-buffer 'pop-to-buffer)
-  ;;
   (require 'mew-md5)))
 
 ;;;
