@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Nov 13, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -113,7 +112,7 @@
 (defun mew-attach-external-body ()
   (interactive)
   (if (not (mew-attach-not-line012-1))
-      (message "Can't insert external-body here")
+      (message "Cannot insert external-body here")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
@@ -124,7 +123,7 @@
       ;; attachdir / file
       (setq filepath (mew-random-filename attachdir 1 nil mew-ext-suffix))
       (if (file-exists-p filepath)
-	  (message "Could not make a file for external-body, sorry.")
+	  (message "Cannot make a file for external-body, sorry")
 	(setq file (file-name-nondirectory filepath))
 	(setq ct (mew-ext-encode filepath))
 	(setq mew-encode-syntax
@@ -167,14 +166,14 @@
 	;; name is quoted
 	(if (string= name "")
 	    (setq ct mew-ct-apo)
-	  (setq ct (mew-ctdb-ct (mew-ctdb-by-file name))))))
+	  (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file name))
+		       (mew-content-type (mew-tinfo-get-case)))))))
       (setq ct (mew-input-type "Type for %s (%s): " name ct
 			       mew-mime-content-type-list))
       (mew-header-insert mew-ct: ct)
       (mew-header-insert mew-cid: (mew-create-content-id))
       (insert "\n")
-      (if (not (string= access-type "mail-server"))
-	  () ;; message-body is not necessary
+      (when (string= access-type "mail-server")
 	;;message-body
 	(insert (read-string "Input message to the mail-server: "))
 	(insert "\n"))
@@ -340,13 +339,13 @@
 	(if (y-or-n-p (format "%s exists. Overwrite? " lfname))
 	    (delete-file localfile)
 	  (setq getit nil)
-	  (message "The file wasn't retrieved")))
+	  (message "The file was not retrieved")))
     (if getit (mew-ext-copy-file-internal remotefile localfile nil))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun mew-ext-tftp (cache begin end &optional params)
-  (message "access-type TFTP is not supported yet"))
+  (message "Access-type TFTP is not supported yet"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -402,7 +401,7 @@
 	(if (y-or-n-p (format "%s exists. Overwrite? " lfname))
 	    (delete-file localfile)
 	  (setq getit nil)
-	  (message "The file wasn't retrieved")))
+	  (message "The file was not retrieved")))
     (when getit
       (cond
        ((eq mew-ext-anon-ftp-method 'ftp)
@@ -504,12 +503,12 @@
       (funcall prog args url))
      ((stringp prog)
       (setq args (append args (list url)))
-      (message "Running command: %s"
+      (message "Executing %s..."
 	       (mapconcat (function append) (append (list prog) args) " "))
       (save-excursion ;; xxx this is necessary in other places
 	(setq pnm (format "*Mew %s*" prog))
 	(apply (function start-process) pnm nil prog args)))
-     (t (message "Program list error.")))))
+     (t (message "Program list error")))))
 
 (defun mew-ext-url-show-by-w3 (dummy url)
   (require 'w3)
