@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -25,6 +24,9 @@
 (defsubst mew-draft-p ()
   (eq major-mode 'mew-draft-mode))
 
+(defsubst mew-draft-or-header-p ()
+  (memq major-mode '(mew-draft-mode mew-header-mode)))
+
 (defsubst mew-thread-p ()
   (mew-vinfo-get-thread-p))
 
@@ -75,7 +77,7 @@ IGNORE-CASE is t, matching is performed by ignoring case."
 
 (defun mew-uniq-list (lst)
   "Destructively uniqfy elements of LST.
-This is O(N^2). So, don't use this function with a large LST."
+This is O(N^2). So, do not use this function with a large LST."
   (let ((tmp lst))
     (while tmp (setq tmp (setcdr tmp (delete (car tmp) (cdr tmp))))))
   lst)
@@ -327,16 +329,18 @@ Words are separated by '/' and '-'."
 (defsubst mew-insert-message (fld msg rcs size)
   (let ((file (mew-expand-folder fld msg)))
     (cond
-     ((or (mew-folder-newsp fld) (mew-folder-imapp fld))
-      (error "%s isn't supported yet" fld))
      ((file-readable-p file)
-      (mew-frwlet
-       rcs mew-cs-dummy
-       (insert-file-contents file nil 0 size))
+      (let ((old-cs (and (boundp 'buffer-file-coding-system)
+			 buffer-file-coding-system)))
+	(mew-frwlet
+	 rcs mew-cs-dummy
+	 (insert-file-contents file nil 0 size))
+	(if (boundp 'buffer-file-coding-system)
+	    (setq buffer-file-coding-system old-cs)))
       ;; return physical size
       (cons (mew-file-get-time file) (mew-file-get-size file)))
      (t
-      (error "%s%s doesn't exist" (file-name-as-directory fld) msg)))))
+      (error "%s%s does not exist" (file-name-as-directory fld) msg)))))
 
 (defsubst mew-insert-manual (&rest args)
   (insert (substitute-command-keys (apply (function concat) args))))
@@ -438,22 +442,21 @@ also ignored."
 (defsubst mew-folder-localp (folder)
   (or (mew-folder-mailp folder) (mew-folder-local-newsp folder)))
 
-(defsubst mew-folder-remotep (folder)
-  (or (mew-folder-newsp folder) (mew-folder-imapp folder)))
-
 (defsubst mew-folder-inboxp (folder)
   (member folder mew-inbox-folders))
 
 (defsubst mew-folder-queuep (folder)
   (member folder mew-queue-folders))
 
+(defsubst mew-folder-mdropp (folder)
+  (member folder mew-mdrop-folders))
+
 (defsubst mew-folder-draftp (folder)
   (equal folder mew-draft-folder))
 
 (defun mew-canonicalize-folder (folder)
   (if (or (mew-folder-mailp folder)
 	  (mew-folder-local-newsp folder)
-	  (mew-folder-imapp folder)
 	  (file-name-absolute-p folder))
       folder
     (concat "+" folder)))
@@ -466,8 +469,6 @@ also ignored."
       (setq dir (expand-file-name subdir mew-mail-path)))
      ((mew-folder-local-newsp folder)
       (setq dir (expand-file-name subdir mew-news-path)))
-     ((mew-folder-imapp folder)
-      ()) ;; IMAP
      ((file-name-absolute-p folder)
       (setq dir (expand-file-name folder))))
     (if message
@@ -477,33 +478,30 @@ also ignored."
 (defun mew-folder-check (folder &optional force-to-create)
   "A function to see if FOLDER exists.
 Return t if exists or created. Otherwise, return nil."
-  (if (not (stringp folder))
-      nil ;; wrong
+  (when (stringp folder)
     (let ((absdir (mew-expand-folder folder))  ;; /home/Mail/foo
 	  (create-it force-to-create))
-      (if (file-exists-p absdir)
-	  (if (file-directory-p absdir)
-	      (progn
-		;; The folder doesn't exist in mew-folder-list if
-		;; a user creates it by hand...
-		(mew-folder-insert folder) ;; just in case
-		t) ;; exists
-	    (message "%s is a file" folder)
-	    nil) ;; xxx exists but a file
-	(unless create-it
-	  (if (mew-folder-imapp folder)
-	      (if (y-or-n-p (format "Maybe %s doesn't exist (cache-dir not found). Create it? " folder))
-		  (setq create-it t))
-	    (if (y-or-n-p (format "%s doesn't exist. Create it? " folder))
-		(setq create-it t))))
-	(if (not create-it)
-	    nil ;; not created
-	  (mew-make-directory absdir)
-	  (mew-folder-insert folder)
-	  (mew-lisp-save mew-folder-list-file mew-folder-list)
-	  (mew-lisp-save mew-folder-alist-file mew-folder-alist)
-	  (message "%s has been created" folder)
-	  t))))) ;; created
+      (when absdir
+	(if (file-exists-p absdir)
+	    (if (file-directory-p absdir)
+		(progn
+		  ;; The folder does not exist in mew-folder-list if
+		  ;; a user creates it by hand...
+		  (mew-folder-insert folder) ;; just in case
+		  t) ;; exists
+	      (message "%s is a file" folder)
+	      nil) ;; xxx exists but a file
+	  (unless create-it
+	    (if (y-or-n-p (format "%s does not exist. Create it? " folder))
+		(setq create-it t)))
+	  (if (not create-it)
+	      nil ;; not created
+	    (mew-make-directory absdir)
+	    (mew-folder-insert folder)
+	    (mew-lisp-save mew-folder-list-file mew-folder-list)
+	    (mew-lisp-save mew-folder-alist-file mew-folder-alist)
+	    (message "%s has been created" folder)
+	    t)))))) ;; created
 
 (defsubst mew-dir-messages (dir)
   ;; (default-file-name-coding-system nil)
@@ -527,7 +525,7 @@ Return t if exists or created. Otherwise, return nil."
       (setq maxfile (int-to-string max))
       (setq maxpath (mew-expand-folder folder maxfile))
       (when (file-exists-p maxpath)
-	;; If NFS is used, readdir() may fail. Emacs 20 doesn't
+	;; If NFS is used, readdir() may fail. Emacs 20 does not
 	;; rescan the directory again. So, we need to rescan in
 	;; the Emacs level.
 	(mew-touch-folder folder);; can clear cache?
@@ -556,11 +554,12 @@ Return t if exists or created. Otherwise, return nil."
 
 (defun mew-touch-folder (fld)
   (let (file)
-    (if (and mew-touch-folder-p
-	     (stringp mew-summary-touch-file)
-	     (setq file (mew-expand-folder fld mew-summary-touch-file))
-	     (file-writable-p file))
-	(write-region "touched by Mew." nil file nil 'no-msg))))
+    (when (and mew-touch-folder-p
+	       (stringp mew-summary-touch-file)
+	       (setq file (mew-expand-folder fld mew-summary-touch-file))
+	       (file-writable-p file))
+      (write-region "touched by Mew." nil file nil 'no-msg)
+      (set-file-modes file mew-file-mode))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -996,7 +995,8 @@ by side-effect."
 	  (pp lisp tmp-buf)
 	  (mew-frwlet
 	   mew-cs-dummy mew-cs-m17n
-	   (write-region (point-min) (point-max) fullname nil 'no-msg))))
+	   (write-region (point-min) (point-max) fullname nil 'no-msg))
+	  (set-file-modes fullname mew-file-mode)))
     (mew-remove-buffer tmp-buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1127,7 +1127,7 @@ by side-effect."
 (defsubst mew-time-ctz-to-rfc (time)
   (let ((system-time-locale "C")
 	(tmzn-int (mew-time-tmzn-int)))
-    ;; XEmacs doesn't have %z
+    ;; XEmacs does not have %z
     (format (format-time-string "%a, %d %b %Y %T %%s (%Z)" time) tmzn-int)))
 
 ;; 2000/07/12 16:22:30
@@ -1162,11 +1162,11 @@ by side-effect."
 
 (defsubst mew-set-buffer-multibyte (arg)
   (if (fboundp 'set-buffer-multibyte)
-      (if mew-unibyte-p () (set-buffer-multibyte arg))))
+      (set-buffer-multibyte arg)))
 
-(defvar mew-unibyte-p 
-  (and (boundp 'enable-multibyte-characters)
-       (null enable-multibyte-characters)))
+(defsubst mew-multibyte-string-p (str)
+  (if (fboundp 'multibyte-string-p)
+      (multibyte-string-p str)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
@@ -1190,7 +1190,7 @@ by side-effect."
 
 (defmacro mew-rendezvous (who)
   ;; Wait for the termination of WHO.
-  ;; Emacs doesn't provide synchronize mechanism with
+  ;; Emacs does not provide synchronize mechanism with
   ;; an asynchronous process. So, take this way. 
   `(while ,who
      (if mew-xemacs-p
