@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -124,7 +123,9 @@ Otherwise, return 't'."
 	 (part (mew-current-get-part (mew-frame-id))))
     (if (not (get-buffer fld))
 	(message "No Summary mode for %s" fld)
-      (mew-pop-to-buffer fld)
+      (if (get-buffer-window fld)
+	  (select-window (get-buffer-window fld))
+	(mew-summary-switch-to-folder fld))
       (cond
        ((and msg part (mew-decode-syntax-p))
 	(setq part (concat "^.. +"
@@ -169,15 +170,14 @@ confused. Please use '\\[mew-message-forward]' instead."
 (defun mew-message-next-msg (&optional arg)
   "Display a message or a part below."
   (interactive "p")
-  (let* ((obuf (current-buffer))
-	 (buf (window-buffer (previous-window))))
-    (mew-pop-to-buffer buf) ;; for the next forward-line
-    (if (not (mew-summary-or-virtual-p))
-	()
+  (let* ((swin (previous-window))
+	 (mbuf (current-buffer)))
+    (select-window swin) ;; for the next forward-line
+    (when (mew-summary-or-virtual-p)
       (forward-line arg) ;; minus arg results in prev
       (mew-summary-display 'redisplay))
     ;; for window config
-    (mew-pop-to-buffer obuf)))
+    (select-window (get-buffer-window mbuf))))
       
 (defun mew-message-prev-msg (&optional arg)
   "Display a message or a part above."
