@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -79,26 +78,31 @@ at completions."
 	(directory-file-name default)
       (directory-file-name folder)))) ;; +foo/ -> +foo
 
-(defun mew-input-folders (default &optional prompt)
-  (let (form folders)
-    (if prompt
-	(progn
-	  (setq form "Folder name : ")
-	  (setq default prompt))
-      (setq form (format "Folder name (%s): " default))
-      (setq prompt "+"))
+(defun mew-input-folders (default)
+  (let (prompt init folders)
+    (cond
+     ((consp default)
+      (setq prompt "Folder name: ")
+      (setq init (mew-join "," default)))
+     (t
+      (setq prompt (format "Folder name (%s): " default))
+      (setq init "+")))
     (mew-input-clear)
     (setq mew-input-complete-function (function mew-complete-folder))
-    (setq folders (read-from-minibuffer form
-					prompt
+    (setq folders (read-from-minibuffer prompt
+					init
 					mew-input-map
 					nil
 					'mew-input-folder-hist))
-    (if (or (string= folders "") (string= folders "+"))
-	(setq folders default))
-    (mapcar (function directory-file-name) ;; +foo/ -> +foo
-	    (mapcar (function mew-chop)
-		    (mew-split folders ?,)))))
+    (setq folders (mapcar (function mew-chop) (mew-split folders ?,)))
+    (when folders
+      (setq folders (delete "+" folders)))
+    (when (and (null folders) (stringp default))
+      (setq folders (list default)))
+    (if folders
+	;; +foo/ -> +foo
+	(mapcar (function directory-file-name) folders)
+      nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -110,16 +114,23 @@ at completions."
 (defun mew-input-address (prompt &optional default)
   (mew-input-clear)
   (setq mew-input-complete-function (function mew-complete-address))
-  (let (tmp)
-    (setq tmp (read-from-minibuffer 
+  (let (val vals addrs ret)
+    (setq val (read-from-minibuffer 
 	       (if default (format prompt default) prompt)
 	       ""
 	       mew-input-map
 	       nil
 	       'mew-input-address-hist))
-    (if (and default (string= tmp ""))
-	(setq tmp default))
-    (mew-addrstr-canonicalize-address tmp)))
+    (if (and default (string= val ""))
+	(setq val default))
+    (setq vals (mapcar (function mew-chop)  (mew-split-quoted val ?,)))
+    (while vals
+      (setq val (car vals))
+      (setq vals (cdr vals))
+      (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
+      (setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+      (setq ret (nconc ret addrs)))
+    ret))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -200,7 +211,9 @@ at completions."
 
 (defun mew-input-range (folder askp)
   (let* (default comp range ret)
-    (if (or (mew-folder-draftp folder) (mew-folder-queuep folder))
+    (if (or (mew-folder-draftp folder)
+	    (mew-folder-queuep folder)
+	    (mew-folder-mdropp folder))
 	;; folders whose messages are dynamically removed.
 	(setq default mew-range-str-all)
       (setq default mew-range-str-update))
@@ -235,6 +248,34 @@ at completions."
     (mew-decode-syntax-delete)
     ret))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Buffer
+;;;
+
+(defun mew-input-buffer (default)
+  (let* ((comp (mapcar (lambda (x) (list (buffer-name x))) (buffer-list)))
+	 (buf (completing-read (format "Buffer (%s): " default) comp)))
+    (if (string= buf "")
+	default
+      buf)))
+
+(defun mew-input-draft-buffer (default)
+  (let ((regex (mew-folder-regex (file-name-as-directory mew-draft-folder)))
+	comp buf)
+    (setq comp (delq nil (mapcar
+			  (lambda (x)
+			    (when (string-match regex (buffer-name x))
+			      (list (buffer-name x))))
+			  (buffer-list))))
+    (if (and (= (length comp) 1)
+	     (string= default (car (car comp))))
+	default
+      (setq buf (completing-read (format "Buffer (%s): " default) comp))
+      (if (string= buf "")
+	  default
+	buf))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; File
@@ -482,6 +523,24 @@ at completions."
       ;; the password process will safely fail.
       (error ""))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Language name
+;;;
+
+;; read-language-name should be used. Unfortunately, that of
+;; XEmacs 21.1.14 is broken. This function should be obsoleted
+;; when Mew quit support of XEmacs 21.1.14
+
+(defun mew-input-language-name (prompt &optional default) 
+  (let* ((completion-ignore-case t) 
+         (name (completing-read prompt 
+                                language-info-alist 
+                                nil 
+                                t nil nil))) 
+    (if (string= name "") (setq name default)) 
+    name)) 
+
 (provide 'mew-minibuf)
 
 ;;; Copyright Notice:
