@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -130,6 +129,7 @@
 ;;(defun mew-cs-decode-string (str cs)
 ;;  (if cs (decode-coding-string str cs) str))
 
+;; This code should be obsoleted when Mule 4.1 goes away.
 (defun mew-cs-decode-string (str cs)
   (if (null cs)
       str
@@ -137,15 +137,18 @@
 	  ret buf)
       (if (null post-conv)
 	  (decode-coding-string str cs)
-	;; Emacs applys post-conv just to a buffer, not to a string.
+	;; Mule 4.1 (Emacs 20.7 + Mule-UCS-0.84) has a bug.
+	;; It applys post-conv just to a buffer, not to a string.
 	;; So, we should use a buffer.
 	(setq buf (generate-new-buffer mew-buffer-prefix))
 	(save-excursion
 	  (set-buffer buf)
 	  (mew-erase-buffer)
-	  (mew-set-buffer-multibyte nil)
+	  ;; The old base64-decode-string() returns a multibyte string.
+	  ;; The new base64-decode-string() returns a unibyte string.
+	  (mew-set-buffer-multibyte (mew-multibyte-string-p str))
 	  (insert str)
-	  (mew-set-buffer-multibyte t)
+	  (or (mew-multibyte-string-p str) (mew-set-buffer-multibyte t))
 	  (decode-coding-region (point-min) (point-max) cs)
 	  (setq ret (buffer-substring (point-min) (point-max))))
 	(mew-remove-buffer buf)
@@ -226,7 +229,10 @@
 	    (if cnt
 		(concat (substring str 0 i) (make-string safe-w 32) "..")
 	      (concat (substring str 0 i) (make-string safe-w 32)))))))
-      (error mew-error-broken-string))))
+      (error
+       (if (> (length mew-error-broken-string) width)
+	   (substring mew-error-broken-string 0 width)
+	 mew-error-broken-string)))))
     
 ;;
 ;; Language specific
