@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Jun 28, 2000
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -14,10 +13,15 @@
 ;;;
 
 (defvar mew-pop-info-list
-  '("status" "cnt" "inbox" "messages" "auth" "server" "ssh-process"
-    "try" "key" "user" "size" "delete" "top" "lmsg" "uidl"
-    "port" "sul" "body-lines" "case" "directive" "diag"
-    "string" "keep-str" "passwd"))
+  '("case"
+    "user" "server" "port"
+    "process" "ssh-process"
+    "directive" "flush" "skip-list" "save-uidl"
+    "status" "cnt" "left" "input" "keep-input" "diag"
+    "auth" "key" "passwd" "try"
+    "rtrs" "dels" "refs" "rmvs" "uidl"
+    "size" "body-lines" "truncated" "delete"
+    "fld" "msgdb"))
 
 (mew-info-defun "mew-pop-" mew-pop-info-list)
 
@@ -28,10 +32,18 @@
 
 (defconst mew-pop-info-prefix "mew-pop-info-")
 
-(defsubst mew-pop-info-name (server &optional sshsrv)
-  (if sshsrv
-      (concat mew-pop-info-prefix sshsrv "-" server)
-    (concat mew-pop-info-prefix server)))
+(defsubst mew-pop-info-name (case)
+  (let ((server (mew-pop-server case))
+	(port (mew-pop-port case))
+	(user (mew-pop-user case))
+	(sshsrv (mew-pop-ssh-server case))
+	(name mew-pop-info-prefix))
+    (setq name (concat name user "@" server))
+    (unless (string= port mew-pop-port)
+      (setq name (concat name ":" port)))
+    (if sshsrv
+	(concat name "%" sshsrv)
+      name)))
 
 (defsubst mew-pop-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
@@ -50,7 +62,8 @@
     ("user"         nil ("+OK" . "pass") ("-ERR" . "wpwd2"))
     ("pass"         nil ("+OK" . "list") ("-ERR" . "wpwd"))
     ("list"         t   ("+OK" . "uidl"))
-    ("uidl"         t   ("+OK" . "umsg") ("-ERR" . "retr"))
+    ("uidl"         t   ("+OK" . "umsg") ("-ERR" . "pre-retr"))
+    ("dels"	    nil ("+OK" . "dels"))
     ("retr"         t   ("+OK" . "dele"))
     ("dele"         nil ("+OK" . "retr"))
     ("quit"         nil ("+OK" . "noop"))))
@@ -72,57 +85,72 @@
 (defsubst mew-pop-message (pnm &rest args)
   (if (mew-pop-get-diag pnm) (apply (function message) args)))
 
-(defun mew-pop-retrieve (case &optional sul rev-del fld directive passwd)
+(defun mew-pop-retrieve (case directive &rest args)
   ;; in +inbox
   (let* ((server (mew-pop-server case))
 	 (port (mew-pop-port case))
 	 (sshsrv (mew-pop-ssh-server case))
-	 (pnm (mew-pop-info-name server sshsrv))
+	 (pnm (mew-pop-info-name case))
 	 (ret t)
 	 (buf (get-buffer-create (mew-pop-buffer-name pnm)))
+	 (diag (not (eq directive 'biff)))
 	 process sshpro sshname lport)
-    (mew-info-clean-up pnm)
-    (mew-pop-set-diag pnm (not (eq directive 'biff)))
-    (if (null sshsrv)
-	(progn
-	  (mew-pop-set-port pnm port)
-	  (setq process (mew-pop-open pnm server port)))
-      (mew-pop-set-port pnm port)
-      (setq sshpro (mew-open-ssh-stream server port sshsrv))
-      (mew-pop-set-ssh-process pnm sshpro)
-      (when sshpro
-	(setq sshname (process-name sshpro))
-	(setq lport (mew-ssh-pnm-to-lport sshname))
-	(if lport (setq process (mew-pop-open pnm "localhost" lport)))))
-    (cond
-     (process
-      (mew-pop-message pnm "Communicating with the POP server ... ")
-      (setq mew-summary-buffer-process process)
-      (mew-pop-set-status pnm "grtg")
-      (mew-pop-set-cnt pnm 0)
-      (mew-pop-set-try pnm 0)
-      (mew-pop-set-keep-str pnm t)
-      (mew-pop-set-server pnm server)
-      (mew-pop-set-sul pnm sul)
-      (mew-pop-set-directive pnm directive)
-      (mew-pop-set-passwd pnm passwd)
-      (mew-pop-set-case pnm case)
-      (mew-pop-set-inbox pnm (or fld (mew-inbox-folder case)))
-      (mew-pop-set-auth pnm (mew-pop-auth case))
-      (mew-pop-set-user pnm (mew-pop-user case))
-      (mew-pop-set-size pnm (mew-pop-size case))
-      (mew-pop-set-body-lines pnm (mew-pop-body-lines case))
-      (if rev-del
-	  (mew-pop-set-delete pnm (not (mew-pop-delete case)))
-	(mew-pop-set-delete pnm (mew-pop-delete case)))
-      ;;
-      (set-process-sentinel process 'mew-pop-sentinel)
-      (set-process-filter process 'mew-pop-filter)
-      (set-process-buffer process buf)
-      (mew-sinfo-set-scan-id nil))
-     (t
-      (mew-pop-tear-down pnm)))
-    ret))
+    (if (mew-pop-get-process pnm)
+	(message "Another POP process is running. Try later")
+      (if (null sshsrv)
+	  (setq process (mew-pop-open pnm server port diag))
+	(setq sshpro (mew-open-ssh-stream server port sshsrv))
+	(when sshpro
+	  (setq sshname (process-name sshpro))
+	  (setq lport (mew-ssh-pnm-to-lport sshname))
+	  (if lport (setq process (mew-pop-open pnm "localhost" lport diag)))))
+      (cond
+       (process
+	(mew-pop-message pnm "Communicating with the POP server...")
+	(mew-summary-lock process "POPing")
+	(mew-info-clean-up pnm)
+	(mew-pop-set-process pnm process)
+	(mew-pop-set-port pnm port)
+	(mew-pop-set-ssh-process pnm sshpro)
+	(mew-pop-set-diag pnm diag)
+	(mew-pop-set-status pnm "grtg")
+	(mew-pop-set-case pnm case)
+	(mew-pop-set-directive pnm directive)
+	(mew-pop-set-cnt pnm 0)
+	(mew-pop-set-try pnm 0)
+	(mew-pop-set-keep-input pnm t)
+	(mew-pop-set-server pnm server)
+	(mew-pop-set-auth pnm (mew-pop-auth case))
+	(mew-pop-set-user pnm (mew-pop-user case))
+	(mew-pop-set-fld pnm (mew-inbox-folder case))
+	(mew-pop-set-size pnm (mew-pop-size case))
+	(mew-pop-set-body-lines pnm (mew-pop-body-lines case))
+	(cond
+	 ((eq directive 'biff)
+	  (mew-pop-set-passwd pnm (nth 0 args))) ;; password
+	 ((eq directive 'inc)
+	  (mew-pop-set-flush pnm (nth 0 args)) ;; no-flush
+	  (mew-pop-set-save-uidl pnm t)
+	  (mew-pop-set-delete pnm (mew-pop-delete case)))
+	 ((eq directive 'get)
+	  (mew-pop-set-fld pnm (nth 0 args))
+	  (mew-pop-set-refs pnm (nth 1 args))
+	  (mew-pop-set-skip-list pnm t)
+	  (mew-pop-set-delete pnm (mew-pop-delete case)))
+	 ((eq directive 'scan)
+	  (mew-pop-set-fld pnm (nth 0 args))
+	  (mew-sinfo-set-mdrop-case case))
+	 ((eq directive 'mdrop)
+	  (mew-pop-set-fld pnm (nth 0 args))
+	  (mew-pop-set-refs pnm (nth 1 args))
+	  (mew-pop-set-rmvs pnm (nth 2 args))
+	  (mew-pop-set-skip-list pnm t)))
+	;;
+	(set-process-sentinel process 'mew-pop-sentinel)
+	(set-process-filter process 'mew-pop-filter)
+	(set-process-buffer process buf)
+	(mew-sinfo-set-scan-id nil)))
+      ret)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -143,7 +171,7 @@
 		 (and mew-use-cached-passwd
 		      (setq passwd (mew-passwd-get-passwd tag))))
 	(mew-sinfo-set-scan-form (mew-summary-scan-form inbox))
-	(mew-pop-retrieve case nil nil nil 'biff passwd)))))
+	(mew-pop-retrieve case 'biff passwd)))))
 
 (defun mew-pop-check ()
   (interactive)
@@ -153,7 +181,7 @@
       (set-buffer inbox)
       (when (mew-summary-exclusive-p)
 	(mew-sinfo-set-scan-form (mew-summary-scan-form inbox))
-	(mew-pop-retrieve case nil nil nil 'biff)))))
+	(mew-pop-retrieve case 'biff)))))
 
 (defun mew-pop-biff-setup ()
   (if (not mew-use-biff)
@@ -162,8 +190,7 @@
     (setq mew-pop-biff-timer-id
 	  (mew-timer (* 60 mew-pop-biff-interval) (function mew-pop-biff))))
   (let ((ent '(mew-biff-string mew-biff-string)))
-    (if (member ent global-mode-string)
-	()
+    (unless (member ent global-mode-string)
       (if global-mode-string
 	  (setq global-mode-string
 		(append global-mode-string (list " " ent)))
@@ -185,22 +212,22 @@
 ;;; Opening POP
 ;;;
 
-(defun mew-pop-open (pnm server port)
+(defun mew-pop-open (pnm server port diag)
   (let ((sprt (mew-port-sanity-check port))
 	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (mew-timer mew-pop-timeout-time 'mew-pop-timeout))
-	  (mew-pop-message pnm "Connecting to the POP server ... ")
+	  (if diag (message "Connecting to the POP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
-	  (mew-pop-message pnm "Connecting to the POP server ... done"))
+	  (if diag (message "Connecting to the POP server...done")))
       (quit
-       (mew-pop-message pnm "Can't connect to the POP server due to time out")
+       (if diag (message "Can't connect to the POP server due to time out"))
        (setq pro nil))
       (error
-       (mew-pop-message pnm "%s, %s" (nth 1 emsg) (nth 2 emsg))
+       (if diag (message "%s, %s" (nth 1 emsg) (nth 2 emsg)))
        (setq pro nil)))
     (if tm (cancel-timer tm))
     pro))
@@ -227,15 +254,15 @@
 (defun mew-pop-tear-down (pnm)
   (let ((process (get-process pnm))
 	(sshpro (mew-pop-get-ssh-process pnm))
-	(inbox (mew-pop-get-inbox pnm)))
+	(fld (mew-pop-get-fld pnm)))
+    ;; (mew-pop-set-process pnm nil)
     (mew-info-clean-up pnm)
     (if (processp process) (delete-process process))
     (if (processp sshpro) (delete-process sshpro))
-    (when inbox
-      (mew-set-summary-buffer-process-status inbox nil)
+    (when (and fld (get-buffer fld))
       (save-excursion
-	(set-buffer inbox)
-	(setq mew-summary-buffer-process nil)))))
+	(set-buffer fld)
+	(mew-summary-unlock)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -248,34 +275,29 @@
       (set-buffer (get-buffer-create mew-buffer-debug))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
-(defun mew-set-summary-buffer-process-status (inbox msg)
-  "Add informational string MSG to summary buffer process status of Mew.
-Usually displayed as \" Running\" in minor mode indicator area.
-If string argument MSG is specified, change as \" Running:...\".
-If MSG is nil, restore usual format."
-  (save-excursion
-    (set-buffer inbox)
-    (setq mew-summary-buffer-process-status
-	  (if (and msg (stringp msg))
-	      (concat " Running:" msg)
-	    " Running"))))
-
 (defun mew-pop-filter (process string)
   (let* ((pnm (process-name process))
 	 (status (mew-pop-get-status pnm))
 	 (cnt (mew-pop-get-cnt pnm))
-	 (msgs (mew-pop-get-messages pnm))
+	 (msgs (mew-pop-get-rtrs pnm))
 	 (msg (car msgs))
-	 (siz (nth 1 msg))
-	 (inbox (mew-pop-get-inbox pnm))
+	 (siz (nth 2 msg))
+	 (fld (mew-pop-get-fld pnm))
 	 (mulrep (mew-pop-fsm-reply status))
 	 stay next func progress total)
     (mew-pop-debug (upcase status) string)
     (mew-filter
      ;; This code may create a long string. So, take care.
-     (if (mew-pop-get-keep-str pnm)
-	 (mew-pop-set-string pnm (concat (mew-pop-get-string pnm) string)))
-     (when (string= status "retr")
+     (if (mew-pop-get-keep-input pnm)
+	 (mew-pop-set-input pnm (concat (mew-pop-get-input pnm) string)))
+     (cond
+      ((string= status "grtg")
+       (mew-summary-set-process-status fld "Auth'ing"))
+      ((string= status "list")
+       (mew-summary-set-process-status fld "Checking"))
+      ((string= status "dels")
+       (mew-summary-set-process-status fld "Deleting"))
+      ((string= status "retr")
        (setq total (+ (length msgs) cnt))
        (if (>= cnt total)
  	   (setq progress nil)
@@ -285,7 +307,8 @@ If MSG is nil, restore usual format."
   				(if (< 10000 siz)
   				    (/ (buffer-size) (/ siz 100))
   				  (/ (* (buffer-size) 100) siz)))))
-       (mew-set-summary-buffer-process-status inbox progress))
+       (mew-summary-set-process-status fld "Receiving" progress)))
+     ;; xxx
      ;; Process's buffer
      (goto-char (point-max))
      (mew-set-buffer-multibyte nil)
@@ -313,15 +336,29 @@ If MSG is nil, restore usual format."
        (if (fboundp func)
 	   (and func (funcall func process pnm))
 	 (error "No function called %s" (symbol-name func)))
-       (mew-pop-set-string pnm nil)
+       (mew-pop-set-input pnm nil)
        (mew-erase-buffer)))))
 
+(defun mew-pop-message2 (pnm msg left)
+  (let (msg2)
+    (cond
+     ((or (null left) (= left 0))
+      )
+     ((= left 1)
+      (setq msg2 " (1 message left)"))
+     (t
+      (setq msg2 (format " (%d messages left)" left))))
+    (if msg2 (setq msg (concat msg msg2)))
+    (mew-pop-message pnm msg)))
+
 (defun mew-pop-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (cnt (mew-pop-get-cnt pnm))
-	 (inbox (mew-pop-get-inbox pnm))
+	 (left (mew-pop-get-left pnm))
+	 (fld (mew-pop-get-fld pnm))
 	 (directive (mew-pop-get-directive pnm))
-	 (n (length (mew-pop-get-messages pnm)))
+	 (flush (mew-pop-get-flush pnm))
+	 (n (length (mew-pop-get-rtrs pnm)))
 	 savep)
     (mew-pop-debug "POP SENTINEL" event)
     (mew-filter
@@ -332,24 +369,31 @@ If MSG is nil, restore usual format."
       ((eq cnt nil)
        ())
       ((= cnt 0)
-       (mew-pop-message pnm "No new message"))
+       (if (eq directive 'mdrop)
+	   (mew-pop-message2 pnm "Deleting...done" left)
+	 (mew-pop-message2 pnm "No new messages" left)))
       ((= cnt 1)
-       (mew-pop-message pnm "1 message has been received.")
+       (mew-pop-message2 pnm "1 message retrieved" left)
        (setq savep t))
       (t
-       (mew-pop-message pnm "%d messages have been received." cnt)
+       (mew-pop-message2
+	pnm (format "%d messages retrieved" cnt) left)
        (setq savep t)))
-     (set-buffer inbox)
+     (set-buffer fld)
      (if savep (mew-summary-folder-cache-save))
      (set-buffer-modified-p nil)
+     (when (eq directive 'mdrop)
+       (let ((msgdb (mew-pop-get-msgdb pnm)))
+	 (while msgdb
+	   (mew-touch-folder (car (car msgdb)))
+	   (setq msgdb (cdr msgdb)))))
      (mew-pop-tear-down pnm)
      (unless (eq directive 'biff)
        (run-hooks 'mew-pop-sentinel-non-biff-hook))
      (run-hooks 'mew-pop-sentinel-hook)
-     (if (and mew-auto-flush-queue
-	      (not (or (eq directive 'biff) (eq directive 'no-flush))))
-	 (mew-smtp-flush-queue
-	  (mew-queue-folder mew-case-output) mew-case-output)))))
+     (when (and mew-auto-flush-queue flush)
+       (mew-smtp-flush-queue
+	(mew-queue-folder mew-case-output) mew-case-output)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -388,7 +432,7 @@ If MSG is nil, restore usual format."
   (mew-pop-set-status pnm "cram-md5"))
 
 (defun mew-pop-command-pwd-cram-md5 (pro pnm)
-  (let ((str (mew-pop-get-string pnm))
+  (let ((str (mew-pop-get-input pnm))
 	(user (mew-pop-get-user pnm))
 	challenge passwd cram-md5)
     (if (string-match " \\([A-Za-z0-9+/]+=*\\)" str) ;; xxx
@@ -416,9 +460,28 @@ If MSG is nil, restore usual format."
 
 (defvar mew-pop-uidl-file ".mew-uidl")
 
+(defsubst mew-time-diff (t1 t2)
+  (/ (+ (* (- (nth 0 t2) (nth 0 t1)) 65536)
+	(- (nth 1 t2) (nth 1 t1)))
+     86400.0)) ;; one day (* 60 60 24)
+
+(defsubst mew-pop-expired-p (time keep)
+  (cond
+   ((and (consp keep)
+	 (stringp (nth 0 keep)) (file-exists-p (nth 0 keep))
+	 (numberp (nth 1 keep)))
+    (if (>= (mew-time-diff time (mew-file-get-time (nth 0 keep))) (nth 1 keep))
+	t))
+   ((numberp keep)
+    (if (>= (mew-time-diff time (current-time)) keep) t))
+   ;; ((eq keep t) t)
+   ;; This case MUST not be included because messages marked with 'T'
+   ;; will be deleted.
+   (t nil)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;;
+;;; Set up and tear down
 ;;;
 
 (defun mew-pop-setup ()
@@ -442,7 +505,7 @@ If MSG is nil, restore usual format."
 (defun mew-pop-command-auth (pro pnm)
   (cond
    ((eq (mew-pop-get-auth pnm) t) ;; t means SASL
-    (let ((str (mew-pop-get-string pnm)) auth func)
+    (let ((str (mew-pop-get-input pnm)) auth func)
       (if (and (string-match "SASL \\([^\n\r]+\\)\r?\n" str)
 	       (setq auth (mew-auth-select
 			   (mew-match 1 str) mew-pop-auth-list))
@@ -473,7 +536,7 @@ If MSG is nil, restore usual format."
 (defun mew-pop-command-pass (pro pnm)
   (let (passwd)
     (setq passwd (mew-pop-input-passwd "POP password: " pnm))
-    (mew-pop-message pnm "Sending your POP password to the POP server ... ")
+    (mew-pop-message pnm "Sending your POP password to the POP server...")
     (process-send-string
      pro (format "pass %s%s" passwd mew-cs-eol))))
 
@@ -496,13 +559,13 @@ If MSG is nil, restore usual format."
       (setq passwd (mew-pop-input-passwd "APOP password: " pnm))
       (setq kmd5 (mew-keyed-md5 key passwd))
       (mew-pop-set-try pnm (1+ try))
-      (mew-pop-message pnm "Sending your APOP password to the POP server ... ")
+      (mew-pop-message pnm "Sending your APOP password to the POP server...")
       (process-send-string
        pro (format "apop %s %s%s" (mew-pop-get-user pnm) kmd5 mew-cs-eol)))))
 
 (defun mew-pop-command-wpwd (pro pnm)
   (let ((auth (mew-pop-get-auth pnm))
-	(str (mew-pop-get-string pnm))
+	(str (mew-pop-get-input pnm))
 	(clear-pass t))
     (cond
      ((and (stringp str) (string-match " lock" str)) ;; very ad hoc
@@ -526,10 +589,10 @@ If MSG is nil, restore usual format."
   (mew-pop-tear-down pnm)) ;; Typical servers disconnect, so no quit
 
 (defun mew-pop-command-list (pro pnm)
-  (mew-pop-message pnm "Communicating with the POP server ... ")
-  (mew-pop-set-keep-str pnm nil)
+  (mew-pop-message pnm "Communicating with the POP server...")
+  (mew-pop-set-keep-input pnm nil)
   (cond
-   ((mew-pop-get-sul pnm)
+   ((mew-pop-get-skip-list pnm)
     (mew-pop-set-status pnm "uidl")
     (process-send-string pro (format "uidl%s" mew-cs-eol)))
    (t
@@ -537,93 +600,185 @@ If MSG is nil, restore usual format."
 
 (defun mew-pop-command-uidl (pro pnm)
   (let (msgs num siz)
-    (while (re-search-forward "^\\([0-9]+\\) \\([0-9]+\\)" nil t)
+    (while (re-search-forward "^\\([0-9]+\\) +\\([0-9]+\\)" nil t)
       (setq num (mew-match 1))
       (setq siz (mew-match 2))
-      (setq msgs (cons (list num siz) msgs)))
+      (setq msgs (cons (list num nil siz) msgs)))
     (if msgs
 	(progn
 	  (setq msgs (nreverse msgs))
-	  (mew-pop-set-messages pnm msgs)
+	  (mew-pop-set-rtrs pnm msgs)
+	  (mew-pop-set-left pnm (length msgs))
 	  (process-send-string pro (format "uidl%s" mew-cs-eol)))
       (mew-pop-set-status pnm "quit")
       (mew-pop-command-quit pro pnm))))
 
 (defun mew-pop-command-umsg (pro pnm)
-  (let ((old-msgs (mew-pop-get-messages pnm)) ;; (num siz)
-	(sul (mew-pop-get-sul pnm))           ;; (siz uid lmsg)
-	(old-uidl (mew-pop-uidl-db-get pnm))
-	num siz uid msgs uidl n siz-uid-lmsg)
-    (while (re-search-forward "^\\([0-9]+\\) +\\([!-~]+\\)" nil t)
+  (let* ((ctime (current-time))
+	 (list-msgs (mew-pop-get-rtrs pnm)) ;; (num nil siz)
+	 (old-uidl (mew-pop-uidl-db-get pnm))
+	 (refs (mew-pop-get-refs pnm)) ;; ((uid siz del (+fld msg)) ...)
+	 (rmvs (mew-pop-get-rmvs pnm))
+	 (directive (mew-pop-get-directive pnm))
+	 (keep (mew-pop-get-delete pnm))
+	 (left 0)
+	 rtr rtrs dels
+	 num uid uidl uid-time ent)
+    (while (re-search-forward "^\\([0-9]+\\) +\\([!-~]*\\)" nil t)
+      (setq left (1+ left))
       (setq num (mew-match 1))
       (setq uid (mew-match 2))
+      (setq uid-time nil)
+      ;; A broken POP server may return a null UID.
+      (if (string= uid "") (setq uid nil))
       (cond
-       (sul ;; (num siz uidl lmsg)
-	(if (setq siz-uid-lmsg (mew-assoc-equal uid sul 1))
-	    (setq msgs (cons (cons num siz-uid-lmsg) msgs))))
-       (t ;; (num siz uidl)
-	(setq uidl (cons uid uidl))
-	(if (member uid old-uidl)
-	    ()
-	  (setq siz (nth 1 (assoc num old-msgs)))
-	  (setq msgs (cons (list num siz uid) msgs))))))
+       ((or (eq directive 'get) (eq directive 'mdrop))
+	(cond
+	 ((setq rtr (assoc uid refs))
+	  (setq rtrs (cons (cons num rtr) rtrs)))
+	 ((member uid rmvs)
+	  (setq dels (cons num dels)))))
+       ((eq directive 'biff)
+	(when (and uid (not (assoc uid old-uidl)))
+	  (setq rtrs (cons (list num) rtrs))))
+       ((eq directive 'scan)
+	(when uid
+	  (setq ent (assoc num list-msgs))
+	  (setcar (cdr ent) uid)
+	  (setq rtrs (cons ent rtrs))))
+       ((eq directive 'inc)
+	(if uid (setq uid-time (cdr (assoc uid old-uidl))))
+	(cond
+	 (uid-time
+	  (setq uidl (cons (cons uid uid-time) uidl))
+	  (if (mew-pop-expired-p uid-time keep)
+	      (setq dels (cons num dels))))
+	 (t
+	  (setq uidl (cons (cons uid ctime) uidl))
+	  (setq ent (assoc num list-msgs))
+	  (setcar (cdr ent) uid)
+	  (setq rtrs (cons ent rtrs)))))))
     (mew-pop-set-uidl pnm uidl)
-    (setq msgs (nreverse msgs))
-    (mew-pop-set-messages pnm msgs)
-    (setq n (length msgs))
-    (if (= n 0)
-	(progn
-	  (mew-pop-message pnm "No new message")
-	  (mew-pop-set-status pnm "quit")
-	  (mew-pop-command-quit pro pnm))
-      (if (= n 1)
-	  (mew-pop-message pnm "Receiving 1 message in background ... " )
-	(mew-pop-message pnm "Receiving %d messages in background ... " n))
+    (setq rtrs (nreverse rtrs))
+    (setq dels (nreverse dels))
+    (mew-pop-set-rtrs pnm rtrs)
+    (mew-pop-set-dels pnm dels)
+    (mew-pop-set-left pnm left)
+    (if dels
+	(mew-pop-command-pre-dels pro pnm)
+      (mew-pop-command-pre-retr pro pnm))))
+
+(defun mew-pop-command-pre-dels (pro pnm)
+  (let* ((dels (mew-pop-get-dels pnm))
+	 (directive (mew-pop-get-directive pnm))
+	 (n (length dels)))
+    (cond
+     ((= n 0) ;; should not occur
+      (mew-pop-command-pre-retr pro pnm))
+     ((= n 1)
+      (if (eq directive 'mdrop)
+	  (mew-pop-message pnm "Deleting 1 message in background..."))
+      (mew-pop-set-status pnm "dels")
+      (mew-pop-command-dels pro pnm))
+     (t
+      (if (eq directive 'mdrop)
+	  (mew-pop-message pnm "Deleting %d messages in background..." n))
+      (mew-pop-set-status pnm "dels")
+      (mew-pop-command-dels pro pnm)))))
+
+(defun mew-pop-command-dels (pro pnm)
+  (let ((dels (mew-pop-get-dels pnm))
+	(left (mew-pop-get-left pnm))
+	num)
+    (if (null dels)
+	(mew-pop-command-pre-retr pro pnm)
+      (setq num (car dels))
+      (mew-pop-set-dels pnm (cdr dels))
+      (mew-pop-set-left pnm (1- left))
+      (process-send-string pro (format "dele %s%s" num mew-cs-eol)))))
+
+(defun mew-pop-command-pre-retr (pro pnm)
+  (let* ((rtrs (mew-pop-get-rtrs pnm))
+	 (n (length rtrs))
+	 (directive (mew-pop-get-directive pnm)))
+    (cond
+     ((= n 0)
+      (unless (eq directive 'mdrop)
+	(mew-pop-message pnm "No new messages"))
+      (mew-pop-set-status pnm "quit")
+      (mew-pop-command-quit pro pnm))
+     ((= n 1)
+      (mew-pop-message pnm "Receiving 1 message in background...")
       (mew-pop-set-status pnm "retr")
-      (mew-pop-command-retr pro pnm))))
+      (mew-pop-command-retr pro pnm))
+     (t
+      (mew-pop-message pnm "Receiving %d messages in background..." n)
+      (mew-pop-set-status pnm "retr")
+      (mew-pop-command-retr pro pnm)))))
 
 (defun mew-pop-command-retr (pro pnm)
   (let* ((directive (mew-pop-get-directive pnm))
-	 (msgs (mew-pop-get-messages pnm))
-	 (msg  (car msgs))
-	 (num  (nth 0 msg))
-	 (siz  (nth 1 msg))
-	 (lmsg (nth 3 msg))
+	 (rtrs (mew-pop-get-rtrs pnm))
+	 (rtr  (car rtrs))
+	 (num  (nth 0 rtr))
+	 (siz  (nth 2 rtr))
 	 (lim (mew-pop-get-size pnm))
 	 (lines (mew-pop-get-body-lines pnm)))
     (cond
-     ((eq directive 'biff)
+     ((or (null rtr) (eq directive 'biff))
+      (mew-pop-set-truncated pnm nil)
       (mew-pop-set-status pnm "quit")
       (mew-pop-command-quit pro pnm))
-     ((null num)
-      (unless (mew-pop-get-sul pnm)
-	(mew-pop-uidl-db-set pnm (mew-pop-get-uidl pnm)))
-      (mew-pop-set-status pnm "quit")
-      (mew-pop-command-quit pro pnm 'save))
-     (lmsg
-      (mew-pop-set-lmsg pnm lmsg)
-      (mew-pop-set-top pnm nil)
+     ((or (eq directive 'get) (eq directive 'mdrop))
+      (mew-pop-set-truncated pnm nil)
       (process-send-string pro (format "retr %s%s" num mew-cs-eol)))
+     ((eq directive 'scan)
+      (mew-pop-set-truncated pnm t)
+      (process-send-string
+       pro (format "top %s 0%s" num mew-cs-eol)))
      ((or (= lim 0) (< (string-to-int siz) lim))
-      (mew-pop-set-top pnm nil)
+      (mew-pop-set-truncated pnm nil)
       (process-send-string pro (format "retr %s%s" num mew-cs-eol)))
      (t
-      (mew-pop-set-top pnm t)
+      (mew-pop-set-truncated pnm t)
       (process-send-string
        pro (format "top %s %d%s" num lines mew-cs-eol))))))
 
+(defun mew-pop-get-new-message (pnm fld msg)
+  (if msg
+      (cons msg (mew-expand-folder fld msg)) ;; 'get
+    (let ((msgdb (mew-pop-get-msgdb pnm))
+	  fld-msg file)
+      (cond
+       ((and msgdb (setq fld-msg (assoc fld msgdb)))
+	(setq msg (cdr fld-msg))
+	(setcdr fld-msg (int-to-string (1+ (string-to-int msg))))
+	(setq file (mew-expand-folder fld msg)))
+       (t
+	(setq file (mew-folder-new-message fld))
+	(setq msg (file-name-nondirectory file))
+	(mew-pop-set-msgdb
+	 pnm
+	 (cons (cons fld (int-to-string (1+ (string-to-int msg)))) msgdb))))
+      (cons msg file))))
+
 (defun mew-pop-command-dele (pro pnm)
-  (let* ((inbox (mew-pop-get-inbox pnm))
-	 (case (mew-pop-get-case pnm))
+  (let* ((case (mew-pop-get-case pnm))
 	 (width (1- (mew-scan-width)))
-	 (msgs (mew-pop-get-messages pnm))
-	 (msg  (car msgs))
-	 (num  (nth 0 msg))
-	 (siz  (nth 1 msg))
-	 (uid  (nth 2 msg))
-	 (lmsg (nth 3 msg))
-	 (truncated (mew-pop-get-top pnm))
-	 vec file)
+	 (left (mew-pop-get-left pnm))
+	 (rtrs (mew-pop-get-rtrs pnm))
+	 (rtr  (car rtrs))
+	 (num  (nth 0 rtr))
+	 (uid  (nth 1 rtr))
+	 (siz  (nth 2 rtr))
+	 (del  (or (eq (nth 3 rtr) t) ;; delete may be a number
+		   (eq (mew-pop-get-delete pnm) t)))
+	 (fld-msg (nth 4 rtr))
+	 (fld  (or (nth 0 fld-msg) (mew-pop-get-fld pnm)))
+	 (msg  (nth 1 fld-msg))
+	 (truncated (mew-pop-get-truncated pnm))
+	 (xmu mew-x-mew-uidl:)
+	 vec file msg-file)
     ;; deleting +OK
     (goto-char (point-min))
     (forward-line)
@@ -642,27 +797,14 @@ If MSG is nil, restore usual format."
     (while (re-search-forward "^\\." nil t)
       (delete-char -1)
       (forward-line))
-    (if lmsg
-	(setq file (mew-expand-folder inbox lmsg))
-      (setq lmsg (mew-pop-get-lmsg pnm))
-      (if lmsg 
-	  (setq file (mew-expand-folder inbox lmsg))
-	(setq file (mew-folder-new-message inbox))
-	(setq lmsg (file-name-nondirectory file)))
-      (mew-pop-set-lmsg pnm (int-to-string (1+ (string-to-int lmsg)))))
+    (setq msg-file (mew-pop-get-new-message pnm fld msg))
+    (setq msg (car msg-file) file (cdr msg-file))
     (when truncated
       (goto-char (point-min))
-      (unless (re-search-forward mew-eoh nil t)
-	(goto-char (point-max)))
       (when uid
 	(if case
-	    (mew-header-insert mew-x-mew-uidl: (concat uid " " siz " " case))
-	  (mew-header-insert mew-x-mew-uidl: (concat uid " " siz))))
-      (forward-line)
-      (insert
-       (format mew-pop-size-over-message
-	       (substitute-command-keys 
-		"\\<mew-summary-mode-map>\\[mew-summary-retrieve-message]"))))
+	    (mew-header-insert xmu (concat uid " " siz " " case))
+	  (mew-header-insert xmu (concat uid " " siz)))))
     (catch 'write-error
       (condition-case nil
 	  (mew-frwlet 
@@ -672,25 +814,37 @@ If MSG is nil, restore usual format."
 	 (mew-pop-set-status pnm "quit")
 	 (mew-pop-command-quit pro pnm)
 	 (throw 'write-error nil)))
-      ;;
       (set-file-modes file mew-file-mode)
       (mew-pop-set-cnt pnm (1+ (mew-pop-get-cnt pnm)))
       ;;
+      (let ((links (nthcdr 5 rtr))
+	    fld msg alias)
+	(while links
+	  (setq fld (nth 0 (car links)))
+	  (setq msg (nth 1 (car links)))
+	  (setq links (cdr links))
+	  (setq alias (cdr (mew-pop-get-new-message pnm fld msg)))
+	  (mew-link file alias)))
+      ;;
       (mew-set-buffer-multibyte t)
       (setq vec (mew-pop-scan-header))
-      (mew-scan-set-folder vec inbox)
-      (mew-scan-set-message vec lmsg)
+      (mew-scan-set-folder vec fld)
+      (mew-scan-set-message vec msg)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line inbox vec width lmsg nil)
-      (mew-pop-set-messages pnm (cdr msgs))
-      (if (and (mew-pop-get-delete pnm) (not truncated))
-	  (process-send-string
-	   pro (format "dele %s%s" num mew-cs-eol))
+      (mew-scan-insert-line fld vec width msg nil)
+      (mew-pop-set-rtrs pnm (cdr rtrs))
+      (if (and del (not truncated))
+	  (progn
+	    (mew-pop-set-left pnm (1- left))
+	    (process-send-string
+	     pro (format "dele %s%s" num mew-cs-eol)))
 	(mew-pop-set-status pnm "retr")
 	(mew-pop-command-retr pro pnm)))))
 
-(defun mew-pop-command-quit (pro pnm &optional save)
-  (if save (mew-lisp-save mew-pop-uidl-file mew-pop-uidl-db))
+(defun mew-pop-command-quit (pro pnm)
+  (when (mew-pop-get-save-uidl pnm)
+    (mew-pop-uidl-db-set pnm (mew-pop-get-uidl pnm))
+    (mew-lisp-save mew-pop-uidl-file mew-pop-uidl-db))
   (process-send-string
    pro (format "quit%s" mew-cs-eol)))
 
@@ -699,8 +853,7 @@ If MSG is nil, restore usual format."
 
 (defun mew-pop-scan-header ()
   (goto-char (point-min))
-  (if (re-search-forward mew-eoh nil t)
-      ()
+  (unless (re-search-forward mew-eoh nil t)
     (goto-char (point-max)))
   (mew-scan-header))
 
