@@ -3,7 +3,6 @@
 ;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;;          Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Jun 11, 1994
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -93,10 +92,10 @@
 	(cons (mew-uniq-list guess) info)
       (cons (list (car guess)) info))))
 
-
-;;
-;; guess control functions
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Guess control functions
+;;;
 
 (defun mew-refile-ctrl-auto-boundary (guess auto)
   (if auto "stop"))
@@ -104,9 +103,10 @@
 (defun mew-refile-ctrl-throw (guess auto)
   (if guess "stop"))
 
-;;
-;; by alist returns: (guess1 guess2 ...) or nil
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By alist returns: (guess1 guess2 ...) or nil
+;;;
 
 (defun mew-refile-guess-by-alist ()
   (mew-refile-guess-by-alist1 mew-refile-guess-alist))
@@ -162,9 +162,10 @@
 		      (substring string (match-end 0)))))
     string))
 
-;;
-;; by newsgroups returns (guess1 guess2 ...) or nil
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By newsgroups returns (guess1 guess2 ...) or nil
+;;;
 
 (defun mew-refile-guess-by-newsgroups ()
   (let ((newsgroups (mew-addrstr-parse-value-list2 
@@ -177,9 +178,10 @@
 	(setq newsgroups (cdr newsgroups)))
       (mew-uniq-list (nreverse ret)))))
 
-;;
-;; by folder returns: (guess1 guess2 ...) or nil
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By folder returns: (guess1 guess2 ...) or nil
+;;;
 
 (defun mew-refile-guess-by-folder ()
   (let ((to-cc (mew-header-parse-address-list mew-refile-guess-key-list))
@@ -194,9 +196,10 @@
       (setq to-cc (cdr to-cc)))
     (mew-uniq-list (nreverse ret))))
 
-;;
-;; by message-thread returns: guess1 or nil
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By message-thread returns: guess1 or nil
+;;;
 
 (defun mew-refile-guess-by-thread ()
   (let ((msgid (or (mew-header-get-value mew-references:)
@@ -206,9 +209,26 @@
 	     (string-match "\\(<[^ \t\n]*>\\)[^>]*\0" (concat msgid "\0")))
 	(nth 1 (assoc (mew-match 1 msgid) mew-refile-msgid-alist)))))
 
-;;
-;; by from-{folder,alist} returns: guess1 or (guess1 ...) or nil
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By from-{folder,alist} returns: guess1 or (guess1 ...) or nil
+;;;
+
+(defun mew-refile-guess-from-dispatch (func &optional addr)
+  "Dispatcher to make mew-refile-guess-by-from-* consider
+mew-refile-guess-from-me-is-special."
+  (let ((from (downcase (or addr (mew-header-parse-address mew-from:) ""))))
+    ;; if From: is my address, addr is the list extracted from To:, Cc:
+    (if (and mew-refile-guess-from-me-is-special
+	     (mew-is-my-address mew-regex-my-address-list from))
+	(let ((addrs (mew-header-parse-address-list mew-refile-guess-key-list))
+	      ret adr)
+	  (while addrs
+	    (if (setq adr (funcall func (car addrs)))
+		(setq ret (cons adr ret)))
+	    (setq addrs (cdr addrs)))
+	  (mew-uniq-list (nreverse ret)))
+      (funcall func from))))
 
 (defun mew-refile-guess-by-from-folder (&optional addr)
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-folder-body addr))
@@ -232,31 +252,13 @@
      (mew-refile-match-in-list user-regex mew-folder-list))))
 
 (defun mew-refile-guess-by-from-body (&optional addr)
-  (let ((from   (downcase (or addr (mew-header-parse-address mew-from:) ""))))
+  (let ((from (downcase (or addr (mew-header-parse-address mew-from:) ""))))
     (cdr (assoc from mew-refile-from-alist))))
 
-;;
-;; dispatcher to make mew-refile-guess-by-from-* consider
-;; mew-refile-guess-from-me-is-special.
-;;
-
-(defun mew-refile-guess-from-dispatch (func &optional addr)
-  (let ((addr (downcase (or addr (mew-header-parse-address mew-from:) ""))))
-    ;; if From: is my address, addr is the list extracted from To:, Cc:
-    (if (and mew-refile-guess-from-me-is-special
-	     (mew-is-my-address mew-regex-my-address-list addr))
-	(let ((addr (mew-header-parse-address-list mew-refile-guess-key-list))
-	      (a nil) (r nil))
-	  (while addr
-	    (if (setq a (funcall func (car addr)))
-		(setq r (cons a r)))
-	    (setq addr (cdr addr)))
-	  (mew-uniq-list (nreverse r)))
-      (funcall func addr))))
-
-;;
-;; by default returns: guess1
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; By default returns: guess1
+;;;
 
 (defun mew-refile-guess-by-default (&optional addr)
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-default-body addr))
@@ -294,7 +296,7 @@
 
 (defun mew-refile-guess-learn (buf result)
   (let ((chosen (car result))  ;; (folder1 folder2 ...)
-	(info   (cdr result))) ;; (guess-func-name guess1 guess2...)
+	(info (cdr result))) ;; (guess-func-name guess1 guess2...)
     (save-excursion
       (set-buffer buf)
       (if (member 'mew-refile-guess-by-from mew-refile-guess-control)
@@ -302,15 +304,53 @@
       (if (member 'mew-refile-guess-by-thread mew-refile-guess-control)
 	  (mew-refile-guess-by-thread-learn chosen info)))))
 ;;
+;; learn from "From:" field
+;;
+
+(defun mew-refile-guess-by-from-learn (chosen info)
+  (let* ((from (downcase (or (mew-header-parse-address mew-from:) "")))
+	 ;; 'my honest opinion' guessed by mew-refile-guess-by-from.
+	 (mho (nth 1 (assoc 'mew-refile-guess-by-from info)))
+	 folder to-cc)
+    ;; default leaning key X is the address derived from From: header, 
+    ;; but only when
+    ;;    mew-refile-guess-from-me-is-special is t
+    ;;    X is my address.
+    ;;    All addresses derived from To: Cc: values
+    ;;      point to the only one address Y.
+    ;; the learning key is set to Y instead of X.
+    (if (and mew-refile-guess-from-me-is-special
+	     (mew-is-my-address mew-regex-my-address-list from)
+	     (setq to-cc
+		   (mew-header-parse-address-list mew-refile-guess-key-list))
+	     (= (length to-cc) 1))
+	(setq from (car to-cc)))
+    (unless (or (or (null from) (null chosen)) (and mho (member mho chosen)))
+      ;; I decide which folder is most important among the user chosen
+      ;; folders. 
+      (catch 'match
+	(while chosen
+	  ;; searching a folder anyone couldn't predict.
+	  (unless (mew-refile-guess-member-p (car chosen) info)
+	    (throw 'match (setq folder (car chosen))))
+	  (setq chosen (cdr chosen))))
+      ;; If candidate was found, I memorize it.
+      (when folder
+	(setq mew-refile-from-alist
+	      (cons (cons from folder)
+		    (delq (assoc from mew-refile-from-alist) ;; delq is right
+			  mew-refile-from-alist)))))))
+
+;;
 ;; learn from msgid
 ;;
 
 (defun mew-refile-guess-by-thread-learn (chosen info)
-  (let* ((msgid  (mew-header-get-value mew-message-id:)) 
+  (let* ((msgid (mew-header-get-value mew-message-id:)) 
 	 (folder (car chosen))
 	 ;; ohter people's honest opinion and my honest opinion.
-	 (oho    info)
-	 (mho    (cdr (assoc 'mew-refile-guess-by-thread info))))
+	 (oho info)
+	 (mho (cdr (assoc 'mew-refile-guess-by-thread info))))
     (if (and msgid (string-match "<[^ \n>]*>" msgid))
 	(setq msgid (mew-match 0 msgid)))
     (when (and msgid chosen)
@@ -330,34 +370,6 @@
 	    (cons (list msgid folder "??")
 		  (delq (assoc msgid mew-refile-msgid-alist) ;; delq is right
 			mew-refile-msgid-alist))))))
-;;
-;; learn from "From:" field
-;;
-
-(defun mew-refile-guess-by-from-learn (chosen info)
-  (let* ((from  (downcase (or (mew-header-parse-address mew-from:) "")))
-	 (folder nil)
-	 ;; 'my honest opinion' guessed by mew-refile-guess-by-from.
-	 (mho    (nth 1 (assoc 'mew-refile-guess-by-from       info))))
-
-    (if (or (or (null from) (null chosen)) ;; if from and/or chosen is empty
-	    (and mho (member mho chosen))) ;; or my opinion was right,
-	()                                 ;; do nothing.
-      ;; I decide which folder is most important among the user chosen
-      ;; folders. 
-      (catch 'match
-	(while chosen
-	  ;; searching a folder anyone couldn't predict.
-	  (unless (mew-refile-guess-member-p (car chosen) info)
-	    (throw 'match (setq folder (car chosen))))
-	  (setq chosen (cdr chosen))))
-
-      ;; If candidate was found, I memorize it.
-      (when folder
-	(setq mew-refile-from-alist
-	      (cons (cons from folder)
-		    (delq (assoc from mew-refile-from-alist) ;; delq is right
-			  mew-refile-from-alist)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -369,11 +381,8 @@
    (concat  "^" (regexp-quote (file-name-as-directory parent)))
    child))
 
-;;
-;; search x in a nested list.
-;;
-
 (defun mew-refile-guess-member-p (x lst)
+  "Search X in a nested list LST."
   (catch 'found
     (while lst
       (cond 
@@ -385,11 +394,8 @@
 	    (throw 'found t))))
       (setq lst (cdr lst)))))
 
-;;
-;; grep with regex in a list of string. (first match)
-;;
-
 (defun mew-refile-match-in-list (regex lst)
+  "Grep with REGEX in a list of string LST. (first match)"
   (let ((case-fold-search t))
     (catch 'found 
       (while lst
@@ -402,25 +408,19 @@
 ;;; Summary mode
 ;;;
 
-(defun mew-refile-get (fld msg)
-  (save-excursion
-    (set-buffer fld)
-    (cdr (assoc msg (mew-sinfo-get-refile)))))
+(defun mew-refile-get (msg)
+  (assoc msg (mew-sinfo-get-refile)))
 
-(defun mew-refile-set (fld msg folders)
-  ;; mew-refile-reset MUST be called before this.
-  (when (get-buffer fld)
-    (save-excursion
-      (set-buffer fld)
-      (mew-sinfo-set-refile
-       (cons (cons msg folders) (mew-sinfo-get-refile))))))
+(defun mew-refile-set (msg folders)
+  (mew-sinfo-set-refile (cons (cons msg folders) (mew-sinfo-get-refile))))
 
-(defun mew-refile-reset (fld msg)
-  (when (get-buffer fld)
-    (save-excursion
-      (set-buffer fld)
-      (mew-sinfo-set-refile
-       (delq (assoc msg (mew-sinfo-get-refile)) (mew-sinfo-get-refile))))))
+(defun mew-refile-reset (msg)
+  (mew-sinfo-set-refile
+   (delq (mew-refile-get msg) (mew-sinfo-get-refile))))
+
+(defun mew-refile-change (src dst)
+  (let ((ref (mew-refile-get src)))
+    (when ref (setcar ref dst))))
 
 ;; mew-refile-decide-folders returns: ((input1 input2...) 
 ;;                                     (guess-func-name1 guess1 guess2...)
@@ -430,31 +430,49 @@
 ;; cdr is needed for learning functions.
 ;;
 
-(defun mew-refile-decide-folders (buf msg cur-folders &optional auto)
-  (let (learn-info folders ret)
+(defun mew-refile-decide-folders (buf msg cur-folders &optional auto exfld)
+  (let (learn-info folders ret cands singlep)
     (save-excursion
       (set-buffer buf)
       (setq learn-info (mew-refile-guess auto)))
-    (cond
-     (auto
-      ;; if auto is set, simply use the guess.
-      (setq folders (car learn-info)))
-     (cur-folders
-      ;; add new folder 
-      (setq folders (mew-input-folders nil (mew-join "," cur-folders))))
-     ((nth 1 (car learn-info))
-      ;; multi guess
-      (setq folders (mew-input-folders nil (mew-join "," (car learn-info)))))
-     (t 
-      ;; single guess
-      (setq folders (mew-input-folders (nth 0 (car learn-info))))))
+    (if auto
+	;; if auto is set, simply use the guess.
+	(setq folders (car learn-info))
+      (cond
+       (cur-folders
+	;; add new folder 
+	(setq cands cur-folders))
+       ((nth 1 (car learn-info))
+	;; multi guess
+	(setq cands (car learn-info)))
+       (t 
+	;; single guess
+	(setq singlep t)
+	(setq cands (list (nth 0 (car learn-info))))))
+      (setq cands (delete nil cands))
+      (when exfld
+	(if (or (null cands) (equal (list exfld) cands))
+	    (setq cands (list "+")))
+	(setq cands (cons exfld cands)))
+      (setq cands (mew-uniq-list cands))
+      (if (and singlep (= (length cands) 1))
+	  (setq folders (mew-input-folders (car cands)))
+	(setq folders (mew-input-folders cands))))
     ;; check folder existence.
+    (setq folders (mew-uniq-list (mew-split (mew-join "," folders) ?,)))
     (while folders
       (if (mew-folder-check (car folders))
 	  (setq ret (cons (car folders) ret)))
       (setq folders (cdr folders)))
     (cons (nreverse ret) (cdr learn-info)))) ;; return value
 
+(defun mew-summary-copy ()
+ (interactive)
+ (mew-summary-msg-or-part
+  (mew-summary-not-in-draft
+   (mew-summary-refile-body
+    nil nil nil nil (mew-summary-folder-name)))))
+
 (defun mew-summary-refile (&optional report)
  "Put the refile mark(default is 'o') on this message. If already
 marked with 'o', it prints where this message will be refiled. This
@@ -466,7 +484,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
   (mew-summary-not-in-draft
    (if report (mew-summary-refile-report) (mew-summary-refile-body)))))
 
-(defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark)
+(defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark exfld)
   (let (msg folders cur-folders mark buf learn-info fld tmp delbuf)
     (save-excursion
       ;; save the cursor position anyway
@@ -498,9 +516,12 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	;; mew-cache-hit should be first since we want to get 
 	;; information form the top level header.
 	(setq buf (or (mew-cache-hit fld msg) (mew-buffer-message))))
-      (if (eq mew-mark-refile mark)
-	  (setq cur-folders (mew-refile-get fld msg)))
-      (setq learn-info (mew-refile-decide-folders buf msg cur-folders auto))
+      (when (and (eq mew-mark-refile mark) (get-buffer fld))
+	(save-excursion
+	  (set-buffer fld)
+	  (setq cur-folders (cdr (mew-refile-get msg)))))
+      (setq learn-info (mew-refile-decide-folders
+			buf msg cur-folders auto exfld))
       (setq folders (car learn-info)))
     ;; we must prevent refiling a message to +queue
     (while folders
@@ -515,11 +536,15 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	(save-excursion
 	  (mew-summary-goto-message)
 	  (or exp-flds auto (mew-refile-guess-learn buf learn-info))
-	  (mew-refile-reset fld msg)
-	  (mew-refile-set fld msg folders)
+	  (when (get-buffer fld)
+	    (save-excursion
+	      (set-buffer fld)
+	      (mew-refile-reset msg)
+	      (mew-refile-set msg folders)))
 	  (mew-mark-unmark))
 	(mew-mark-put-mark mew-mark-refile no-msg) ;; cursor may move
-	(mew-summary-mark-in-physical fld msg mew-mark-refile)))
+	(if (mew-virtual-p)
+	    (mew-summary-mark-in-physical fld msg mew-mark-refile))))
     (if delbuf (mew-remove-buffer buf))
     ;; memorize last-folder
     (setq mew-refile-last-folder folders)
@@ -527,19 +552,16 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
     folders)) ;; return value
 
 (defun mew-summary-refile-report ()
-  (let ((buf (current-buffer))
+  (let ((fld (mew-summary-folder-name))
+	(msg (mew-summary-message-number2))
+	(win (selected-window))
 	(tmpbuf (generate-new-buffer mew-buffer-prefix))
-	fld msg guess 
 	(customize-var '(mew-refile-ctrl-multi
 			 mew-refile-guess-key-list
 			 mew-refile-guess-strip-domainpart
-			 mew-refile-guess-from-me-is-special)))
+			 mew-refile-guess-from-me-is-special))
+	guess)
     (save-excursion
-      ;; save the cursor position anyway
-      (mew-summary-goto-message)
-      ;; on the message
-      (setq fld (mew-summary-folder-name))
-      (setq msg (mew-summary-message-number));; msg is never nil
       (set-buffer tmpbuf)
       (mew-erase-buffer)
       (mew-insert-message fld msg mew-cs-autoconv
@@ -569,7 +591,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	 (setq guess (cdr guess)))))
     (mew-message-clear-end-of)
     (set-buffer-modified-p nil)
-    (mew-pop-to-buffer buf)))
+    (select-window win)))
 
 (defun mew-summary-refile-again ()
   "Put a refile mark on this message according to the previous 
@@ -606,8 +628,8 @@ If '\\[universal-argument]' is specified, only messages marked with
        (t
 	(setq msg (format "Refile all messages except those marked with '%c' and '%c'? " mew-mark-refile mew-mark-delete))))
       (if (and mew-refile-auto-refile-confirm (null (yes-or-no-p msg)))
-	  (message "Not refiled.")
-	(message "Auto refiling ... ")
+	  (message "Not refiled")
+	(message "Auto refiling...")
 	(save-window-excursion
 	  (goto-char (point-min))
 	  (while (not (eobp))
@@ -621,11 +643,11 @@ If '\\[universal-argument]' is specified, only messages marked with
 		    (and mark mew-refile-auto-refile-skip-any-mark)
 		    (mew-summary-refile-body nil t 'no-msg)))
 	      (if (= (% (/ (* 100 line) lines) 10) 0)
-		  (message "Auto refiling ... %s%%"
+		  (message "Auto refiling...%s%%"
 			   (/ (* 100 line) lines)))
 	      (setq line (1+ line)))
 	    (forward-line))
-	  (message "Auto refiling ... done")))))))
+	  (message "Auto refiling...done")))))))
 
 ;;
 ;; "mx" extension
@@ -639,14 +661,14 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
   (let ((after-change-functions nil)
 	(mew-use-highlight-x-face nil)
 	(last nil))
-    (message "Mark refiling ... ")
+    (message "Mark refiling...")
     (save-excursion
       (save-window-excursion
 	(goto-char (point-min))
 	(while (re-search-forward mew-regex-msg-review nil t)
 	  (setq last (mew-summary-refile-body last nil 'no-msg))
 	  (forward-line))
-	(message "Mark refiling ... done")))))
+	(message "Mark refiling...done")))))
 
 (provide 'mew-refile)
 
