@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -26,20 +25,20 @@
 'mew-scan-form-mark-delete' and 'mew-scan-form-mark-review'
 effect to this function."
   (let ((id (MEW-ID)) duplicated review)
-    (if mew-scan-form-mark-delete
-	(if (not (string-match mew-regex-id id))
-	    ()
-	  (setq id (mew-match 1 id))
-	  (if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
-	      (setq duplicated t)
-	    (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
-    (if mew-scan-form-mark-review
-	(let ((to (mew-addrstr-parse-address-list (MEW-TO))))
-	  (catch 'loop
-	    (while to
-	      (if (mew-is-my-address mew-regex-my-address-list (car to))
-		  (throw 'loop (setq review t)))
-	      (setq to (cdr to))))))
+    (when mew-scan-form-mark-delete
+      (when (string-match mew-regex-id id)
+	(setq id (mew-match 1 id))
+	(if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
+	    (setq duplicated t)
+	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
+    (when mew-scan-form-mark-review
+      (let* ((mew-header-max-depth nil)
+	     (to (mew-addrstr-parse-address-list (MEW-TO))))
+	(catch 'loop
+	  (while to
+	    (if (mew-is-my-address mew-regex-my-address-list (car to))
+		(throw 'loop (setq review t)))
+	    (setq to (cdr to))))))
     (cond
      (duplicated (char-to-string mew-mark-delete))
      (review     (char-to-string mew-mark-review))
@@ -48,10 +47,10 @@ effect to this function."
 (defun mew-scan-form-type ()
   "A function to return a mark of content type."
   (let ((ct (MEW-CT))
-	(uidl (mew-scan-usc-uidl (mew-scan-split-usc (MEW-UIDL))))
+	(uid (mew-scan-usc-uid (mew-scan-split-usc (MEW-UIDL))))
 	(case-fold-search t))
     (cond
-     ((not (string= uidl ""))                      "T")
+     ((not (string= uid ""))                       "T")
      ((string-match "Multipart/Signed"    ct)      "S")
      ((string-match "Multipart/Encrypted" ct)      "E")
      ((string-match "Application/X-Pkcs7-Mime" ct) "E")
@@ -72,10 +71,10 @@ effect to this function."
 (defun mew-scan-form-date ()
   "A function to return a date, MM/DD."
   (let ((s (MEW-DATE)))
-    (if (or (string= s "")
-	    (not (string-match mew-time-rfc-regex s)))
-	(setq s (mew-time-ctz-to-rfc
-		 (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+    (when (or (string= s "")
+	      (not (string-match mew-time-rfc-regex s)))
+      (setq s (mew-time-ctz-to-rfc
+	       (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
     (if (string-match mew-time-rfc-regex s)
 	(format "%02d/%02d"
 		(mew-time-mon-str-to-int (mew-time-rfc-mon))
@@ -104,9 +103,9 @@ with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
   (let ((len-1 (1- (length mew-scan-form-size-unit)))
 	(SIZE (mew-scan-usc-size (mew-scan-split-usc (MEW-UIDL))))
 	(i 0) size unit)
-    (if (string= SIZE "")
-	(setq size (mew-file-get-size (mew-expand-folder (MEW-FLD) (MEW-NUM))))
-      (setq size (string-to-int SIZE)))
+    (if (string-match "^[0-9]+$" SIZE)
+	(setq size (string-to-int SIZE))
+      (setq size (mew-file-get-size (mew-expand-folder (MEW-FLD) (MEW-NUM)))))
     (while (and (< i len-1) (>= size 1000))
       (setq size (/ size 1000))
       (setq i (1+ i)))
@@ -159,7 +158,7 @@ case, 'mew-scan-form-from-me-prefix' is prepended to the address.
 Otherwise, an address on From: is returned.
 
 Address is converted by 'mew-scan-form-extract-addr'. See also
-'mew-scan-form-extract rule'."
+'mew-scan-form-extract-rule'."
   (let* ((FROM (MEW-FROM)) (TO (MEW-TO))
 	 (from (or (mew-addrstr-parse-address FROM) "")))
     (cond
@@ -192,7 +191,7 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
 (defsubst mew-scan-split-usc (xuidl)
   (mew-split xuidl 32))
 
-(defsubst mew-scan-usc-uidl (usc)
+(defsubst mew-scan-usc-uid (usc)
   (or (nth 0 usc) ""))
 
 (defsubst mew-scan-usc-size (usc)
@@ -210,15 +209,13 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
       (if (symbolp spec)
 	  (setq width 1 func spec)
 	(setq width (nth 0 spec) func (nth 1 spec)))
-      (if (not (and (integerp width) (symbolp func)))
-	  ()
+      (when (and (integerp width) (symbolp func))
 	(if (= width 0) (setq width (- WIDTH TOTAL 1)))
 	(if (< width 0) (setq width (abs width) rightp t))
 	(setq TOTAL (+ TOTAL width))
 	(setq func (intern-soft
 		    (concat mew-scan-form-func-prefix (symbol-name func))))
-	(if (not (fboundp func))
-	    ()
+	(when (fboundp func)
 	  (setq str (funcall func))
 	  (if rightp
 	      (if (<= (length str) width)
@@ -231,7 +228,7 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
   (let* ((TOTAL 0)
 	 (line (mapconcat (function mew-scan-get-piece)
 			  (mew-sinfo-get-scan-form) ""))
-	 (my-id "") (par-id "") fld msg ld first-irt onlyone start)
+	 (my-id "") (par-id "") (first-irt "") fld msg ld onlyone start)
     (if (string-match mew-regex-id (MEW-ID))
 	(setq my-id (mew-match 1 (MEW-ID))))
     ;; RFC 2822 says: the "In-Reply-To:" field may be used to identify
@@ -291,9 +288,10 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
 (defun mew-summary-scan-form (folder &optional column)
   "Get scan-form from 'mew-summary-refine-scan-form'.
 See also 'mew-scan-form-list-list'."
-  (let ((lst mew-scan-form-list) key keys values ret form
+  (let ((lst mew-scan-form-list)
 	(str-func (mew-scan-form-list-string-func))
-	(lst-func (mew-scan-form-list-list-func)))
+	(lst-func (mew-scan-form-list-list-func))
+	key keys values ret form col)
     (catch 'loop
       (while lst
 	(setq keys (car (car lst)))
@@ -311,14 +309,13 @@ See also 'mew-scan-form-list-list'."
 	    (setq keys (cdr keys))
 	    (if (funcall lst-func key folder)
 		(throw 'loop (setq ret values))))))))
-    (cond
-     (column
-      (if (and ret (numberp (nth 1 ret))) (nth 1 ret) mew-thread-column))
-     (t
-      (if ret
-	  (setq form (nth 0 ret))
-	(setq form mew-scan-form))
-      (mew-summary-final-scan-form form)))))
+    (if ret
+	(setq form (nth 0 ret) col (nth 1 ret))
+      (setq form mew-scan-form))
+    (setq form (mew-summary-final-scan-form form))
+    (if column
+	(if (numberp col) col (mew-thread-column form))
+      form)))
 
 (defun mew-summary-final-scan-form (form)
   "Canonicalize the scan-form specified by FORM.
@@ -329,6 +326,26 @@ Otherwise append 'mew-scan-form-header' to FORM."
       (cons (list (car form) 'num) (cons 'mark (cdr form)))
     (append mew-scan-form-header form)))
 
+(defun mew-thread-column (form)
+  (let ((col 0) ret ent)
+    (catch 'loop
+      (while form
+	(setq ent (car form))
+	(setq form (cdr form))
+	(cond
+	 ((consp ent)
+	  (setq col (+ col (abs (car ent)))))
+	 ((stringp ent)
+	  (setq col (+ col (string-width ent))))
+	 ((eq ent t)
+	  (throw 'loop (setq ret col)))
+	 (t
+	  (setq col (1+ col))))))
+    (or ret mew-thread-column)))
+
+(defun mew-summary-scan-form-num (folder)
+  (abs (car (rassoc '(num) (mew-summary-scan-form folder)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Exchanging the cursor
@@ -342,7 +359,7 @@ Otherwise append 'mew-scan-form-header' to FORM."
   (mew-summary-only
    (cond
     ((not (mew-folder-inboxp (mew-summary-folder-name 'ext)))
-     (message "Can't be used in this folder"))
+     (message "Cannot be used in this folder"))
     ((not (marker-position mew-summary-inbox-position))
      (message "No previous folder"))
     (t
@@ -383,7 +400,6 @@ If called with '\\[universal-argument]', +queue is not flushed."
   (let* ((case mew-case-input)
 	 (inbox (mew-inbox-folder case))
 	 (mailbox (mew-mailbox-type case))
-	 (directive (if no-flush 'no-flush))
 	 range)
     ;; for mew-summary-exchange-point.
     (cond
@@ -412,60 +428,79 @@ If called with '\\[universal-argument]', +queue is not flushed."
       (mew-sinfo-set-scan-form (mew-summary-scan-form inbox))
       (cond
        ((eq mailbox 'pop)
-	(mew-pop-retrieve case nil nil nil directive))
+	(mew-pop-retrieve case 'inc (not no-flush)))
        ((eq mailbox 'mbox)
 	(mew-mbox-retrieve case))))))
 
 (defun mew-header-get-xuidl-from-file (fld msg)
   (let ((buf (generate-new-buffer mew-buffer-prefix))
+	(xmu mew-x-mew-uidl:)
 	xuidl)
     (save-excursion
       (set-buffer buf)
       (mew-erase-buffer)
       (mew-insert-message
        fld msg mew-cs-text-for-read mew-header-reasonable-size)
-      (setq xuidl (mew-header-get-value mew-x-mew-uidl:)))
+      (setq xuidl (mew-header-get-value xmu)))
     (mew-remove-buffer buf)
     xuidl))
 
 (defun mew-summary-retrieve-message (&optional rev-del)
-  "Retrieve the entire of a truncated('T') message."
+  "Retrieve the entire of a truncated('T') message.
+If 'mew-pop-delete' is 't', delete the message from the mailbox.
+When executed with '\\[universal-argument]', 'mew-pop-delete' is considered reversed."
   (interactive "P")
-  (mew-summary-msg-or-part
+  (mew-summary-msg
    (mew-summary-only
-    (when (mew-summary-exclusive-p)
-      (let* ((fld (mew-summary-folder-name 'ext))
-	     (msg (mew-summary-message-number))
-	     (xuidl (mew-header-get-xuidl-from-file fld msg))
-	     usc uidl size case)
-	(if (null xuidl)
-	    (message "No %s field" mew-x-mew-uidl:)
-	  (setq usc (mew-scan-split-usc xuidl))
-	  (setq uidl (mew-scan-usc-uidl usc))
-	  (setq size (mew-scan-usc-size usc))
-	  (setq case (mew-scan-usc-case usc))
-	  (mew-sinfo-set-scan-form (mew-summary-scan-form fld))
-	  (mew-pop-retrieve case (list (list size uidl msg)) rev-del fld)))))))
+    (mew-summary-not-in-mdrop
+     (when (mew-summary-exclusive-p)
+       (let* ((fld (mew-summary-folder-name 'ext))
+	      (msg (mew-summary-message-number))
+	      (xuidl (mew-header-get-xuidl-from-file fld msg))
+	      usc uid siz case del rtr rtrs)
+	 (if (null xuidl)
+	     (message "No %s field" mew-x-mew-uidl:)
+	   (setq usc (mew-scan-split-usc xuidl))
+	   (setq uid (mew-scan-usc-uid usc))
+	   (setq siz (mew-scan-usc-size usc))
+	   (setq case (mew-scan-usc-case usc))
+	   (setq del (eq (mew-pop-delete case) t))
+	   (if rev-del (setq del (not del)))
+	   (setq rtr (list uid siz del (list fld msg)))
+	   (setq rtrs (list rtr))
+	   (mew-sinfo-set-scan-form (mew-summary-scan-form fld))
+	   (mew-pop-retrieve case 'get fld rtrs))))))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
-  "Retrieve the entire of truncated('T') messages marked with '*'."
+  "Retrieve the entire of truncated('T') messages marked with '*'.
+If 'mew-pop-delete' is 't', delete the message from the mailbox.
+When executed with '\\[universal-argument]', 'mew-pop-delete' is considered reversed."
   (interactive "P")
   (mew-summary-only
-   (let* ((fld (mew-summary-folder-name 'ext))
-	  msgs msg usc xuidl uidl size case sul)
-     (setq msgs (mew-summary-mark-collect mew-mark-review))
-     (while msgs
-       (setq msg (car msgs))
-       (setq msgs (cdr msgs))
-       (setq xuidl (mew-header-get-xuidl-from-file fld msg))
-       (when xuidl
-	 (setq usc (mew-scan-split-usc xuidl))
-	 (setq uidl (mew-scan-usc-uidl usc))
-	 (setq size (mew-scan-usc-size usc))
-	 (setq case (mew-scan-usc-case usc))
-	 (setq sul (cons (list size uidl msg) sul))))
-     (mew-sinfo-set-scan-form (mew-summary-scan-form fld))
-     (mew-pop-retrieve case sul rev-del fld))))
+   (mew-summary-not-in-mdrop
+    (let* ((fld (mew-summary-folder-name 'ext))
+	   (msgs (mew-summary-mark-collect mew-mark-review))
+	   msg usc xuidl uid siz case del rtr rtrs)
+      (if (null msgs)
+	  (message "No marked messages")
+	(if rev-del (setq del (not del)))
+	(while msgs
+	  (setq msg (car msgs))
+	  (setq msgs (cdr msgs))
+	  (setq xuidl (mew-header-get-xuidl-from-file fld msg))
+	  (when xuidl
+	    (setq usc (mew-scan-split-usc xuidl))
+	    (setq uid (mew-scan-usc-uid usc))
+	    (setq siz (mew-scan-usc-size usc))
+	    (setq case (mew-scan-usc-case usc)) ;; xxx
+	    (setq del (eq (mew-pop-delete case) t))
+	    (if rev-del (setq del (not del)))
+	    (setq rtr (list uid siz del (list fld msg)))
+	    (setq rtrs (cons rtr rtrs)))))
+      (if (null rtrs)
+	  (message "No %s field" mew-x-mew-uidl:)
+	(mew-sinfo-set-scan-form (mew-summary-scan-form fld))
+	(mew-pop-retrieve case 'get fld rtrs))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -481,61 +516,62 @@ If called with '\\[universal-argument]', +queue is not flushed."
 	(setq args (list "-e" mbox-command "-m" mbox-command-arg))
       (setq args (list "-e" mbox-command)))
     (setq args (append args (mew-scan-mewls-src inbox)))
-    (mew-scan args)))
+    (mew-scan args  nil nil 'flush)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Listing messages
+;;; Scanning a folder
 ;;;
 
-(defun mew-summary-ls (&optional stay jump)
+(defun mew-summary-ls (&optional jump)
   "List this folder asynchronously.
 If executed with '\\[universal-argument]', the cursor stays in the
 current position."
   (interactive "P")
-  (mew-summary-only
-   (let* ((folder (mew-summary-folder-name 'ext))
-	  (askp mew-ask-range)
-	  scanp range lines oldformatp)
-     (mew-summary-folder-cache-load)
-     (save-excursion
-       (goto-char (point-min))
-       (if (and (not (eobp))
-		(not (looking-at mew-regex-thread))
-		(y-or-n-p "The format of this scan cache is old. Type 'y' when ready. "))
-	   (setq oldformatp t)))
-     (if oldformatp
-	 (progn
-	   (setq range mew-range-all)
-	   (setq lines (mew-summary-mark-collect3 mew-mark-collect))
-	   (mew-erase-buffer)
-	   (mew-scan (mew-scan-mewls-src folder range) lines))
-       ;; check scanp before (mew-mark-clean)
-       (cond
-	((interactive-p)
-	 (setq askp t)
-	 (setq scanp t))
-	((mew-folder-remotep folder)
-	 (setq askp t)
-	 (setq scanp t))
-	((mew-summary-folder-dir-newp)
-	 (setq scanp t)))
-       ;;
-       (mew-mark-clean)
-       (set-buffer-modified-p nil)
-       (if jump (goto-char (point-max)))
-       (cond
-	(scanp
-	 (setq range (mew-input-range folder askp))
-	 (or stay (goto-char (point-max)))
-	 (when (mew-range-erase-p range)
-	   (if (string= range mew-range-all)
-	       (setq lines (mew-summary-mark-collect3 mew-mark-collect)))
-	   (mew-erase-buffer))
-	 (mew-scan (mew-scan-mewls-src folder range) lines))
-	(t
-	 (or stay (goto-char (point-max)))
-	 (run-hooks 'mew-summary-ls-no-scan-hook)))))))
+  (when (mew-summary-exclusive-p)
+    (mew-summary-only
+     (let* ((folder (mew-summary-folder-name 'ext))
+	    (askp mew-ask-range)
+	    scanp range mkdb oldformatp)
+       (mew-summary-folder-cache-load)
+       (save-excursion
+	 (goto-char (point-min))
+	 (if (and (not (eobp))
+		  (not (looking-at mew-regex-thread))
+		  (y-or-n-p "The format of this scan cache is old. Type 'y' when ready. "))
+	     (setq oldformatp t)))
+       (if oldformatp
+	   (progn
+	     (setq range mew-range-all)
+	     (setq mkdb (mew-summary-mark-collect4))
+	     (mew-erase-buffer)
+	     (mew-scan (mew-scan-mewls-src folder range) mkdb))
+	 (cond
+	  ((interactive-p);; "s"
+	   (setq askp t)
+	   (setq scanp t))
+	  ((mew-summary-folder-dir-newp);; "g"
+	   (setq askp nil)
+	   (setq scanp t)))
+	 (set-buffer-modified-p nil)
+	 (if (or (interactive-p) jump) (goto-char (point-max)))
+	 (if (not scanp)
+	     (run-hooks 'mew-summary-ls-no-scan-hook)
+	   ;; asking range when "s"
+	   (setq range (mew-input-range folder askp))
+	   (when (mew-range-erase-p range)
+	     (setq mkdb (mew-summary-mark-collect4))
+	     (mew-erase-buffer))
+	   (if (mew-folder-mdropp folder)
+	       (mew-scan-mdrop folder)
+	     (mew-scan (mew-scan-mewls-src folder range) mkdb))))))))
+
+(defun mew-scan-mdrop (mdrop)
+  (let* ((mdropdir (mew-expand-folder mdrop))
+	 (msgs (mew-dir-messages mdropdir)))
+    (mew-summary-clean-msgs mdrop msgs))
+  (mew-sinfo-set-scan-form (mew-summary-scan-form mdrop))
+  (mew-pop-retrieve mew-case-input 'scan mdrop))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -608,97 +644,109 @@ current position."
       (list "-s" (format "%s %s" folder range))
     (list "-s" folder)))
 
-(defun mew-scan (args &optional reviews virtualp)
+(defun mew-scan (args &optional mkdb virtualp flush)
   (cond
    ((not (mew-summary-exclusive-p))
     ())
    ((not (mew-which-exec mew-prog-mewls))
-    (message "'mewls' not found!"))
+    (message "'%s' not found!" mew-prog-mewls))
    (t
-    (condition-case nil
-	(let ((process-connection-type mew-connection-type2)
-	      process buf bnm)
-	  (cond
-	   (virtualp
-	    (or (mew-virtual-p) (mew-virtual-mode)))
-	   (t 
-	    (or (mew-summary-p) (mew-summary-mode))))
-	  (mew-window-configure 'summary)
-	  (mew-current-set nil nil nil)
-	  (mew-decode-syntax-delete)
-	  (setq bnm (mew-summary-folder-name 'ext))
-	  (mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
-	  (mew-sinfo-set-start-point (point))
-	  (mew-sinfo-set-direction 'down)
-	  (mew-sinfo-set-scan-id nil)
-	  (message "Listing %s ... " bnm)
-	  (setq buf (get-buffer-create (mew-scan-buffer-name bnm)))
-	  (save-excursion
-	    (set-buffer buf)
-	    (mew-erase-buffer))
-	  (setq args (append (list "-b" mew-mail-path "-c" mew-news-path
-				   "-w" (int-to-string mew-scan-wait-for)
-				   "-f" (mapconcat (function identity)
-						   (nthcdr 2 mew-scan-fields)
-						   ","))
-			     args))
-	  (setq process (apply (function start-process)
-			       (mew-scan-info-name bnm)
-			       buf mew-prog-mewls args))
-	  (setq mew-summary-buffer-process process)
-	  (mew-set-process-cs process mew-cs-text-for-read mew-cs-dummy)
-	  ;; text may be broken, so undecided is very dangerous!
-	  (set-process-filter   process 'mew-scan-filter)
-	  (set-process-sentinel process 'mew-scan-sentinel)
-	  (mew-sinfo-set-reviews reviews)
-	  (process-kill-without-query mew-summary-buffer-process))
-      (quit
-       (set-process-sentinel mew-summary-buffer-process nil)
-       (mew-sinfo-set-start-point nil)
-       (mew-sinfo-set-reviews nil)
-       (setq mew-summary-buffer-process nil))))))
+    (let ((process-connection-type mew-connection-type2)
+	  process buf bnm)
+      (condition-case nil
+	  (progn
+	    (cond
+	     (virtualp
+	      (or (mew-virtual-p) (mew-virtual-mode)))
+	     (t 
+	      (or (mew-summary-p) (mew-summary-mode))))
+	    (mew-window-configure 'summary)
+	    (mew-current-set nil nil nil)
+	    (mew-decode-syntax-delete)
+	    (setq bnm (mew-summary-folder-name 'ext))
+	    (mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	    (mew-sinfo-set-start-point (point))
+	    (mew-sinfo-set-direction 'down)
+	    (mew-sinfo-set-scan-id nil)
+	    (mew-sinfo-set-mkdb mkdb)
+	    (mew-sinfo-set-flush flush)
+	    (message "Scanning %s..." bnm)
+	    (setq buf (get-buffer-create (mew-scan-buffer-name bnm)))
+	    (save-excursion
+	      (set-buffer buf)
+	      (mew-erase-buffer))
+	    (setq args (append (list "-b" mew-mail-path "-c" mew-news-path
+				     "-w" (int-to-string mew-scan-wait-for)
+				     "-f" (mapconcat (function identity)
+						     (nthcdr 2 mew-scan-fields)
+						     ","))
+			       args))
+	    (setq process (apply (function start-process)
+				 (mew-scan-info-name bnm)
+				 buf mew-prog-mewls args))
+	    (mew-summary-lock process "Scanning")
+	    (mew-set-process-cs process mew-cs-text-for-read mew-cs-dummy)
+	    ;; text may be broken, so undecided is very dangerous!
+	    (set-process-filter   process 'mew-scan-filter)
+	    (set-process-sentinel process 'mew-scan-sentinel)
+	    (process-kill-without-query process))
+	(quit
+	 (set-process-sentinel process nil)
+	 (mew-sinfo-set-start-point nil)
+	 (mew-sinfo-set-mkdb nil)
+	 (mew-sinfo-set-flush nil)
+	 (mew-summary-unlock)))))))
 
 ;; See also mew-summary-cook-region
 (defun mew-scan-insert-line (folder vec width lmsg virtualp)
-  (save-excursion
-    (set-buffer folder)
-    (let* ((after-change-functions nil)
-	   (line (mew-scan-get-line vec width virtualp))
-	   (opos (point))
-	   (omax (point-max))
-	   beg med face mark)
-      (mew-elet
-       (cond
-	(lmsg
-	 (goto-char (point-min))
-	 (if (not (re-search-forward (mew-regex-jmp-msg lmsg) nil t))
-	     (goto-char (point-max)) ;; xxx
-	   (beginning-of-line)
+  (when (get-buffer folder)
+    (save-excursion
+      (set-buffer folder)
+      (let* ((after-change-functions nil)
+	     (line (mew-scan-get-line vec width virtualp))
+	     (opos (point))
+	     (omax (point-max))
+	     beg med face mark)
+	(mew-elet
+	 (if (null lmsg)
+	     (goto-char (point-max))
+	   ;; a message marked with 'T'.
+	   (goto-char (point-min))
+	   (if (not (re-search-forward (mew-regex-jmp-msg lmsg) nil t))
+	       (goto-char (point-max));; xxx
+	     (beginning-of-line)
+	     (setq beg (point))
+	     (forward-line)
+	     ;; To avoid inserting a line AFTER the cursor underline,
+	     ;; keep this line and make it invisible.
+	     (put-text-property beg (point) 'invisible t)
+	     (forward-line -1)))
+	 (setq beg (point))
+	 ;; To "insert" just after mew-marker-decode-syntax-end.
+	 (insert (car line))
+	 (setq med (point))
+	 (goto-char beg)
+	 (mew-front-nonsticky beg med);; for XEmacs
+	 (if (and mew-use-highlight-mark
+		  (looking-at mew-regex-msg-mark)
+		  (setq mark (string-to-char (mew-match 2)))
+		  (setq face (mew-highlight-mark-get-face mark)))
+	     (put-text-property beg med 'face face))
+	 (if mew-use-highlight-mouse-line
+	     (put-text-property
+	      beg med 'mouse-face mew-highlight-mouse-line-face))
+	 (goto-char med)
+	 (insert (cdr line))
+	 (put-text-property med (1- (point)) 'invisible t)
+	 ;; Removing the invisible line.
+	 (when lmsg
 	   (setq beg (point))
 	   (forward-line)
 	   (delete-region beg (point))))
-	(t
-	 (goto-char (point-max))))
-       (setq beg (point))
-       (insert (car line))
-       (setq med (point))
-       (goto-char beg)
-       (mew-front-nonsticky beg med) ;; for XEmacs
-       (if (and mew-use-highlight-mark
-		(looking-at mew-regex-msg-mark)
- 		(setq mark (string-to-char (mew-match 2)))
- 		(setq face (mew-highlight-mark-get-face mark)))
- 	   (put-text-property beg med 'face face))
-       (if mew-use-highlight-mouse-line
-	   (put-text-property
-	    beg med 'mouse-face mew-highlight-mouse-line-face))
-       (goto-char med)
-       (insert (cdr line))
-       (put-text-property med (1- (point)) 'invisible t))
-      (if (or (eq opos (mew-sinfo-get-start-point))
-	      (/= opos omax))
-	  ;; move the cursor to the original position.
-	  (goto-char opos)))))
+	(if (or (eq opos (mew-sinfo-get-start-point))
+		(/= opos omax))
+	    ;; move the cursor to the original position.
+	    (goto-char opos))))))
 
 (defun mew-scan-filter (process string)
   (let* ((width (1- (mew-scan-width))) ;; xxx
@@ -723,76 +771,37 @@ current position."
   (let* ((pnm (process-name process))
 	 (folder (mew-scan-pnm-to-bnm pnm))
 	 (virtualp (mew-folder-virtualp folder))
-	 opos reviews)
+	 opos mdb ent msg mrk flush)
     (mew-filter
      (mew-set-buffer-multibyte t)
      (set-buffer folder)
      (setq opos (point))
-     (setq reviews (mew-sinfo-get-reviews))
-     (while reviews
-       (goto-char 1)
-       (forward-line (1- (car reviews)))
-       (mew-summary-mark-as mew-mark-collect)
-       (setq reviews (cdr reviews)))
-     (mew-summary-folder-cache-save)
+     (setq mdb (mew-sinfo-get-mkdb))
+     (setq flush (mew-sinfo-get-flush))
+     (goto-char (point-min))
+     (while mdb
+       (setq ent (car mdb))
+       (setq mdb (cdr mdb))
+       (setq msg (car ent))
+       (setq mrk (cdr ent))
+       (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
+	 (mew-summary-mark-as mrk 'force)
+	 (forward-line)))
      (goto-char opos)
-     (set-buffer-modified-p nil)
      (if (and virtualp (mew-vinfo-get-func))
 	 (funcall (mew-vinfo-get-func)))
      (mew-vinfo-set-func nil)
      (mew-sinfo-set-start-point nil)
-     (mew-sinfo-set-reviews nil)
-     (setq mew-summary-buffer-process nil)
-     (mew-touch-folder folder)
-     (message "Listing %s ... done" folder)
-     (run-hooks 'mew-scan-sentinel-hook))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Process control
-;;; mew-summary-buffer-process is a key to see if exclusive
-;;
-
-(defun mew-summary-exclusive-p ()
-  (cond
-   ((eq mew-summary-buffer-process t)
-    (message "Try again later.")
-    nil) ;; not exclusive
-   ((processp mew-summary-buffer-process)
-    (message (substitute-command-keys
-	      "\\<mew-summary-mode-map>Another process is running. Try later or type \\[mew-summary-kill-subprocess] to kill it."))
-    nil) ;; not exclusive
-   (t t))) ;; exclusive
-
-(defun mew-summary-kill-subprocess (&optional no-msg)
-  "\\<mew-summary-mode-map>
-Kill a process in Summary mode.
-Sometime a process accidentally remains in Summary mode. 
-In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
-Use this command to solve this problem."
-  (interactive)
-  (unwind-protect
-      (let ((process mew-summary-buffer-process)
-	    pnm sshpro)
-	(if (null (processp process))
-	    (or no-msg (message "No process to kill. This buffer is unlocked anyway."))
-	  (cond
-	   ((eq process t)
-	    ;; Emacs is locking
-	    )
-	   ((memq (process-status process) '(open closed))
-	    ;; Network process
-	    (setq pnm (process-name process))
-	    (if (string-match mew-pop-info-prefix pnm)
-		(setq sshpro (mew-pop-get-ssh-process pnm)))
-	    (if (processp sshpro)
-		(delete-process sshpro))
-	    (delete-process process))
-	   (t 
-	    ;; Local process
-	    (kill-process process)))
-	  (or no-msg (message "The process was killed"))))
-    (setq mew-summary-buffer-process nil)))
+     (mew-sinfo-set-mkdb nil)
+     (mew-sinfo-set-flush nil)
+     (mew-summary-folder-cache-save)
+     (set-buffer-modified-p nil)
+     (mew-summary-unlock)
+     (message "Scanning %s...done" folder)
+     (run-hooks 'mew-scan-sentinel-hook)
+     (when (and mew-auto-flush-queue flush)
+       (mew-smtp-flush-queue
+	(mew-queue-folder mew-case-output) mew-case-output)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -838,46 +847,48 @@ Use this command to solve this problem."
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
 	 (cache (mew-expand-folder folder mew-summary-cache-file)))
-    (if (and (mew-summary-folder-cache-updatep folder)
-	     (file-readable-p cache)
-	     (mew-summary-folder-cache-newp))
-	(mew-elet
-	 (mew-erase-buffer)
-	 (mew-frwlet
-	  mew-cs-m17n mew-cs-dummy
-	  (insert-file-contents cache))
-	 (mew-sinfo-set-cache-time (mew-file-get-time cache))
-	 (setq mew-summary-buffer-raw t)
-	 (mew-mark-undo-mark mew-mark-refile 'no-msg)
-	 (set-buffer-modified-p nil)))))
+    (when (and (mew-summary-folder-cache-updatep folder)
+	       (file-readable-p cache)
+	       (mew-summary-folder-cache-newp))
+      (mew-elet
+       (mew-erase-buffer)
+       (mew-frwlet
+	mew-cs-m17n mew-cs-dummy
+	(insert-file-contents cache))
+       (mew-sinfo-set-cache-time (mew-file-get-time cache))
+       (setq mew-summary-buffer-raw t)
+       (mew-mark-undo-mark mew-mark-refile 'no-msg)
+       (set-buffer-modified-p nil)))))
 
 (defun mew-summary-folder-cache-save ()
   (let* ((folder (mew-summary-folder-name 'ext))
 	 (cache (mew-expand-folder folder mew-summary-cache-file)))
-    (if (and (mew-summary-folder-cache-updatep folder)
-	     (file-writable-p cache))
-	(save-restriction
-	  (widen)
-	  (if (mew-decode-syntax-p)
-	      (let ((cbuf (current-buffer))
-		    (min (point-min))
-		    (max (point-max))
-		    (beg (mew-decode-syntax-begin))
-		    (end (mew-decode-syntax-end))
-		    (tmpbuf (generate-new-buffer mew-buffer-prefix)))
-		(set-buffer tmpbuf)
-		(mew-erase-buffer)
-		(mew-insert-buffer-substring cbuf min beg)
-		(mew-insert-buffer-substring cbuf end max)
-		(mew-frwlet
-		 mew-cs-dummy mew-cs-m17n
-		 (write-region (point-min) (point-max) cache nil 'no-msg))
-		(set-buffer cbuf)
-		(mew-remove-buffer tmpbuf))
-	    (mew-frwlet
-	     mew-cs-dummy mew-cs-m17n
-	     (write-region (point-min) (point-max) cache nil 'no-msg)))
-	  (mew-sinfo-set-cache-time (mew-file-get-time cache))))))
+    (when (and (mew-summary-folder-cache-updatep folder)
+	       (file-writable-p cache))
+      (mew-touch-folder folder)
+      (save-restriction
+	(widen)
+	(if (mew-decode-syntax-p)
+	    (let ((cbuf (current-buffer))
+		  (min (point-min))
+		  (max (point-max))
+		  (beg (mew-decode-syntax-begin))
+		  (end (mew-decode-syntax-end))
+		  (tmpbuf (generate-new-buffer mew-buffer-prefix)))
+	      (set-buffer tmpbuf)
+	      (mew-erase-buffer)
+	      (mew-insert-buffer-substring cbuf min beg)
+	      (mew-insert-buffer-substring cbuf end max)
+	      (mew-frwlet
+	       mew-cs-dummy mew-cs-m17n
+	       (write-region (point-min) (point-max) cache nil 'no-msg))
+	      (set-buffer cbuf)
+	      (mew-remove-buffer tmpbuf))
+	  (mew-frwlet
+	   mew-cs-dummy mew-cs-m17n
+	   (write-region (point-min) (point-max) cache nil 'no-msg))
+	  (set-file-modes cache mew-file-mode))
+	(mew-sinfo-set-cache-time (mew-file-get-time cache))))))
 
 (provide 'mew-scan)
 
