@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Dec 14, 1999
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -56,7 +55,7 @@
   (if (string-match "/\\([0-9]+\\)$" pnm) (mew-match 1 pnm)))
 
 ;; user@host. port should be used here.
-;; However, SSH doesn't symbolic port name "ssh" for the "-p" option
+;; However, SSH does not symbolic port name "ssh" for the "-p" option
 ;; while OpenSSH does. Sigh...
 (defsubst mew-ssh-passtag (name)
   (if (string-match (format "^%s/[^/]+/" mew-ssh-info-prefix) name)
@@ -98,7 +97,7 @@ after '/'. "
 	  (process-connection-type mew-connection-type2)
 	  (i 0) (N mew-ssh-process-exec-cnt)
 	  name pnm pro status)
-      (message "Connecting to the SSH server ... ")
+      (message "Connecting to the SSH server...")
       (catch 'loop
 	(while (< i N)
 	  (setq name (mew-ssh-info-name sshserver localport))
@@ -118,9 +117,11 @@ after '/'. "
 	  (mew-timing)
 	  (cond
 	   ((not (processp pro))
+	    (message "Connecting to the SSH server...FAILED")
 	    (throw 'loop nil))
 	   ((not (memq (process-status pro) '(run)))
 	    (delete-process pro)
+	    (message "Connecting to the SSH server...FAILED")
 	    (throw 'loop nil)))
 	  ;; ssh is now running.
 	  (process-kill-without-query pro)
@@ -136,10 +137,12 @@ after '/'. "
 	   ((eq status t)
 	    (throw 'loop pro)) ;; return value
 	   ((not (eq status 'bound))
+	    (message "Connecting to the SSH server...FAILED")
 	    (throw 'loop nil))) ;; return value
 	  ;; 'bound
 	  (setq localport (1+ localport))
 	  (setq i (1+ i))
+	  (message "Connecting to the SSH server...FAILED")
 	  nil)))))) ;; return value
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -167,21 +170,21 @@ after '/'. "
       (if (and (processp process)
 	       (memq (process-status process) '(run)))
 	  (progn
-	    (message "Sending your SSH password to the SSH server ... ")
+	    (message "Sending password to the SSH server...")
 	    (process-send-string process (concat pass "\n")))
 	(mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
-	(message "Can't find ssh process")))
+	(message "Cannot find SSH process")))
      ((or (string-match (mew-ssh-get mew-ssh-msg-denied) string)
 	  (string-match (mew-ssh-get mew-ssh-msg-refused) string))
-      (message "Can't connect to the SSH server")
+      (message "Cannot connect to the SSH server")
       (mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
       (mew-ssh-set-status pnm 'denied))
      ((string-match (mew-ssh-get mew-ssh-msg-connected) string)
       (mew-ssh-set-status pnm t)
-      (message "Connecting to the SSH server ... done"))
+      (message "Connecting to the SSH server...done"))
      ((string-match "Local: bind: Address already in use" string)
       (mew-ssh-set-status pnm 'bound)
-      (message "Connecting to the SSH server ... done")))))
+      (message "Connecting to the SSH server...done")))))
 
 (defun mew-ssh-sentinel (process event)
   (let ((pnm (process-name process)))
