@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Feb  1, 1999
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -42,7 +41,7 @@ If you have bogus messages and the second pass is carried out, thread
 structure MAY loop. This results in an infinite loop of visualizing 
 threads (not making threads). 
 
-Mew doesn't provide any loop detection/avoidance mechanism. So, you
+Mew does not provide any loop detection/avoidance mechanism. So, you
 should understand this risk."
   :group 'mew-summary
   :type 'boolean)
@@ -89,8 +88,7 @@ All members must have the same length."
 
 
 (defvar mew-thread-indent-length nil)
-(defvar mew-thread-indent-regex nil)
-(defvar mew-thread-indent-lregex nil)
+(defvar mew-thread-indent-list nil)
 
 (defvar mew-use-thread-separator nil)
 (defvar mew-thread-separator "--")
@@ -150,23 +148,12 @@ All members must have the same length."
 	  (idt2 (aref mew-fancy-thread-indent-strings 1))
 	  (idt3 (aref mew-fancy-thread-indent-strings 2))
 	  (idt4 (aref mew-fancy-thread-indent-strings 3)))
-      (if (and (= (length idt1) (length idt2))
-	       (= (length idt2) (length idt3))
-	       (= (length idt3) (length idt4)))
-	  ()
+      (unless (and (= (length idt1) (length idt2))
+		   (= (length idt2) (length idt3))
+		   (= (length idt3) (length idt4)))
 	(error
-	 "\
-All members of mew-fancy-thread-indent-strings must have the same length."))
-      (setq mew-thread-indent-length (length idt1))
-      (setq mew-thread-indent-regex
-	    (concat "\\(" (regexp-quote idt1) "\\|" (regexp-quote idt2)
-		    "\\|" (regexp-quote idt3) "\\|" (regexp-quote idt4) "\\)"))
-      (setq mew-thread-indent-lregex
-	    (concat "[^"
-		    (regexp-quote
-		     (format "%c%c%c%c"
-			     (aref idt1 0) (aref idt2 0)
-			     (aref idt3 0) (aref idt4 0))) "]"))))
+	 "All members of mew-fancy-thread-indent-strings must have the same length."))
+      (setq mew-thread-indent-length (length idt1))))
    (t
     (let* ((i 0)
 	   (size mew-thread-indent-array-size)
@@ -177,12 +164,7 @@ All members of mew-fancy-thread-indent-strings must have the same length."))
 	(setq i (1+ i))
 	(setq indent (concat indent mew-thread-indent-string)))
       (setq mew-thread-indent-array array))
-    (setq mew-thread-indent-length (length mew-thread-indent-string))
-    (setq mew-thread-indent-regex (regexp-quote mew-thread-indent-string))
-    (setq mew-thread-indent-lregex
-	  (concat "[^"
-		  (regexp-quote
-		   (format "%c" (aref mew-thread-indent-string 0))) "]")))))
+    (setq mew-thread-indent-length (length mew-thread-indent-string)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -265,9 +247,12 @@ threads are created, see 'mew-use-complete-thread'."
 	    (vfolder (concat "+" folder))
 	    (disp (mew-sinfo-get-disp-msg))
 	    (ctime (mew-sinfo-get-cache-time))
-	    tm1 tm2 tm3 tm4 tm5 tm6
-	    db size top start me prnt prnt-cld my-id prnt-id func msg line)
-       (message "Making thread (first pass) ... ")
+	    (column (mew-summary-scan-form folder 'column))
+	    (numlen (mew-summary-scan-form-num folder))
+ 	    overnum spacer indentlst
+	    db size top start me prnt prnt-cld my-id prnt-id func msg line
+	    tm1 tm2 tm3 tm4 tm5 tm6)
+       (message "Making thread (first pass)...")
        (save-restriction
 	 (narrow-to-region beg end)
 	 (setq size (mew-count-lines beg end))
@@ -282,6 +267,19 @@ threads are created, see 'mew-use-complete-thread'."
 	   (setq size 18211)))
 	 (setq db (make-vector size 0))
 	 (save-excursion
+	   ;; Calculate message number width
+ 	   (goto-char (point-max))
+ 	   (mew-summary-goto-message)
+ 	   (when (looking-at mew-regex-msg)
+ 	     (setq overnum (length (mew-match 1)))
+ 	     (if (<= overnum numlen)
+		 (setq overnum nil)
+	       (setq column (+ column (- overnum numlen)))
+	       (setq spacer (make-vector overnum nil))
+	       (setq size 0)
+	       (while (< size overnum)
+		 (aset spacer size (make-string size ? ))
+		 (setq size (1+ size)))))
 	   (goto-char (point-min))
 	   (if mark
 	       (setq func (lambda ()
@@ -301,8 +299,13 @@ threads are created, see 'mew-use-complete-thread'."
 	       ;; This is faster than inheriting properties.
 	       (setq line (mew-buffer-substring start (point)))
 	       (setq me (mew-thread-make-entry))
-	       (mew-thread-set-msg  me msg)
-	       (mew-thread-set-line me line)
+	       (mew-thread-set-msg me msg)
+	       (if (and overnum (> overnum (length msg)))
+		   (mew-thread-set-line
+		    me
+		    (concat (aref spacer (- overnum (max (length msg) numlen)))
+			    line))
+ 		 (mew-thread-set-line me line))
 	       (if (string= my-id "")
 		   (setq top (cons me top))
 		 (mew-thread-set-myid me my-id)
@@ -319,7 +322,7 @@ threads are created, see 'mew-use-complete-thread'."
 		       (mew-thread-set-child prnt (list me))))))))))
        ;;
        (if (null me)
- 	   (message "No target messages.")
+ 	   (message "No target messages")
 	 (setq tm2 (current-time))
 	 (mew-summary-switch-to-folder vfolder 'thread)
 	 (mew-erase-buffer)
@@ -329,8 +332,9 @@ threads are created, see 'mew-use-complete-thread'."
 	 (setq mew-summary-buffer-raw t)
 	 (mew-vinfo-set-top nil)
 	 (mew-vinfo-set-db db)
+ 	 (mew-vinfo-set-column column)
 	 ;;
-	 (message "Making thread (second pass) ... ")
+	 (message "Making thread (second pass)...")
 	 (setq tm3 (current-time))
 	 (if (null mew-use-complete-thread)
 	     (mew-vinfo-set-top (nreverse top))
@@ -351,31 +355,24 @@ threads are created, see 'mew-use-complete-thread'."
 	     (setq top (cdr top))))
 	 ;;
 	 (setq tm4 (current-time))
-	 (message "Displaying thread ... ")
+	 (message "Displaying thread...")
 	 (setq tm5 (current-time))
+ 	 (setq mew-thread-indent-list nil)
 	 (cond
 	  (mew-use-fancy-thread
 	   (mew-summary-fancy-thread-print-top
-	    (mew-vinfo-get-top) folder
-	    (mew-summary-scan-form folder 'column)))
+ 	    (mew-vinfo-get-top) folder column))
 	  (t
 	   (mew-summary-thread-print
-	    (mew-vinfo-get-top) folder
-	    (mew-summary-scan-form folder 'column))))
+	    (mew-vinfo-get-top) folder column)))
+	 ;; Indent property insert
+ 	 (mew-elet
+ 	  (while (setq indentlst (car mew-thread-indent-list))
+ 	    (put-text-property (nth 0 indentlst) (nth 1 indentlst)
+ 			       'mew-thread-indent (nth 2 indentlst))
+ 	    (setq mew-thread-indent-list (cdr mew-thread-indent-list))))
 	 ;;
-	 (if mark (mew-mark-undo-mark mew-mark-review))
-	 (when mew-use-thread-cursor
-	   (mew-elet
-	    (let ((column (mew-summary-scan-form folder 'column)))
-	      (goto-char (point-min))
-	      (while (not (eobp))
-		(when (and (looking-at mew-regex-msg)
-			   (= (move-to-column column) column)
-			   (re-search-forward mew-thread-indent-lregex nil t))
-		  (put-text-property (match-beginning 0)
-				     (1+ (match-beginning 0))
-				     'mew-thread-indent t))
-		(forward-line)))))
+	 (when mark (mew-mark-undo-mark mew-mark-review))
 	 (setq tm6 (current-time))
 	 (when mew-gemacs-p
 	   (jit-lock-register 'mew-summary-cook-region))
@@ -385,7 +382,7 @@ threads are created, see 'mew-use-complete-thread'."
 	   (goto-char (point-max)))
 	 (mew-summary-display nil)
 	 (mew-summary-thread-move-cursor)
-	 (message "Displaying thread ... done")
+	 (message "Displaying thread...done")
 	 (run-hooks 'mew-thread-display-hook)
 	 (when (mew-debug 'thread)
 	   (let* ((t1 (mew-time-calc tm2 tm1))
@@ -400,7 +397,7 @@ threads are created, see 'mew-use-complete-thread'."
 
 (defun mew-summary-thread-print (tree folder column)
   (let ((tree-stack nil)
-	(level 0))
+	(level 0) pos)
     (while tree
       (let* ((me (car tree))
 	     (cld (mew-thread-get-child me)))
@@ -409,8 +406,17 @@ threads are created, see 'mew-use-complete-thread'."
 	 (insert (mew-thread-get-line me))
 	 (forward-line -1)
 	 (move-to-column column)
-	 (if (< level mew-thread-indent-array-size)
-	     (insert (aref mew-thread-indent-array level))
+ 	 (cond
+ 	  ((= level 0)
+ 	   (setq mew-thread-indent-list
+ 		 (cons (list (point) (1+ (point)) 0) mew-thread-indent-list)))
+ 	  ((< level mew-thread-indent-array-size)
+ 	   (setq pos (point))
+ 	   (insert (aref mew-thread-indent-array level))
+ 	   (setq mew-thread-indent-list
+ 		 (cons (list pos (point) level) mew-thread-indent-list)))
+ 	  (t
+ 	   (setq pos (point))
 	   (let* ((i 0)
 		  (max (1- mew-thread-indent-array-size)) ;; take care
 		  (lim (/ level max))
@@ -418,7 +424,9 @@ threads are created, see 'mew-use-complete-thread'."
 	     (while (< i lim)
 	       (insert (aref mew-thread-indent-array max))
 	       (setq i (1+ i)))
-	     (insert (aref mew-thread-indent-array j))))
+ 	     (insert (aref mew-thread-indent-array j))
+ 	     (setq mew-thread-indent-list
+ 		   (cons (list pos (point) level) mew-thread-indent-list)))))
 	 (if (search-forward "\r" nil t)
 	     (insert " " folder " " (mew-thread-get-msg me)))
 	 (forward-line))
@@ -448,6 +456,8 @@ threads are created, see 'mew-use-complete-thread'."
        (insert (mew-thread-get-line me))
        (forward-line -1)
        (move-to-column column)
+       (setq mew-thread-indent-list
+ 	     (cons (list (point) (1+ (point)) 0) mew-thread-indent-list))
        (if (search-forward "\r" nil t)
 	   (insert " " folder " " (mew-thread-get-msg me)))
        (forward-line))
@@ -457,7 +467,7 @@ threads are created, see 'mew-use-complete-thread'."
 (defun mew-summary-fancy-thread-print-tree (tree folder column)
   (let ((tree-stack nil)
 	(prefix "")
-	(level 0))
+	(level 1) pos)
     (while tree
       (let* ((me (car tree))
 	     (next (cdr tree))
@@ -466,9 +476,12 @@ threads are created, see 'mew-use-complete-thread'."
 	 (insert (mew-thread-get-line me))
 	 (forward-line -1)
 	 (move-to-column column)
+ 	 (setq pos (point))
 	 (if next
 	     (insert prefix (aref mew-fancy-thread-indent-strings 0))
 	   (insert prefix (aref mew-fancy-thread-indent-strings 1)))
+ 	 (setq mew-thread-indent-list
+ 	       (cons (list pos (point) level) mew-thread-indent-list))
 	 (if (search-forward "\r" nil t)
 	     (insert " " folder " " (mew-thread-get-msg me)))
 	 (forward-line))
@@ -520,8 +533,10 @@ threads are created, see 'mew-use-complete-thread'."
   (interactive)
   (mew-thread-only
    (let ((folders (mew-summary-refile-body nil nil nil 'no-mark))
-	 (alist (mew-thread-mark mew-mark-refile)))
-     (mew-refile-set-from-alist alist folders))))
+	 alist)
+     (when folders
+       (setq alist (mew-thread-mark mew-mark-refile))
+       (mew-refile-set-from-alist alist folders)))))
 
 (defun mew-refile-set-from-alist (alist folders)
   (let (ent fld msg msgs)
@@ -533,39 +548,40 @@ threads are created, see 'mew-use-complete-thread'."
       (while msgs
 	(setq msg (int-to-string (car msgs)))
 	(setq msgs (cdr msgs))
-	(mew-refile-reset fld msg)
-	(mew-refile-set fld msg folders)))))
+	(when (get-buffer fld)
+	  (save-excursion
+	    (set-buffer fld)
+	    (mew-refile-reset msg)
+	    (mew-refile-set msg folders)))))))
 
 (defun mew-thread-mark (mark)
   (mew-thread-only
    (mew-summary-msg-or-part
-    (let* ((folder (mew-summary-folder-name))
-	   (column (mew-summary-scan-form folder 'column))
-	   (regex "") fld msg alist)
+    (let ((column (mew-vinfo-get-column))
+	  indent cindent fld msg alist)
       (mew-summary-goto-message)
       (mew-summary-thread-move-cursor)
       (mew-decode-syntax-delete)
       (save-excursion
 	(move-to-column column)
-	(while (looking-at mew-thread-indent-regex)
-	  (setq regex (concat regex mew-thread-indent-regex))
-	  (forward-char mew-thread-indent-length))
-	(setq regex (concat regex mew-thread-indent-regex))
+ 	(setq indent (get-text-property (point) 'mew-thread-indent))
 	(mew-summary-mark-as mark 'force)
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
 	(mew-mark-alist-set alist fld msg)
 	(forward-line)
 	(catch 'loop
-	  (while t
+ 	  (while (not (eobp))
 	    (move-to-column column)
 	    (unless (mew-syntax-number)
-	      (if (not (looking-at regex))
-		  (throw 'loop nil)
-		(mew-summary-mark-as mark 'force)
-		(setq fld (mew-summary-folder-name))
-		(setq msg (mew-summary-message-number))
-		(mew-mark-alist-set alist fld msg)))
+	      (when (setq cindent
+			  (get-text-property (point) 'mew-thread-indent))
+		(if (>= indent cindent)
+		    (throw 'loop nil)
+		  (mew-summary-mark-as mark 'force)
+		  (setq fld (mew-summary-folder-name))
+		  (setq msg (mew-summary-message-number))
+		  (mew-mark-alist-set alist fld msg))))
 	    (forward-line)))
 	(mew-summary-mark-in-physical-alist alist mark))
       (mew-summary-goto-message)
@@ -577,36 +593,64 @@ threads are created, see 'mew-use-complete-thread'."
       (mew-summary-display-after mew-summary-mark-direction)
       alist))))
 
-;; XXX: This command is not efficient because unmarking is done by
-;; a loop, not a batch.
+(defsubst mew-thread-unmark-one ()
+  (beginning-of-line)
+  (when (re-search-forward mew-regex-msg nil t)
+    (mew-mark-delete-here)))
+
 (defun mew-thread-unmark ()
   "Unmark messages under this sub-thread."
   (interactive)
   (mew-thread-only
    (mew-summary-msg-or-part
-    (let* ((folder (mew-summary-folder-name))
-	   (column (mew-summary-scan-form folder 'column))
-	   (count 1)
-	   (regex ""))
+    (let ((column (mew-vinfo-get-column))
+	  fld msg alist indent cindent)
       (mew-summary-goto-message)
       (mew-summary-thread-move-cursor)
       (mew-decode-syntax-delete)
       (save-excursion
 	(move-to-column column)
-	(while (looking-at mew-thread-indent-regex)
-	  (setq regex (concat regex mew-thread-indent-regex))
-	  (forward-char mew-thread-indent-length))
-	(setq regex (concat regex mew-thread-indent-regex))
+ 	(setq indent (get-text-property (point) 'mew-thread-indent))
+	(setq fld (mew-summary-folder-name))
+	(setq msg (mew-summary-message-number))
+	(mew-mark-alist-set alist fld msg)
+	(mew-thread-unmark-one)
 	(forward-line)
 	(catch 'loop
-	  (while t
+ 	  (while (not (eobp))
 	    (move-to-column column)
-	    (unless (mew-syntax-number)
-	      (if (not (looking-at regex))
+	    (when (setq cindent (get-text-property
+				 (point) 'mew-thread-indent))
+	      (if (>= indent cindent)
 		  (throw 'loop nil)
-		(setq count (1+ count))))
-	    (forward-line))))
-      (mew-summary-undo count)))))
+		(setq fld (mew-summary-folder-name))
+		(setq msg (mew-summary-message-number))
+		(mew-mark-alist-set alist fld msg)
+		(mew-thread-unmark-one)))
+	    (forward-line)))
+	(mew-thread-unmark-physical-from-alist alist))))))
+
+(defun mew-thread-unmark-physical-from-alist (alist)
+  (let (ent fld msg msgs)
+    (while alist
+      (setq ent (car alist))
+      (setq alist (cdr alist))
+      (setq fld (car ent))
+      (setq msgs (sort (cdr ent) (function <)))
+      (when (get-buffer fld)
+	(set-buffer fld)
+	(save-excursion
+	  (goto-char (point-min))
+	  (while msgs
+	    (setq msg (int-to-string (car msgs)))
+	    (setq msgs (cdr msgs))
+	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
+	      (beginning-of-line)
+	      (when (eq mew-mark-refile
+			(and (looking-at mew-regex-msg-mark)
+			     (string-to-char (mew-match 2))))
+		(mew-refile-reset msg))
+	      (mew-thread-unmark-one))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -618,53 +662,41 @@ threads are created, see 'mew-use-complete-thread'."
 a top node, move onto the top of the previous thread."
   (interactive)
   (mew-thread-only
-   (let* ((folder (substring (mew-summary-folder-name 'ext) 1))
-	  (column (mew-summary-scan-form folder 'column))
-	  here)
+   (let (here pos)
      (mew-summary-goto-message)
      (save-excursion
        (mew-decode-syntax-delete)
-       (if (save-excursion
-	     (forward-line -1)
-	     (not (mew-summary-message-number)))
-	   (forward-line -1))
+       (beginning-of-line)
+       (setq pos (point))
        (catch 'loop
-	 (while (= (forward-line -1) 0)
-	   (move-to-column column)
-	   (unless (looking-at mew-thread-indent-regex)
-	     (throw 'loop (setq here (point)))))))
-     (if here
-	 (progn
-	   (goto-char here)
-	   (mew-summary-thread-move-cursor)
-	   (mew-summary-display nil))
-       (message "No more thread")))))
+ 	 (while (and (not (bobp))
+ 		     (setq pos (previous-single-property-change
+ 				pos 'mew-thread-indent)))
+ 	   (when (and pos (eq (get-text-property pos 'mew-thread-indent) 0))
+ 	     (throw 'loop (setq here pos))))))
+     (if (not here)
+	 (message "No more threads")
+       (goto-char here)
+       (mew-summary-thread-move-cursor)
+       (mew-summary-display nil)))))
 
 (defun mew-summary-thread-down ()
   "Move onto the top of the next thread."
   (interactive)
   (mew-thread-only
-   (let* ((folder (substring (mew-summary-folder-name 'ext) 1))
-	  (column (mew-summary-scan-form folder 'column))
-	  here)
+   (let (here)
      (mew-summary-goto-message)
      (save-excursion
        (mew-decode-syntax-delete)
-       (catch 'loop
-	 (while (= (forward-line) 0)
-	   (move-to-column column)
-	   (if (looking-at mew-thread-indent-regex)
-	       ()
-	     (if (eobp)
-		 (throw 'loop nil)
-	       (throw 'loop (setq here (point))))))))
-     (if here
-	 (progn
-	   (goto-char here)
-	   (if (not (mew-summary-message-number)) (forward-line))
-	   (mew-summary-thread-move-cursor)
-	   (mew-summary-display nil))
-       (message "No more thread")))))
+       (forward-line)
+       (setq here (text-property-any (point) (point-max)
+ 				     'mew-thread-indent 0)))
+     (if (not here)
+	 (message "No more threads")
+       (goto-char here)
+       (if (not (mew-summary-message-number)) (forward-line))
+       (mew-summary-thread-move-cursor)
+       (mew-summary-display nil)))))
 
 (defun mew-summary-thread-parent ()
   "Move onto the parent message of the current message."
@@ -674,9 +706,10 @@ a top node, move onto the top of the previous thread."
   (let ((pos (point)) key)
     (if (not (looking-at mew-regex-virtual-thread))
 	(message "No required info")
-      (setq key (format " <%s> " (mew-match 5)))
+      (setq key (mew-match 5))
       (if (string= key "")
 	  (message "No required info")
+	(setq key (format " <%s> " key))
 	(if (or (search-backward key nil t)
 		(progn
 		  (goto-char (point-max))
@@ -684,7 +717,7 @@ a top node, move onto the top of the previous thread."
 	    (progn
 	      (mew-summary-thread-move-cursor)
 	      (mew-summary-display nil)
-	      (message "A parent found"))
+	      (message "Parent found"))
 	  (goto-char pos)
 	  (message "No parent"))))))
 
@@ -696,9 +729,10 @@ a top node, move onto the top of the previous thread."
   (let ((pos (point)) key)
     (if (not (looking-at mew-regex-virtual-thread))
 	(message "No required info")
-      (setq key (format " <%s>\n" (mew-match 4)))
+      (setq key (mew-match 4))
       (if (string= key "")
 	  (message "No required info")
+	(setq key (format " <%s>\n" key))
  	(if (or (search-forward key nil t)
  		(progn
 		  (goto-char (point-min))
@@ -707,7 +741,7 @@ a top node, move onto the top of the previous thread."
 	      (forward-line -1) ;; due to \n search
 	      (mew-summary-thread-move-cursor)
 	      (mew-summary-display nil)
-	      (message "A child found"))
+	      (message "Child found"))
 	  (goto-char pos)
 	  (message "No child"))))))
 
@@ -717,14 +751,15 @@ a top node, move onto the top of the previous thread."
   (let ((pos (point)) key)
     (if (not (looking-at mew-regex-virtual-thread))
 	(message "No required info")
-      (setq key (format " <%s>\n" (mew-match 5)))
+      (setq key (mew-match 5))
       (if (string= key "")
 	  (message "No required info")
+	(setq key (format " <%s>\n" key))
 	(if (search-backward key nil t)
 	    (progn
 	      (mew-summary-thread-move-cursor)
 	      (mew-summary-display nil)
-	      (message "A sibling found"))
+	      (message "Sibling found"))
 	  (goto-char pos)
 	  (message "No sibling"))))))
 
@@ -734,16 +769,17 @@ a top node, move onto the top of the previous thread."
   (let ((pos (point)) key)
     (if (not (looking-at mew-regex-virtual-thread))
 	(message "No required info")
-      (setq key (format " <%s>\n" (mew-match 5)))
+      (setq key (mew-match 5))
       (if (string= key "")
 	  (message "No required info")
+	(setq key (format " <%s>\n" key))
 	(forward-line)
 	(if (search-forward key nil t)
 	    (progn
 	      (forward-line -1)
 	      (mew-summary-thread-move-cursor)
 	      (mew-summary-display nil)
-	      (message "A sibling found"))
+	      (message "Sibling found"))
 	  (goto-char pos)
 	  (message "No sibling"))))))
 
@@ -751,16 +787,18 @@ a top node, move onto the top of the previous thread."
 ;;;
 ;;; Thread sub-functions
 ;;;
-
 (defun mew-summary-thread-move-cursor ()
   "Move cursor after indentation of thread."
-      (beginning-of-line)
   (if (and mew-use-thread-cursor
 	   (mew-thread-p)
 	   (mew-summary-message-number))
-      (let ((pos (text-property-any (point) (progn (end-of-line) (point))
-				    'mew-thread-indent t)))
-	(if pos (goto-char pos) (beginning-of-line)))))
+      (let (indent)
+ 	(move-to-column (mew-vinfo-get-column))
+ 	(if (setq indent (get-text-property (point) 'mew-thread-indent))
+ 	    (unless (= indent 0)
+ 	      (goto-char (next-single-property-change (point) 'mew-thread-indent)))
+ 	  (beginning-of-line)))
+    (beginning-of-line)))
 
 (defun mew-summary-thread-get-msglst (tree &optional add-separator)
   "Get a list of message in the thread order specified by TREE."
