@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Dec  4, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -12,6 +11,7 @@
 
 (defvar mew-prog-plain    '(mew-mime-text/plain mew-mime-text/plain-ext))
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
+(defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-audio
@@ -19,12 +19,14 @@
 (defvar mew-prog-audio2
   `(,mew-prog-shell (,mew-prog-shell-arg "cat < /dev/audio") nil))
 (defvar mew-prog-image           '(mew-mime-image/* mew-mime-image/*-ext))
-(defvar mew-prog-mpeg            '("mpeg_play" () t))
+(defvar mew-prog-video           '("mpeg_play" () t))
 (defvar mew-prog-rfc822          'mew-mime-message/rfc822)
 (defvar mew-prog-external-body   '(mew-mime-external-body mew-mime-external-body-ext))
 (defvar mew-prog-delivery-status 'mew-mime-text/plain)
 (defvar mew-prog-postscript      '("gv" ("-geometry" "+0+0") t))
 (defvar mew-prog-pdf             '("acroread" ("-geometry" "+0+0") t))
+(defvar mew-prog-xml2            '(mew-mime-application/xml
+				   mew-mime-application/xml-ext))
 (defvar mew-prog-pgp-keys        '(mew-mime-pgp-keys mew-mime-pgp-keys-ext))
 (defvar mew-prog-octet-stream    'mew-mime-application/octet-stream)
 ;; Some broken UAs labels Office files as Application/Octet-Stream
@@ -36,14 +38,27 @@
 				   mew-mime-application/mspowerpoint-ext))
 
 ;;;
-;;; Text/Html, Image
+;;; Text/Html, Application/Xml, Image
 ;;;
 
+(defvar mew-format-html "%s.html")
+(defvar mew-format-xml  "%s.xml")
+
 (defvar mew-prog-text/html         'mew-mime-text/html-w3m) ;; See w3m.el
 (defvar mew-prog-text/html-ext     "netscape")
 (defvar mew-prog-text/html-ext-arg '("-remote" "openFile(%s)")
   "%s will be replaced with a file name.")
 
+(defvar mew-prog-text/xml         'mew-mime-text/html-w3m) ;; See w3m.el
+(defvar mew-prog-text/xml-ext     "netscape")
+(defvar mew-prog-text/xml-ext-arg '("-remote" "openFile(%s)")
+  "%s will be replaced with a file name.")
+
+(defvar mew-prog-application/xml         nil)
+(defvar mew-prog-application/xml-ext     "netscape")
+(defvar mew-prog-application/xml-ext-arg '("-remote" "openFile(%s)")
+  "%s will be replaced with a file name.")
+
 (defvar mew-prog-image/*         'mew-mime-image/*)
 (defvar mew-prog-image/*-ext     "xv")
 (defvar mew-prog-image/*-ext-arg '("-geometry" "+0+0"))
