@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Nov 22, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -131,6 +130,11 @@
   :group 'mew-folder
   :type 'string)
 
+(defcustom mew-mdrop-folder "+mdrop"
+  "*The maildrop folder of a POP server."
+  :group 'mew-folder
+  :type 'string)
+
 (defcustom mew-attach-folder "+attach"
   "*The folder where attachments are contained."
   :group 'mew-folder
@@ -138,7 +142,8 @@
 
 ;; must be here
 (defvar mew-basic-folders
-  `(,mew-inbox-folder ,mew-draft-folder ,mew-trash-folder ,mew-queue-folder))
+  `(,mew-inbox-folder ,mew-draft-folder ,mew-trash-folder
+		      ,mew-queue-folder ,mew-mdrop-folder))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -270,7 +275,7 @@ is as follows:
   :type 'hook)
 
 (defcustom mew-summary-ls-no-scan-hook nil
-  "*Hook called when mew-summary-ls doesn't scan a folder." 
+  "*Hook called when mew-summary-ls does not scan a folder." 
   :group 'mew-summary
   :type 'hook)
 
@@ -279,6 +284,16 @@ is as follows:
   :group 'mew-summary
   :type 'hook)
 
+(defcustom mew-sort-hook nil
+  "*Hook called when sort finished." 
+  :group 'mew-summary
+  :type 'hook)
+
+(defcustom mew-pack-hook nil
+  "*Hook called when pack finished." 
+  :group 'mew-summary
+  :type 'hook)
+
 (defcustom mew-summary-toggle-disp-msg-hook nil
   "*Hook called when mew-summary-toggle-disp-msg finished."
   :group 'mew-summary
@@ -484,8 +499,9 @@ Currently, \"CRAM-MD5\" can be used."
   :type '(repeat (choice (const "CRAM-MD5"))))
 
 (defcustom mew-pop-delete t
-  "*If nil, messages on the POP server are not deleted after
-retrieval. Otherwise they are deleted."
+  "*If t, messages on the POP server are deleted after retrieval. 
+If numeric, messages on the POP server are deleted N days after
+the first access. Otherwise they are not deleted."
   :group 'mew-pop
   :type 'boolean)
 
@@ -495,9 +511,6 @@ retrieval. Otherwise they are deleted."
   :group 'mew-pop
   :type 'integer)
 
-(defvar mew-pop-size-over-message
-  "[[[TOO LARGE. Truncated BY Mew. Type '%s' to get this message.]]]\n\n")
-
 (defcustom mew-pop-body-lines 40
   "*The limit of body lines to get when the size of message
 exceeds 'mew-pop-size'."
@@ -545,30 +558,47 @@ by one mark, this variable MUST be consist of 'num and 'mark."
                         (choice (const num) (const mark)))))
 
 (defconst mew-custom-type-of-scan-form
-        '(choice
-          (cons integer
-                (repeat (choice string
-                                (list integer
-                                      (choice (const type) (const time)
-                                              (const date) (const year)
-                                              (const size) (const from)
-                                              (const subj)))
-                                 (choice (const type) (const time)
-                                         (const date) (const year)
-                                         (const size) (const from)
-                                         (const subj)))))
-          (repeat (choice string
-                          (list integer (choice (const type) (const time)
-                                                (const date) (const year)
-                                                (const size) (const from)
-                                                (const subj)))
-                          (choice (const type) (const time)
-                                  (const date) (const year)
-                                  (const size) (const from)
-                                  (const subj))))))
-                          
+  '(choice
+    (cons :tag "Setup message number column"
+	  (integer :tag "Message number column" )
+	  (repeat (choice (string :tag "Insert string")
+			  (const :tag "Thread indent" t)
+			  (list :tag "Width and symbol"
+				:value (0 type)
+				(integer :tag "Display width")
+				(choice :tag "Symbol"
+					:value type
+					(const type) (const time)
+					(const date) (const year)
+					(const size) (const from)
+					(const subj) (symbol :tag "Other")))
+			  (choice :tag "Symbol"
+				  :value type
+				  (const type) (const time)
+				  (const date) (const year)
+				  (const size) (const from)
+				  (const subj) (symbol :tag "Other")))))
+    (repeat :tag "Use default number column"
+	    (choice (integer :tag "Message number column")
+		    (string :tag "Insert string")
+		    (const :tag "Thread indent" t)
+		    (list :tag "Width and symbol"
+			  :value (0 type)
+			  (integer :tag "Display width")
+			  (choice :tag "Symbol"
+				  :value type
+				  (const type) (const time)
+				  (const date) (const year)
+				  (const size) (const from)
+				  (const subj) (symbol :tag "Other")))
+		    (choice :tag "Symbol"
+			    :value type
+			    (const type) (const time)
+			    (const date) (const year)
+			    (const size) (const from)
+			    (const subj) (symbol :tag "Other"))))))
 
-(defcustom mew-scan-form '(type (5 date) " " (14 from) " " (0 subj))
+(defcustom mew-scan-form '(type (5 date) " " (14 from) " " t (0 subj))
   "*The format in Summary mode, called scan-form. Scan-form is a list
 of list, symbol, and string.
 
@@ -594,7 +624,7 @@ ASCII string.
 
 0 means the remaining width and is treated as a positive value.
 
-(c) A symbol is equivalent to (1 symbol).
+(c) A symbol except 't' is equivalent to (1 symbol).
 
 (d) If the first element of scan-form is an integer N, this integer is
 converted into '((N num) mark). Otherwise, the value of
@@ -602,8 +632,10 @@ mew-scan-form-header is inserted in front of scan-form value when used
 as scan format. Also note that users MUST not use the symbol 'num and
 'mark in 'mew-scan-form'. See also 'mew-scan-form-header'.
 
+(e) 't' means the position where 'mew-thread-indent-string' is inserted.
+
 An example is as follows:
-	'(type (5 date) \" \" (-4 size) \" \" (14 from) \" \" (0 subj))
+	'(type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (0 subj))
 
 You can also set this value in 'mew-scan-form-list'."
   :group 'mew-summary
@@ -611,7 +643,7 @@ You can also set this value in 'mew-scan-form-list'."
 
 (defcustom mew-scan-form-list nil
   "*A list to define 'mew-scan-form' for each folder. Each component
-is (key scan-form column). 
+is (key scan-form). 
 
 If KEY is t, all folders matches it. The corresponding SCAN-FORM is
 returned always.
@@ -635,21 +667,22 @@ Candicate values for 'mew-scan-form-list-string-type' and
 
 SCAN-FORM is a list. See 'mew-scan-form' for more information.
 
-COLUMN is a position where 'mew-thread-indent-string's are inserted.
-See also 'mew-thread-column'.
-
 An example is as follows:
       '(((\"+inbox\")
-	 (type (5 date) \" \" (-4 size) \" \" (14 from) \" \" (0 subj)) 33)
+	 (type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (0 subj)))
 	(t
-	 (type (5 date) \" \" (14 from) \" \" (0 subj)) 28))
+	 (type (5 date) \" \" (14 from) \" \" t (0 subj))))
 "
   :group 'mew-summary
-  :type '(choice (const nil)
-                (repeat
-                  (list (choice (const t) string (repeat string))
-                        mew-custom-type-of-scan-form
-                        integer))))
+  :type `(choice
+	  (const nil)
+	  (repeat
+	   (list (choice (string :tag "Folder")
+			 (repeat :tag "Folders" (string :tag "Folder"))
+			 (const :tag "Other" t))
+		 ,mew-custom-type-of-scan-form
+		 (choice (integer :tag "Thread indent column")
+			 (const nil))))))
 
 (defcustom mew-scan-form-list-string-type 'regex
   "*A value to specify an action if the key is a string in 
@@ -776,8 +809,8 @@ between Emacs and mewls smoother.")
 
 (defcustom mew-thread-column 28
   "A position where 'mew-thread-indent-string' is inserted
-for indentation of thread. You can also set this value in
-'mew-scan-form-list'."
+for indentation of thread. This valuable is for backward compatibility
+only. Use 't' in 'mew-scan-form instead."
   :group 'mew-summary
   :type 'integer)
 
@@ -791,7 +824,7 @@ for indentation of thread. You can also set this value in
   :group 'mew-env
   :type 'boolean)
 
-(defcustom mew-demo-picture mew-xemacs-p
+(defcustom mew-demo-picture (or mew-xemacs-p mew-gemacs-p)
   "*A picture of cats is displayed if *non-nil* on XEmacs."
   :group 'mew-env
   :type 'boolean)
@@ -865,13 +898,20 @@ to this file."
 ;;; Ask?
 ;;; 
 
-(defcustom mew-ask-cc t
-  "*Prompt user for CC: field if *non-nil*."
+(defcustom mew-ask-subject nil
+  "*If *non-nil* and Subject: is empty, ask a user its value."
   :group 'mew-draft
   :type 'boolean)
 
-(defcustom mew-ask-subject nil
-  "*Prompt user for Subject: field when send if *non-nil*."
+(defcustom mew-ask-fcc nil
+  "*If *non-nil* and a folder on Fcc: does not exist, ask a user to
+create it. If nil, folders which are not present are created without
+any query."
+  :group 'mew-draft
+  :type 'boolean)
+
+(defcustom mew-ask-newsgroups nil
+  "*If *non-nil* and Newsgroups: exists, ask a user to send this really."
   :group 'mew-draft
   :type 'boolean)
 
@@ -881,11 +921,6 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-ask-newsgroups nil
-  "*If *non-nil*, prompt user if he/she want to include Newsgroups: field."
-  :group 'mew-draft
-  :type 'boolean)
-
 (defcustom mew-ask-cite-prefix nil
   "*If *non-nil*, ask citation prefix when cite a message."
   :group 'mew-cite
@@ -911,6 +946,11 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-ask-mark-process nil
+  "*If *non-nil*, mark processing when exiting Emacs."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Use
@@ -948,6 +988,12 @@ into the 7bit form."
   :group 'mew-draft
   :type 'boolean)
 
+(defcustom mew-use-charset-sanity-check t
+  "*If non-nil, the sanity check of character set is used when
+composing."
+  :group 'mew-draft
+  :type 'boolean)
+
 (defcustom mew-use-burst-folder-history nil
   "*If non-nil, the folder specified at the previous burst job
 is used as a folder candidate instead of inbox."
@@ -978,7 +1024,7 @@ your POP server becomes non-zero. To use this feature,
 If From: contains one address and SMTP MAIL FROM
 ('mew-smtp-mail-from') is different from it, this variable effects.
 If non-nil, Mew adds Sender: and specifies SMTP MAIL FROM to its value.
-Otherwise, Mew doesn't add Sender:."
+Otherwise, Mew does not add Sender:."
   :group 'mew-draft
   :type 'boolean)
 
@@ -1093,6 +1139,11 @@ to a region."
 
 (defvar mew-highlight-timer-interval 3)
 
+(defcustom mew-syntax-treat-filename-function nil
+  "*A function to modify file names specified by Content-Disposition:"
+  :group 'mew-summary
+  :type 'symbol)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Message mode
@@ -1114,9 +1165,9 @@ to a region."
   :type 'boolean)
 
 (defcustom mew-insert-final-newline t
-  "*If *non-nil*, and if text/plain doesn't have final newline,
+  "*If *non-nil*, and if text/plain does not have final newline,
 insert a newline at the end of buffer. Due to Emacs 19 and XEmacs bug,
-mew-break-pages can't work without this."
+mew-break-pages cannot work without this."
   :group 'mew-message
   :type 'boolean)
 
@@ -1127,11 +1178,6 @@ hidden before visible fields (and after invisible fields)."
   :group 'mew-message
   :type 'boolean)
 
-(defcustom mew-summary-display-raw-header nil
-  "*If *non-nil*, '\\<mew-summary-mode-map>\\[mew-summary-display-asis]' displays a header as is."
-  :group 'mew-message
-  :type 'boolean)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Draft mode
@@ -1248,7 +1294,7 @@ The value of the second field becomes the second argment for mew-cite-format.
 ....
 If this is nil, label is not generated."
   :group 'mew-cite
-  :type '(list string))
+  :type '(repeat string))
 
 (defcustom mew-cite-format "From: %s\nSubject: %s\nDate: %s\n\n"
   "*Format for the citation label."
@@ -1359,12 +1405,12 @@ as the final part."
 ;;;
 
 (defcustom mew-noreplyto-to-list `(,mew-from:)
-  "*To: field list picked at mew-summary-reply if Reply-To: doesn't exist."
+  "*To: field list picked at mew-summary-reply if Reply-To: does not exist."
   :group 'mew-reply
   :type '(repeat string))
 
 (defcustom mew-noreplyto-cc-list `(,mew-to: ,mew-cc: ,mew-apparently-to:)
-  "Cc: field list picked at mew-summary-reply if Reply-To: doesn't exist."
+  "Cc: field list picked at mew-summary-reply if Reply-To: does not exist."
   :group 'mew-reply
   :type '(repeat string))
 
@@ -1419,6 +1465,12 @@ as the final part."
   :group 'mew-draft
   :type 'boolean)
 
+(defcustom mew-attach-move-next-after-copy nil
+  "If non-nil, the cursor moves the next position after copying/linking
+in Attachments."
+  :group 'mew-draft
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Completion
@@ -1434,23 +1486,26 @@ as the final part."
   :type '(repeat string))
   
 (defcustom mew-complete-case-ignore-case nil
-  "*Non-nil means don't consider case significant in case completion."
+  "*Non-nil means Mew does not consider case is significant in case
+completion."
   :group 'mew-complete
   :type 'boolean)
 
 (defcustom mew-complete-folder-ignore-case nil
-  "*Non-nil means don't consider case significant in folder completion."
+  "*Non-nil means Mew does not consider case is significant in folder
+completion."
   :group 'mew-complete
   :type 'boolean)
 
 (defcustom mew-complete-address-ignore-case t
-  "*Non-nil means don't consider case significant in address completion."
+  "*Non-nil means Mew does not consider case is significant in address
+completion."
   :group 'mew-complete
   :type 'boolean)
 
 (defcustom mew-circular-complete-domain-ignore-case t
-  "*Non-nil means don't consider case significant in circular domain 
-completion."
+  "*Non-nil means Mew does not consider case is significant in
+circular domain completion."
   :group 'mew-complete
   :type 'boolean)
 
@@ -1573,14 +1628,6 @@ e.g. '+from/kazu'"
   :group 'mew-folder
   :type 'string)
 
-(defcustom mew-folders-ignore
-  `(,mew-folders-default-folder ,mew-draft-folder ,mew-queue-folder "=")
-  "*Folder list to contain messages whose To: is you and From: is
-personal, e.g. a friend. Directories under this folder is ignored
-for refile guess."
-  :group 'mew-folder
-  :type '(repeat string))
-
 (defcustom mew-touch-folder-p nil
   "*If non-nil, 'mew-summary-touch-file' is used to store
 time-stamp of a directory"
@@ -1658,7 +1705,7 @@ The format is like this:
     mew-refile-guess-by-default))
 
 (defcustom mew-refile-auto-refile-skip-any-mark nil
-  "*If *non-nil*, 'mew-summary-auto-refile' doesn't touch
+  "*If *non-nil*, 'mew-summary-auto-refile' does not touch
 any alredy marked message."
   :group 'mew-refile
   :type 'boolean)
@@ -1960,7 +2007,7 @@ will be automatically leaned as alias."
   :type 'regexp)
 
 (defcustom mew-addrbook-append-domain-p t
-  "If non-nil, addresses, which don't have domain part in a header,
+  "If non-nil, addresses, which do not have domain part in a header,
 will be appended (mew-mail-domain) when composing."
   :group 'mew-addrbook
   :type 'boolean)
@@ -2081,7 +2128,7 @@ the password is used."
   :type 'boolean)
 
 (defcustom mew-encrypt-to-myself t
-  "*if non-ni, any message/part to be encrypted is encrypted with
+  "*if non-nil, any message/part to be encrypted is encrypted with
 your public key as well as receivers' one."
   :group 'mew-privacy
   :type 'boolean)
@@ -2138,10 +2185,10 @@ your public key as well as receivers' one."
 (defvar mew-prog-shell-arg   "-c")
 
 (defvar mew-prog-grep "grep")
-(defvar mew-prog-grep-opts '("-e"))
+(defvar mew-prog-grep-opts '("-l" "-e"))
 
 (defvar mew-prog-vgrep "grep")
-(defvar mew-prog-vgrep-opts '("-e"))
+(defvar mew-prog-vgrep-opts '("-l" "-e"))
 
 (defvar mew-prog-grep-max-msgs 10000)
 
@@ -2247,6 +2294,12 @@ Content-Disposition:."
 
 (defvar mew-no-warning-params ()) ;; downcased
 
+(defvar mew-warning-field-level 1
+  "*If warning level of a field is greater than or equal to this value,
+the warning is displayed on X-Mew:.
+Level 1: syntax is correct, but not recommenced.
+Level 2: syntax error.")
+
 (provide 'mew-vars)
 
 ;;; Copyright Notice:
