@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Feb  1, 1999
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -25,7 +24,8 @@
 (defvar mew-mark-unlink ?X)
 (defvar mew-mark-refile ?o)
 (defvar mew-mark-tmp    ?\0) ;; temporary use only.
-(defvar mew-mark-collect      mew-mark-review)
+(defvar mew-mark-clean
+  `(,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile))
 (defvar mew-mark-default-walk mew-mark-review)
 (defvar mew-mark-walk         mew-mark-default-walk)
 (defvar mew-mark-show-list (list mew-mark-review))
@@ -52,7 +52,7 @@
 
 (defvar mew-subject-simplify-replace-alist
   (list
-   ;; (REGEXP . NEW)   ... NEW may be nil for deletion (same to "")
+   ;; (REGEXP . NEW) --- NEW may be nil for deletion (same to "")
    ;; replace multiple Re: and Fw: into single Re:
    (cons (concat "^" ;; regexp
 		 mew-reply-regex
@@ -150,7 +150,7 @@ for privacy reasons.")
 (defvar mew-page-delimiter "^\^L")
 (defvar mew-keyval "^\\([^ \t:]+:?\\)[ \t]*")
 ;; "^\\([^ \t:]+:?\\)[ \t]*\\(.*\\)$" faces a stupid error. 
-;; "$" doesn't mean end-of-line if the second match is empty, sigh.
+;; "$" does not mean end-of-line if the second match is empty, sigh.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -198,39 +198,14 @@ for privacy reasons.")
   (string-match "^Message/" (capitalize ct)))
 
 (defsubst mew-ct-linebasep (ct)
-  (mew-member-case-equal ct mew-mime-content-type-text-list))
-
-(defvar mew-mime-content-type-text-list
-  '("Text/Plain" "Text/Html" "Text/Css" "Text/Xml" "Text/Sgml"
-    "Application/Postscript"
-    "Message/Rfc822")
-  "Content-Type: list for text which are treated as text when encoded.")
+  (or (mew-ct-textp ct)
+      (mew-case-equal ct mew-ct-aps)
+      (mew-case-equal ct mew-ct-msg))) ;; xxx mew-ct-messagep?
 
 (defvar mew-mime-content-type-multipart-list
-  '("Multipart/Mixed" "Multipart/Alternative")
-  ;;"Multipart/Digest" "Multipart/Parallel"
-  "Candidate of 'Content-Type: Multipart/' when CT: is 
-changed in draft buffer.")
-
-(defvar mew-mime-content-type-binary-list
-  '("Application/Octet-Stream"
-    "Application/Pdf"
-    "Application/Msword" "Application/Vnd.Ms-Excel"
-    "Application/Vnd.Ms-Powerpoint"
-    "Image/Jpeg" "Image/Gif" "Image/Tiff"
-    "Image/x-xwd" "Image/x-xbm" "Image/x-xpm" 
-    "Image/x-bmp" "Image/Png" "Audio/Basic" "Video/Mpeg" "Video/Quicktime")
-  "Content-Type: list to save as binary file in mew-summary-save on Mule.")
-
-(defvar mew-mime-content-type-others-list
-  '("Message/Rfc822" "Message/External-body"))
-
-(defvar mew-mime-content-type-list
-  (append mew-mime-content-type-text-list
-	  mew-mime-content-type-multipart-list
-	  mew-mime-content-type-binary-list
-	  mew-mime-content-type-others-list)
-  "Candidate of Content-Type: when CT: is changed in draft buffer.")
+  `(,mew-ct-mlm ,mew-ct-mla)
+  "Candidate of 'Content-Type: Multipart/' when CT: is changed
+in draft buffer.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -299,7 +274,13 @@ changed in draft buffer.")
 
 (defvar mew-mime-content-type
   '(("multipart/"  nil         nil     nil            mew-icon-multipart)
-    ("audio/basic" "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
+    ;;
+    ("audio/basic"  "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
+    ("audio/x-wav"  "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
+    ("audio/x-aiff" "\\.aif?f$" mew-b64 mew-prog-audio mew-icon-audio)
+    ("audio/x-midi" "\\.midi?$" mew-b64 mew-prog-audio mew-icon-audio)
+    ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$" mew-b64 mew-prog-audio mew-icon-audio)
+    ;;
     ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/tiff"  "\\.tiff$"  mew-b64 mew-prog-image mew-icon-image)
     ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image)
@@ -308,19 +289,27 @@ changed in draft buffer.")
     ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-xpm" "\\.xpm$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image)
+    ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image)
+    ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/"      "^$"        mew-b64 mew-prog-image mew-icon-image)
-    ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-mpeg  mew-icon-video)
-    ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-mpeg  mew-icon-video)
+    ;;
+    ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-video mew-icon-video)
+    ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-video mew-icon-video)
+    ("video/x-msvideo" "\\.avi$"   mew-b64 mew-prog-video mew-icon-video)
+    ;;
     ("message/rfc822"          "^[0-9]+$" nil
      mew-prog-rfc822           mew-icon-message/rfc822)
     ("message/external-body"   "\\.ext$"  nil
      mew-prog-external-body    mew-icon-message/external-body)
     ("message/delivery-status" "^$"       nil
      mew-prog-delivery-status  mew-icon-text)
+    ;;
     ("application/postscript"        "\\.e?ps$" mew-qp
      mew-prog-postscript             mew-icon-application/postscript)
     ("application/pdf"               "\\.pdf$"  mew-b64
      mew-prog-pdf                    mew-icon-application/postscript)
+    ("application/xml"               "\\.xml$"  mew-b64
+     mew-prog-xml2                   mew-icon-text)
     ("application/msword"            "\\.doc$"  mew-b64
      mew-prog-msword                 mew-icon-text)
     ("application/vnd.ms-excel"      "\\.xls$"  mew-b64
@@ -332,16 +321,21 @@ changed in draft buffer.")
     ("application/x-pkcs7-signature" "\\.p7s$"  mew-b64
      nil			     mew-icon-unknown)
     ("application/octet-stream"
-     "\\.tar$\\|\\.tar\\.gz$\\|\\.tgz$\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tar\\.bz2?$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$|\\.gpg$"
+     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
       mew-b64 mew-prog-octet-stream mew-icon-application/octet-stream)
-    ("text/html"     "\\.html$" nil     mew-prog-html      mew-icon-text)
-    ("text/enriched" "\\.rtf$"  nil     mew-prog-enriched  mew-icon-text)
-    ("text/css"      "\\.css$"  nil     mew-prog-text      mew-icon-text)
-    ("text/xml"      "\\.xml$"  nil     mew-prog-text      mew-icon-text)
-    ("text/sgml"     "\\.sgml$" nil     mew-prog-text      mew-icon-text)
-    ("text/plain"    "\\.txt$"  nil     mew-prog-plain     mew-icon-text)
-    ("text/"         "^$"       nil     mew-prog-text      mew-icon-text)
-    (t               "^$"       nil     mew-prog-octet-stream mew-icon-unknown)
+    ;;
+    ("text/html"     "\\.html?$" nil     mew-prog-html      mew-icon-text)
+    ("text/enriched" "\\.rtf$"   nil     mew-prog-enriched  mew-icon-text)
+    ("text/css"      "\\.css$"   nil     mew-prog-text      mew-icon-text)
+    ("text/sgml"     "\\.sgml$"  nil     mew-prog-text      mew-icon-text)
+    ("text/plain"    "\\.txt$\\|\\.c$\\|\\.h$\\|\\.el$\\|\\.diff$\\|\\.patch$"
+                                 nil     mew-prog-plain     mew-icon-text)
+    ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
+    ("text/"         "^$"        nil     mew-prog-text      mew-icon-text)
+    ;; Unkown CT: matches here.
+    (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
+    ;; Unknown suffix matches here and return the entry specified 
+    ;; by mew-content-type.
     (nil             ".*"))
   "(content-type filename <encoding> <prog> <icon>)")
 
@@ -349,11 +343,7 @@ changed in draft buffer.")
   (mew-assoc-match2 ct mew-mime-content-type 0))
 
 (defsubst mew-ctdb-by-file (file)
-  (let ((ctdb (mew-assoc-match2 file mew-mime-content-type 1))
-	(case (mew-tinfo-get-case)))
-    (if (null (mew-ctdb-ct ctdb))
-	(mew-ctdb-by-ct (mew-content-type case))
-      ctdb)))
+  (mew-assoc-match2 file mew-mime-content-type 1))
 
 (defsubst mew-ctdb-ct (attr)
   (and (nth 0 attr) (mew-capitalize (nth 0 attr))))
@@ -617,7 +607,7 @@ changed in draft buffer.")
     ("^\\(To\\|Apparently-To\\):$" t
      mew-face-header-important
      mew-face-header-to)
-    ("^Cc:$" t
+    ("^\\(Cc\\|Bcc\\):$" t
      mew-face-header-important
      mew-face-header-to)
     ("^Newsgroups:$" t
@@ -735,7 +725,7 @@ present, mew-face-header-marginal is used."
     4th and 5th is the case that the old mark has state.
         4th means that the old mark is overrode.
         5th means that the old mark remains.
-    6th is the case that the old mark doesn't have state.
+    6th is the case that the old mark does not have state.
 7th is the case where level of the new mark is smaller than that of the 
     old one.
 
