@@ -2,7 +2,6 @@
 
 ;; Author:  Shuichi Kitaguchi <kit@Mew.org>
 ;; Created: Dec 05, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -16,10 +15,10 @@
 ;;
 
 ;;; for PRINTING
-;;    mew-w32-prog-print     ...  print command
-;;    mew-w32-prog-print-arg ...  print command argument
-;;    mew-w32-cs-print       ...  coding-system for printing
-;;    define-process-argument-editing  ...  for argument-editing
+;;    mew-w32-prog-print     --- print command
+;;    mew-w32-prog-print-arg ---  print command argument
+;;    mew-w32-cs-print       ---  coding-system for printing
+;;    define-process-argument-editing  ---  for argument-editing
 ;;
 ;;; example
 ;;   (setq mew-w32-prog-print     "notepad.exe")
@@ -104,33 +103,46 @@
 
 
 ;; MIME setting
+
 (defvar mew-prog-plain    '(mew-mime-text/plain mew-mime-text/plain-ext))
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
+(defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-image           '(mew-mime-image/* mew-mime-image/*-ext))
-(defvar mew-prog-mpeg            `(,mew-w32-exec           () t))
+(defvar mew-prog-video           `(,mew-w32-exec           () t))
 (defvar mew-prog-rfc822          'mew-mime-message/rfc822)
 (defvar mew-prog-external-body   '(mew-mime-external-body mew-mime-external-body-ext))
 (defvar mew-prog-delivery-status 'mew-mime-text/plain)
 (defvar mew-prog-postscript      `(,mew-w32-exec           () t))
 (defvar mew-prog-pgp-keys        '(mew-mime-pgp-keys mew-mime-pgp-keys-ext))
 (defvar mew-prog-pdf             `(,mew-w32-exec           () t))
+(defvar mew-prog-xml2            '(mew-mime-application/xml
+				   mew-mime-application/xml-ext))
 (defvar mew-prog-octet-stream    `(,mew-w32-exec           () t))
 (defvar mew-prog-msword          `(,mew-w32-exec           () t))
 (defvar mew-prog-msexcel         `(,mew-w32-exec           () t))
 (defvar mew-prog-mspowerpoint    `(,mew-w32-exec           () t))
 
-(defvar mew-prog-w32             `(,mew-w32-exec           () t))
+;;;
+;;; Text/Html, Application/Xml, Image
+;;;
 
-;;;
-;;; Text/Html, Image
-;;;
+(defvar mew-format-html "%s.htm")
+(defvar mew-format-xml  "%s.xml")
 
 (defvar mew-prog-text/html         'mew-mime-text/html-w3m) ;; See w3m.el
 (defvar mew-prog-text/html-ext     mew-w32-exec)
 (defvar mew-prog-text/html-ext-arg nil)
 
+(defvar mew-prog-text/xml         'mew-mime-text/html-w3m) ;; See w3m.el
+(defvar mew-prog-text/xml-ext     mew-w32-exec)
+(defvar mew-prog-text/xml-ext-arg nil)
+
+(defvar mew-prog-application/xml         nil)
+(defvar mew-prog-application/xml-ext     mew-w32-exec)
+(defvar mew-prog-application/xml-ext-arg nil)
+
 (defvar mew-prog-image/*         'mew-mime-image/*)
 (defvar mew-prog-image/*-ext     mew-w32-exec)
 (defvar mew-prog-image/*-ext-arg nil)
@@ -139,65 +151,6 @@
 (defvar mew-prog-application/msexcel nil)
 (defvar mew-prog-application/mspowerpoint nil)
 
-(setq mew-mime-content-type
-      (append
-       '(
-	 ;; Audio (mew-prog-audio)
-	 ("audio/x-wav"     "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
-	 ("audio/x-aiff"    "\\.aif?f$" mew-b64 mew-prog-audio mew-icon-audio)
-	 ("audio/x-midi"    "\\.midi?$" mew-b64 mew-prog-audio mew-icon-audio)
-	 ;; Image (mew-prog-image)
-	 ("image/x-pcx"     "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image)
-	 ("image/x-tga"     "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
-	 ;; Video (mew-prog-mpeg)
-	 ("video/x-msvideo" "\\.avi$"   mew-b64 mew-prog-mpeg  mew-icon-video)
-	 ;; Executable (mew-prog-w32)
-	 ("application/octet-stream" "\\.exe$\\|\\.com$\\|\\.cmd$\\|\\.bat$\\|\\.dll$"
-	  mew-b64 mew-prog-w32 mew-icon-application/octet-stream)
-	 ;; Archive (mew-prog-w32)
-	 ("application/octet-stream" "\\.arj$\\|\\.zoo$\\|\\.rar$|\\.cab$"
-	  mew-b64 mew-prog-w32 mew-icon-application/octet-stream)
-	 ;; Document (mew-prog-w32)
-	 ("application/x-dvi" "\\.dvi$"
-	  mew-b64 mew-prog-w32 mew-icon-application/postscript)
-	 ;; OASYS (mew-prog-w32)
-	 ("application/vnd.fujitsu.oasys" "\\.oas$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ("application/vnd.fujitsu.oasys2" "\\.oa2$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ;; Ichitaro V.2,3,4,5,6,7,8,9,10,Lite,Lite2 (mew-prog-w32)
-	 ("application/octet-stream" "\\.jtd$\\|\\.jfw$\\|\\.jbw$\\|\\.jaw$\\|\\.jsw$\\|\\.jxw$\\|\\.jtt$\\|\\.jvw$\\|\\.juw$\\|\\.jtw$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ;; Shockwave (mew-prog-w32)
-	 ("application/x-shockwave-flash" "\\.swf$"
-	  mew-b64 mew-prog-w32 mew-icon-image)
-	 ;; MPEG audio (mew-prog-w32)
-	 ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$"
-	  mew-b64 mew-prog-w32 mew-icon-image)
-	 )
-       mew-mime-content-type))
-
-(setq mew-mime-content-type-list
-      (append
-       '(
-	 ;; Image
-	 "image/x-pcx" "image/x-pic"
-	 ;; Audio
-	 "audio/x-wav" "audio/x-aiff" "audio/x-midi"
-	 ;; Video
-	 "video/x-msvideo"
-	 ;; Document
-	 "application/x-dvi"
-	 ;; OASYS
-	 "application/vnd.fujitsu.oasys"
-	 "application/vnd.fujitsu.oasys2"
-	 ;; Shockwave
-	 "application/x-shockwave-flash"
-	 ;; MPEG audio
-	 "audio/x-mpeg" "audio/mpeg"
-	 )
-       mew-mime-content-type-list))
-
 (cond
  ((eq mew-mule-ver 2)
   (setq mew-cs-database-for-arg
