@@ -2,7 +2,6 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
@@ -101,14 +100,14 @@ If event is button 3, show a menu."
   (interactive)
   (if (null nums) 
       (message "No message")
-    (let ((buf (current-buffer)))
+    (let ((win (selected-window)))
       (mew-summary-toggle-disp-msg 'on)
       (unwind-protect
 	  (progn
 	    (mew-window-configure 'message)
 	    ;; message buffer
 	    (mew-summary-display-part cache nums))
-	(mew-pop-to-buffer buf)))))
+	(select-window win)))))
 
 (defun mew-draft-button ()
   "Call back function for toolbar of Draft mode. 
@@ -169,13 +168,13 @@ If event is button 3, show a menu."
 		 glyph) 
 	    (if format
 		(progn
-		  (message "Loading image ... ")
+		  (message "Loading image...")
 		  (setq glyph (make-glyph (vector format :file file)))
 		  (if (eq format 'xbm)
 		      (set-glyph-property glyph 'face 'x-face))
 		  (set-extent-begin-glyph
 		   (mew-overlay-make (point-max) (point-max)) glyph)
-		  (message "Loading image ... done"))
+		  (message "Loading image...done"))
 	      (if (and (stringp program) (mew-which-exec program))
 		  (if async
 		      (mew-mime-start-process program options file)
@@ -285,10 +284,11 @@ popup menu can be implemented."
 ;;;
 
 (defun mew-image-inline-p (format)
+  ;; XEmacs checks window-system by itself
   (valid-image-instantiator-format-p format))
   
 (defun mew-mime-image (cache begin end format)
-  (message "Loading image ... ")
+  (message "Loading image...")
   (cond 
    ((eq format 'xbm)
     ;; XEmacs depends on libX when it converts XBM to bitmap.
@@ -310,6 +310,16 @@ popup menu can be implemented."
 	  (mew-overlay-make (point-min) (point-min)) glyph)
 	 (if (file-exists-p tfile)
 	     (delete-file tfile))))))
+   ((eq format 'gif)
+    (set-buffer (mew-buffer-message))
+    (let ((glyph (mew-xemacs-make-gif-glyph cache begin end)))
+      (mew-elet
+       (if glyph
+	   (set-extent-end-glyph
+	    (mew-overlay-make (point-min) (point-min)) glyph)
+	 (insert "Cannot display this GIF image.\n")
+	 (insert "Please install \"gifsicle\".\n")
+	 (insert "\thttp://www.lcdf.org/gifsicle/\n")))))
    (t
     (set-buffer (mew-buffer-message))
     (mew-elet
@@ -321,7 +331,86 @@ popup menu can be implemented."
 					  (set-buffer cache)
 					  (mew-buffer-substring 
 					   begin end))))))))
-  (message "Loading image ... done"))
+  (message "Loading image...done"))
+
+;; Fix Animation GIF and Interlaced GIF
+;; This code is stolen form w3m-xmas.el.
+(defun mew-xemacs-make-gif-glyph (cache begin end)
+  (let ((prog (mew-which-exec "gifsicle"))
+	(image (save-excursion
+		 (set-buffer cache)
+		 (mew-buffer-substring begin end)))
+	should-be-non-interlaced should-unoptimize size1 size2 tmpbuf)
+    (if (null prog)
+	nil
+      (setq tmpbuf (generate-new-buffer mew-buffer-prefix))
+      (save-excursion
+	(set-buffer tmpbuf)
+	(mew-erase-buffer)
+	(mew-set-buffer-multibyte nil)
+	(mew-plet
+	 (insert image)
+	 (goto-char (point-min))
+	 (when (looking-at "GIF8[79]")
+	   (call-process-region (point-min) (point-max)
+				prog
+				t t nil "--info")
+	   (goto-char (point-min))
+	   ;; Check whether a `data' is interlaced.
+	   (setq should-be-non-interlaced
+		 (re-search-forward
+		  "  \\+ image #[0-9]+ \\([0-9]+x[0-9]+\\).* interlaced"
+		  nil t))
+	   (goto-char (point-min))
+	   ;; Check whether a `data' is optimized or larger than
+	   ;; the value of `w3m-animated-gif-maximum-size'.
+	   (when (looking-at ".+ \\([0-9]+\\) images\r?$")
+	     (setq size1 (string-to-number (match-string 1)))
+	     (forward-line 1)
+	     (unless (and (looking-at ".+ \\([0-9]+\\)x\\([0-9]+\\)\r?$")
+			  (natnump (setq size1 (* size1
+						  (string-to-number
+						   (match-string 1)))))
+			  (natnump (setq size1 (* size1
+						  (string-to-number
+						   (match-string 2))))))
+	       ;; It should be truncated to be only one frame.
+	       (setq should-unoptimize "#0"))
+	     (setq size1 nil)
+	     (while (and (not should-unoptimize)
+			 (re-search-forward
+			  "  \\+ image #[0-9]+ \\([0-9]+x[0-9]+\\)"
+			  nil t))
+	       (if size1
+		   (if (string-equal size1 (setq size2 (match-string 1)))
+		       (setq size1 size2)
+		     (setq should-unoptimize "--unoptimize"))
+		 (setq size1 (match-string 1)))))
+	   (erase-buffer)
+	   (insert image)
+	   (when (or should-unoptimize should-be-non-interlaced))
+	   ;; Unoptimize anyway.
+	   (if should-unoptimize
+	       (call-process-region (point-min) (point-max)
+				    prog
+				    t t nil should-unoptimize
+				    "--no-interlace")
+	     (call-process-region (point-min) (point-max)
+				  prog
+				  t t nil "--no-interlace"))
+	   (goto-char (point-min))
+	   (unless (looking-at "GIF8[79]")
+	     ;; Unoptimization is failed. :-<
+	     ;; Attempt to extract the first frame.
+	     (erase-buffer)
+	     (insert data)
+	     (call-process-region (point-min) (point-max)
+				  prog
+				  t t nil "#0" "--no-interlace")))
+	 ;; buffer is fixed gif
+	 (setq image (mew-buffer-substring (point-min) (point-max))))
+	(kill-buffer tmpbuf))
+      (make-glyph (vector 'gif :data image)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
