@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Jul 26, 2001
+;; Revised: Nov  1, 2001
 
 ;;; Commentary:
 
@@ -45,7 +45,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "2.0"
+(defconst mew-version-number "2.1"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -79,7 +79,7 @@
 ;;;
 
 (defvar mew-debug nil
-  "'decode, 'encode, 'net, 'pgp, 'thread, t for all.")
+  "'decode, 'encode, 'net, 'pgp, 'thread, 'sort, 'pack, t for all.")
 (defsubst mew-debug (category)
   (or (eq mew-debug t) (eq mew-debug category)))
 
@@ -154,19 +154,19 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 
 (defun mew-init ()
   (let ((buf (generate-new-buffer mew-buffer-prefix)))
-    (switch-to-buffer buf)
-    (mew-window-configure '(1 0))
-    (mew-hello)
-    (message "Setting Mew world ... ")
     (run-hooks 'mew-env-hook)
-    (mew-set-environment)
     (load mew-rc-file 'no-err)
+    (switch-to-buffer buf)
+    (mew-window-configure '(1 0)) ;; Using mew-mode-name-message
+    (mew-hello)
+    (message "Setting up Mew world...")
+    (mew-set-environment)
     (run-hooks 'mew-init-hook)
     (mew-status-update t)
     (mew-passwd-setup)
     (mew-highlight-timer-setup)
     (setq mew-init-p t)
-    (message "Setting Mew world ... done")
+    (message "Setting up Mew world...done")
     (mew-remove-buffer buf)))
 
 (defun mew-set-environment (&optional no-dir)
@@ -222,16 +222,18 @@ set mew-touch-folder-p to t"
 information of folders is also updated in addition to that of
 Addrbook. The default value is 't'."
   (interactive "P")
-  (message "Updating status ... ")
+  (message "Updating status...")
   (if (interactive-p) (mew-set-environment 'no-dir))
+  ;; These two must be before mew-folder-setup
   (mew-config-setup)
+  (mew-regex-setup)
   (if arg (mew-folder-setup (interactive-p)))
   (mew-folder-init mew-basic-folders)
   (mew-folder-init mew-inbox-folders)
   (mew-folder-init mew-queue-folders)
+  (mew-folder-init mew-mdrop-folders)
   (mew-touch-folder-check)
   (mew-refile-setup)
-  (mew-regex-setup)
   (mew-addrbook-setup)
   (mew-scan-setup)
   (mew-pgp-setup)
@@ -240,11 +242,20 @@ Addrbook. The default value is 't'."
   (mew-thread-setup)
   (mew-theme-setup)
   (mew-pop-biff-setup)
-  (mew-uniq-variables)
-  (message "Updating status ... done"))
+  (mew-ct-setup)
+  (message "Updating status...done"))
 
-(defun mew-uniq-variables ()
-  (setq mew-mime-content-type-list (mew-uniq-list mew-mime-content-type-list)))
+(defvar mew-mime-content-type-list nil
+  "Candidate of Content-Type: when CT: is changed in draft buffer.")
+
+(defun mew-ct-setup ()
+  (let ((cts mew-mime-content-type) ct)
+    (while cts
+      (setq ct (car (car cts)))
+      (setq cts (cdr cts))
+      (if (and (stringp ct) (not (string-match "/$" ct)))
+	  (setq mew-mime-content-type-list
+		(cons (capitalize ct) mew-mime-content-type-list))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -347,7 +358,7 @@ according to 'mew-folder<'. Binary search is used for speed reasons."
 
 (defun mew-folder-delete (folder)
   ;; used to delete +virtual only. 
-  ;; So, this doesn't take care of +foo/
+  ;; So, this does not take care of +foo/
   (setq mew-folder-list (delete folder mew-folder-list))
   (setq mew-folder-alist
 	;; delq is right. no need to use delete.
@@ -374,7 +385,7 @@ messages whose name is numeric are not gathered. This makes it faster
 to check wether or not each candidate is a directory in 'while' loop.
 
 The other is to see if the link count of a directory is 2. If so, the
-directory doesn't have subdirectories. So, it is not necessary to
+directory does not have subdirectories. So, it is not necessary to
 trace down. This technique can be used on UNIX variants."
   (let ((default-directory (expand-file-name dir default-directory))
 	file dirent dirs ent subdirs)
@@ -453,21 +464,12 @@ to check wether or not each candidate is a directory in 'while' loop."
 	 (subdir (file-name-nondirectory dir)))
 	 ;; foo/bar -> bar 
 	 ;; foo -> foo
-    (if (mew-folders-ignore-p folder)
+    (if (or (mew-folder-virtualp folder)
+	    (mew-folder-local-newsp folder)
+	    (string-match mew-regex-ignore-folders folder))
 	(list folder nil)
       (list folder subdir))))
 
-(defun mew-folders-ignore-p (folder)
-  (let ((ignores mew-folders-ignore))
-    (if (mew-folder-virtualp folder)
-	t
-      (catch 'ignore
-	;; while always returns nil
-	(while ignores
-	  (if (string-match (mew-folder-regex (car ignores)) folder)
-	      (throw 'ignore t))
-	  (setq ignores (cdr ignores)))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Current status
@@ -549,7 +551,7 @@ a list of two numbers.
 Two numbers means the ratio of the upper window and 
 the lower window (i.e. the window of Message buffer).
 
-If Message buffer doesn't exist, it will be created. If the height of
+If Message buffer does not exist, it will be created. If the height of
 the lower window is not zero, switch to the buffer."
   (let* ((windows
 	  (if (listp action) 
@@ -676,6 +678,7 @@ It is typically called by kill-emacs-hook."
 (defvar mew-regex-attach-beg nil)
 (defvar mew-regex-attach-end nil)
 (defvar mew-regex-my-address-list nil)
+(defvar mew-regex-ignore-folders nil)
 
 (defsubst mew-mark-regex (mark)
   (concat mew-regex-msg (regexp-quote (char-to-string mark))))
@@ -700,7 +703,20 @@ It is typically called by kill-emacs-hook."
 	(concat "^" mew-draft-attach-boundary-beg "$"))
   (setq mew-regex-attach-end
 	(concat "^" mew-draft-attach-boundary-end "$"))
-  (setq mew-regex-my-address-list (mew-get-my-address-regex-list)))
+  (setq mew-regex-my-address-list (mew-get-my-address-regex-list))
+  (setq mew-regex-ignore-folders
+	(mapconcat
+	 (function mew-folder-regex)
+	 (mew-uniq-list
+	  (append
+	   mew-basic-folders
+	   mew-inbox-folders
+	   mew-queue-folders
+	   mew-mdrop-folders
+	   (list
+	    mew-folders-default-folder
+	    mew-attach-folder)))
+	 "\\|")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
